
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#include "rc4.h"

RC4StreamCipher::RC4StreamCipher(unsigned char* key, long keylen)
{
    unsigned char ka[256], t;
    unsigned long i, j;
    
    klen = keylen;
    for (i = 0; i < 256; i++)
    {
        sbox[i] = (unsigned char)i;
        ka[i] = key[i % keylen];
    }
    for(i = j = 0; i < 256; i++)
    {
        j = (j + sbox[i] + ka[i]) & 0xff;
        t = sbox[i];
        sbox[i] = sbox[j];
        sbox[j] = t;
    }
    i_ = j_ = 0;
}

unsigned char RC4StreamCipher::Next()
{
    unsigned char t;
    
    i_ = (i_ + 1) & 0xff;
    j_ = (j_ + sbox[i_]) & 0xff;
    t = sbox[i_];
    sbox[i_] = sbox[j_];
    sbox[j_] = t;
    return sbox[(sbox[i_] + sbox[j_]) & 0xff];
}

void RC4StreamCipher::Encrypt(unsigned char* in, unsigned char* out, long len)
{
    while (len > 0)
    {
        *out++ = (unsigned char)(*in++ ^ Next());
        len--;
    }
}

void RC4StreamCipher::Decrypt(unsigned char* in, unsigned char* out, long len)
{
    Encrypt(in, out, len);
}

