
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#ifndef _SIMPLE_XML_C68FF25D_571F_4A84_84EF_2E150EED81D3_
#define _SIMPLE_XML_C68FF25D_571F_4A84_84EF_2E150EED81D3_

#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

class SimpleString
{
public:
    SimpleString()
    {
        m_pData = 0;
    }
    SimpleString(SimpleString& str)
    {
        m_pData = 0;
        LoadData(str.m_pData);
    }
    SimpleString(const char* data, int len = -1)
    {
        m_pData = 0;
        LoadData(data, len);
    }
    ~SimpleString()
    {
        if (m_pData)
            free(m_pData);
    }

	void Format(const char* szFormat, ...)
	{
		char* buffer = (char*)malloc(102400);
		va_list argList;
		va_start(argList, szFormat);
		vsprintf(buffer, szFormat, argList);
		va_end(argList);
		LoadData(buffer);
		free(buffer);
	}
	void Append(const char* szFormat, ...)
	{
		char* buffer = (char*)malloc(102400);
		va_list argList;
		va_start(argList, szFormat);
		vsprintf(buffer, szFormat, argList);
		va_end(argList);
		AppendData(buffer);
		free(buffer);
	}

    void LoadData(const char* data, int len = -1)
    {
        if (m_pData)
        {
            free(m_pData);
            m_pData = 0;
        }
        if (data)
        {
            if (len < 0)
            {
                len = strlen(data);
            }
            m_pData = (char*)malloc((len + 1) * sizeof(char));
            memset(m_pData, 0, (len + 1) * sizeof(char));
            if (len > 0)
            {
                memcpy(m_pData, data, len * sizeof(char));
            }
        }
    }
	void AppendData(const char* data, int len = -1)
	{
        if (len < 0)
        {
            len = strlen(data);
        }
		int oldlen = 0;
		if (m_pData)
		{
			oldlen = strlen(m_pData); 
		}
		char* p = (char*)malloc((oldlen + len + 1) * sizeof(char));
		memset(p, 0, (oldlen + len + 1) * sizeof(char));
		if (oldlen)
			memcpy(p, m_pData, oldlen * sizeof(char));
		memcpy(p + oldlen, data, len * sizeof(char));
		free(m_pData);
		m_pData = p;
	}
    char* GetData()
    {
        if (!m_pData)
        {
	        LoadData("");
        }
        return m_pData;
    }
    char* GetBuffer(int len)
    {
        if (m_pData)
        {
            free(m_pData);
            m_pData = 0;
        }
        m_pData = (char*)malloc((len + 1) * sizeof(char));
        memset(m_pData, 0, (len + 1) * sizeof(char));
        return m_pData;
    }

    bool FindItem(const char *item, char *buffer)
    {
        buffer[0] = 0;
        if (!m_pData) return 0;
        char* p = strstr(m_pData, item);
        if (p)
        {
            p += strlen(item);
            char* q = strstr(p, "\n");
            if (q)
            {
                memcpy(buffer, p, q - p);
                if (buffer[q - p - 1] == 0x0D)
                {
    	            buffer[q - p - 1] = 0;
                }
                else
                {
                	buffer[q - p] = 0;
                }
            }
            else
            {
                strcpy(buffer, p);
            }
            return 1;
        }
        return 0;
    }

    operator char*()
    {
        return GetData();
    }
    SimpleString& operator =(const char* data)
    {
        LoadData(data);
	    return *this;
    }
    SimpleString& operator =(const SimpleString& str)
    {
        LoadData(str.m_pData);
	    return *this;
    }

    int Length()
    {
        return m_pData ? strlen(m_pData) : 0;
    }

private:
    char* m_pData;
};

class SimpleXML
{
public:
    SimpleXML()
    {
        m_pData = 0;
        m_pText = 0;
    }
    SimpleXML(SimpleXML& xml)
    {
        m_pData = 0;
        m_pText = 0;
        LoadData(xml.m_pData);
    }
    SimpleXML(const char* data, int len = -1)
    {
        m_pData = 0;
        m_pText = 0;
        LoadData(data, len);
    }
    ~SimpleXML()
    {
        if (m_pData)
            free(m_pData);
        if (m_pText)
            free(m_pText);
    }

    void LoadData(const char* data, int len = -1)
    {
        if (m_pData)
        {
            free(m_pData);
            m_pData = 0;
        }
        if (m_pText)
        {
            free(m_pText);
            m_pText = 0;
        }
        if (data)
        {
            if (len < 0)
            {
                len = strlen(data);
            }
            if (len)
            {
                m_pData = (char*)malloc((len + 1) * sizeof(char));
                memset(m_pData, 0, (len + 1) * sizeof(char));
                memcpy(m_pData, data, len * sizeof(char));
            }
        }
    }
    void Copy(SimpleXML& xml)
    {
        LoadData(xml.m_pData);
    }

    int FindTag(const char* name, SimpleXML& tag, SimpleXML& data, int start = 0)
    {
        if (m_pData)
        {
            char buf[100];
            strcpy(buf, "<");
            strcat(buf, name);
            char* p = m_pData + start;

            do
            {
                p = strstr(p, buf);
                if (!p)
                    break;
                p += strlen(buf);
                if ((*p == ' ') || (*p == '>') || (*p == '/'))
                {
                    break;
                }
            }
            while (p);

            if (p)
            {
                while (isspace(*p))
                    p++;
                char* q = strchr(p, '>');
                if (q)
                {
                    if (*(q - 1) == '/')
                    {
                        q--;
                        tag.LoadData(p, q - p);
                        data.LoadData(0);
                        return q - m_pData + 2;
                    }
                    tag.LoadData(p, q - p);
                    p = q + 1;
                    strcpy(buf, "</");
                    strcat(buf, name);
                    q = strstr(p, buf);
                    if (q)
                    {
                        char* r = strchr(q, '>');
                        if (r)
                        {
                            data.LoadData(p, q - p);
                            r++;
                            return r - m_pData;
                        }
                    }
                }
            }
        }
        return -1;
    }

    char* GetData()
    {
        if (!m_pText && m_pData)
        {
            int len = strlen(m_pData);
            m_pText = (char*)malloc((len + 1) * sizeof(char));
            memset(m_pText, 0, (len + 1) * sizeof(char));
	    int j = 0;
            for (int i = 0; i < len; i++)
            {
                if (m_pData[i] == '&')
                {
                    if (m_pData[i + 1] == '#')
                    {
                        if (m_pData[i + 2] == 'x')
                        {
                            m_pText[j++] = (char)strtoul(m_pData + i + 3, 0, 16);
                        }
                        else
                        {
                            m_pText[j++] = (char)strtoul(m_pData + i + 2, 0, 10);
                        }
                    }
                    else
                    {
                        if (memcmp(m_pData + i , "&lt;", 4 * sizeof(char)) == 0)
                        {
                            m_pText[j++] = '<';
                        }
                        else if (memcmp(m_pData + i , "&gt;", 4 * sizeof(char)) == 0)
                        {
                            m_pText[j++] = '>';
                        }
                        else if (memcmp(m_pData + i , "&amp;", 5 * sizeof(char)) == 0)
                        {
                            m_pText[j++] = '&';
                        }
                        else if (memcmp(m_pData + i , "&quot;", 6 * sizeof(char)) == 0)
                        {
                            m_pText[j++] = '\"';
                        }
                        else if (memcmp(m_pData + i , "&apos;", 6 * sizeof(char)) == 0)
                        {
                            m_pText[j++] = '\'';
                        }
                    }
                    char* p = strchr(m_pData + i, ';');
                    if (p)
                    {
                        i = p - m_pData;
                    }
                    else
                    {
                        break;
                    }
                }
                else
                {
                    m_pText[j++] = m_pData[i];
                }
            }
            m_pText[j] = 0;
        }
        return m_pText;
    }

private:
    char* m_pData, *m_pText;
};

#endif
