
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "Main.h"
#import "Swizzle.h"

//==================================================================================================
static BOOL g_hideDownloads;
//==================================================================================================
@interface NSObject (Main)
- (void)swizzledShowDownloads: (id)sender;
- (void)swizzledDownloadDidFinish: (id)sender;
- (void)swizzledDownload: (id)sender didFailWithError: (id)error;
- (void)tryHide;
@end
//==================================================================================================
@implementation NSObject (Main)
//--------------------------------------------------------------------------------------------------
- (void)swizzledShowDownloads: (id)sender
{
    if ([sender isEqual: [[Main sharedInstance] autohideDownloadsMenu]])
    {
        NSString *path = [[NSBundle bundleForClass: [Main class]] bundlePath];
        NSString *filename = [path stringByAppendingPathComponent: @"Contents/Resources/flag"];
        NSFileManager *fm = [NSFileManager defaultManager];
        
        if (NSOnState == [[[Main sharedInstance] autohideDownloadsMenu] state])
        {
            [[[Main sharedInstance] autohideDownloadsMenu] setState: NSOffState];
            [fm removeItemAtPath: filename error: nil];
        }
        else
        {
            [[[Main sharedInstance] autohideDownloadsMenu] setState: NSOnState];
            [fm createSymbolicLinkAtPath: filename withDestinationPath: path error: nil];
        }
        
        g_hideDownloads = [fm fileExistsAtPath: filename] ? YES : NO;
    }
    else
        [self swizzledShowDownloads: sender];
}
//--------------------------------------------------------------------------------------------------
- (void)swizzledDownloadDidFinish: (id)sender
{ 
    if (g_hideDownloads)
        [self tryHide];
    
    [self swizzledDownloadDidFinish: sender];
}
//--------------------------------------------------------------------------------------------------
- (void)swizzledDownload: (id)sender didFailWithError: (id)error
{ 
    if (g_hideDownloads)
        [self tryHide];
    
    [self swizzledDownload: sender didFailWithError: error];
}
//--------------------------------------------------------------------------------------------------
- (void)tryHide
{
    if (1 == [[NSClassFromString(@"DownloadMonitor") sharedDownloadMonitor] totalBusy])
    {
        [[NSClassFromString(@"DownloadWindowController") sharedDownloadWindowController] close];
    }    
}
//--------------------------------------------------------------------------------------------------
@end
//==================================================================================================
@implementation Main
@synthesize autohideDownloadsMenu = _autohideDownloadsMenu;
//--------------------------------------------------------------------------------------------------
+ (void)install
{
    NSString *path = [[NSBundle bundleForClass: [Main class]] bundlePath];
    NSFileManager *fm = [NSFileManager defaultManager];
    
    g_hideDownloads = [fm fileExistsAtPath: [path stringByAppendingPathComponent: @"Contents/Resources/flag"]] ? YES : NO;
        
    [Main sharedInstance];
    
    NSLog(@"AppPortal extension for Safari is loaded from: %s", [path UTF8String]);
}
//--------------------------------------------------------------------------------------------------
+ (Main *)sharedInstance
{
    static Main *plugin = nil;
    
    if (nil == plugin)
        plugin = [[Main alloc] init];
    
    return plugin;
}
//--------------------------------------------------------------------------------------------------
- (id)init
{
    if (self = [super init])
    {
        _autohideDownloadsMenu = [[NSMenuItem alloc] initWithTitle: @"Auto hide downloads" action: @selector(showDownloads:) keyEquivalent: @""];
        [_autohideDownloadsMenu setState: g_hideDownloads ? NSOnState : NSOffState];
        [[NSApp windowsMenu] insertItem: _autohideDownloadsMenu atIndex: 11];
        [_autohideDownloadsMenu release];
        
        [NSClassFromString(@"AppController") swizzleMethod: @selector(showDownloads:) withMethod: @selector(swizzledShowDownloads:)];
        [NSClassFromString(@"DownloadMonitor") swizzleMethod: @selector(downloadDidFinish:) withMethod: @selector(swizzledDownloadDidFinish:)];
        [NSClassFromString(@"DownloadMonitor") swizzleMethod: @selector(download:didFailWithError:) withMethod: @selector(swizzledDownload:didFailWithError:)];
    }
    
    return self;
}
//--------------------------------------------------------------------------------------------------
@end
//==================================================================================================
