
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <pthread.h>
#include <boost/bind.hpp>
#include <string>
#include <time.h>
#include </opt/local/include/popt.h>
#include <arpa/inet.h>
#include "pnllmcli.h"
#include "../crypto/crypto.h"
#include "pnsession.h"
#include "pnserver.h"
#include "tmp.h"
#include <string>

#define MAX_LAYOUT_NUM 41

//---------------------------------------------------------------------------
//    pnsession
//---------------------------------------------------------------------------
pnsession::pnsession(boost::asio::io_service& io_service, pnserver * server)
    : socket_(io_service),
      server_ (server)
{
    syslog (LOG_NOTICE, "");
    syslog (LOG_NOTICE, "Instantiating session %p", this );
    syslog (LOG_NOTICE, "");

    server_->inc_session_count();
}

//---------------------------------------------------------------------------
pnsession::~pnsession()
{
    syslog (LOG_NOTICE, "");
    syslog (LOG_NOTICE, "Deleting session %p", this );
    syslog (LOG_NOTICE, "");

    server_->dec_session_count();
    server_->remove_session (this);
}

//---------------------------------------------------------------------------
boost::asio::ip::tcp::socket& pnsession::socket()
{
    return socket_;
}

//---------------------------------------------------------------------------
char * pnsession::data()
{
    return data_;
}

//---------------------------------------------------------------------------
pnserver * pnsession::server()
{
    return server_;
}


//---------------------------------------------------------------------------
void pnsession::start ()
{
    socket_.async_read_some(boost::asio::buffer(data_, max_length),
                            boost::bind(&pnsession::handle_read, this,
                                        boost::asio::placeholders::error,
                                        boost::asio::placeholders::bytes_transferred));
}

//---------------------------------------------------------------------------
void pnsession::handle_read(const boost::system::error_code& error, size_t bytes_transferred)
{
    if (!error)
    {
        pthread_t tid;
        data_[bytes_transferred] = 0;

        if (!strncmp(data_, "#PNTSC#", 7))
        {
            //syslog (LOG_NOTICE, "Handling a GUI request: %s", &data_[7]);
/*
            if (!strncmp(&data_[7], "END", 3))
            {
                pthread_join(&tid);
                server_->remove_session (this);
                delete this;
//                throw std::logic_error ("stop request");
    	    }
*/
            pthread_create(&tid, NULL, proceed, this);

        }
        else
        {
            //syslog (LOG_NOTICE, "Handling non-GUI request: %s", data_);
            server_->register_session (std::string (data_), this);
        }

        socket_.async_read_some(boost::asio::buffer(data_, max_length),
                                      boost::bind(&pnsession::handle_read, this,
                                            boost::asio::placeholders::error,
                                            boost::asio::placeholders::bytes_transferred));
    }
    else
    {
        syslog (LOG_NOTICE, "Entering read error handling (%i) %s, %i", error.value(), error.message().c_str(), bytes_transferred);
        if (error != boost::asio::error::eof)
        {
            syslog (LOG_NOTICE, "Requesting remove session in handle_read(). Also deleting the session object.");
        }
        else
        {
            delete this;
        }
    }

    syslog (LOG_NOTICE, "Leaving handle_read()");
}

//---------------------------------------------------------------------------
std::string pnsession::get_list (const std::string& broker, int type)
{
    return server_->get_list (broker, type);
}

//---------------------------------------------------------------------------
void pnsession::set_list (const std::string& broker, const std::string& list, int type)
{
    server_->set_list (broker, list, type);
}

//---------------------------------------------------------------------------
pnsession * pnsession::find_session (const std::string& session_name)
{
    return server_->find_session (session_name);
}

//---------------------------------------------------------------------------
void pnsession::send (const char * message)
{
    try
    {
    boost::asio::async_write(socket_,
                      boost::asio::buffer(message, strlen (message)),
                      boost::bind(&pnsession::handle_write, this,
                                  boost::asio::placeholders::error));
    }
    catch (std::exception& e)
    {
        syslog (LOG_NOTICE, "send error: %s", e.what());
        server_->remove_session (this);
        delete this;
    }

}

//---------------------------------------------------------------------------
void pnsession::handle_write(const boost::system::error_code& error)
{
    if (error)
    {
        syslog (LOG_NOTICE, "write error: %s", error.message().c_str());
        server_->remove_session (this);
        delete this;
    }
}


//---------------------------------------------------------------------------
//    pnsession::utils
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int pnsession::utils::find_option(const char *haystack, const char *needle,
                                  char *buffer)
{
	buffer[0] = 0;
	if (haystack == NULL) return 0;
	char *p = strstr(haystack, needle), *q;
	if (p)
	{
		p += strlen(needle);
		while (isspace(*p)) p++;
		if (*p)
		{
			q = p;
			while (*p && !isspace(*p)) p++;
			strncpy(buffer, q, p-q);
			return 1;
		}
	}
	return 0;
}

//---------------------------------------------------------------------------
int pnsession::utils::get_free_local_ip(unsigned int *ip)
{
    // try to find free looopback address in 127.0.0.[1 - 254] interval
    //int err_bind = 1;
    int i = 1;
    *ip = 0;
    int s = ::socket (AF_INET, SOCK_STREAM, 0);
    
    setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
    
    if (s >= 0)
    {
        struct sockaddr_in addr;
		bzero(&addr.sin_zero, sizeof addr.sin_zero );
        addr.sin_family = AF_INET;
        addr.sin_port = htons(3389);
        for (i = 1; i < 254; ++i)
        {
            unsigned long a = (unsigned long)((unsigned long)127<<24L) |
                                            ((unsigned long)i);
            addr.sin_addr.s_addr = htonl(a);
			int res = bind(s, (struct sockaddr*) &addr, sizeof(addr));
            if (res == 0)
            {
                errno = 0;
                *ip = i;
                break;
            }
			else syslog (LOG_NOTICE, " failed: res %d, errno %d", res, errno);
        }
        ::close (s);
    }
    return errno;
}

//---------------------------------------------------------------------------
bool pnsession::utils::get_one_broker(char* broker_list, char* onebroker)
{
    if (!broker_list[0])
    {
        onebroker[0] = 0;
        return false;
    }

    char* brokers[64];
    brokers[0] = broker_list;
    int count = 1;
    char* p;
    while (p = strchr(brokers[count - 1], ','))
    {
        *p = 0;
        brokers[count] = p + 1;
        count++;
    }
    if (count == 1)
    {
        strcpy(onebroker, broker_list);
        broker_list[0] = 0;
        return true;
    }

    srand(time(0));
    int c = rand() % count;
    strcpy(onebroker, brokers[c]);

    char buffer[256];
    buffer[0] = 0;
    for (int i = 0; i < count; i++)
    {
        if (i != c)
        {
            if (buffer[0])
                strcat(buffer, ",");
            strcat(buffer, brokers[i]);
        }
    }
    strcpy(broker_list, buffer);
    return true;
}

//---------------------------------------------------------------------------
void pnsession::utils::wrap_brokers(char *broker_list, const char *protocol, const char *port)
{
    char *p = broker_list, *q;
    char broker_line[1024];
    strcpy(broker_line, protocol);
    while (q = strchr(p, ','))
    {
        *q = 0;
        if (!strncmp(p, "http://", 7)) p += 7;
        else if (!strncmp(p, "https://", 8)) p += 8;
        strcat (broker_line, p);
        strcat (broker_line, ":");
        strcat (broker_line, port);
        strcat (broker_line, ",");
        strcat (broker_line, protocol);
    	p = q+1;
    }
    strcat (broker_line, p);
    strcat (broker_line, ":");
    strcat (broker_line, port);
    strcpy(broker_list, broker_line);
}
/*
void pnsession::utils::put_options_in_quotes(char *options)
{
    char *p = options;
	char *r1, r2, sp;
	
	r1 = strstr(options, "-r");
	if (r1)
	{
		p = r1+2;
		while(isspace(p)) ++p;
		if (*p == '\"')
			p = strchr(p+1, '\"')+1;
		else 
			while(!isspace(p) && *p != '=') ++p;

		
		sp = strchr(p, ' '); // skip the space following by the '-r'
		if (r2 == NULL || sp < r2-1)
	}
}
*/
void changeX11ClipboardState(bool state)
{
    char args[0x100]; 
    char task[] = "/usr/bin/defaults write org.x.x11 ";
    char value[0x20] = " -boolean ";
    
    strcat(value, state ? "true" : "false");
    
    *args = 0;
    strcat(args, task);
    strcat(args, "sync_clipboard_to_pasteboard");
    strcat(args, value);
    system(args);
    
    *args = 0;
    strcat(args, task);
    strcat(args, "sync_pasteboard");
    strcat(args, value);
    system(args);
    
    *args = 0;
    strcat(args, task);
    strcat(args, "sync_pasteboard_to_clipboard");
    strcat(args, value);
    system(args);
    
    *args = 0;
    strcat(args, task);
    strcat(args, "sync_pasteboard_to_primary");
    strcat(args, value);
    system(args);
    
    *args = 0;
    strcat(args, task);
    strcat(args, "sync_primary_on_select");
    strcat(args, value);
    system(args);
}
//---------------------------------------------------------------------------
void * pnsession::proceed (void * param)
{

    char applications[8192] = "";
    char response[8192] = "";
    char port[32] = "";
    char protocol[32] = "";
    char broker_line[1024] = "";
    char app_full[256] = "";
    char broker[256] = "";
    //char *protocol_name[] = {"http://", "https://"};
    /*char *layouts[MAX_LAYOUT_NUM] = {"local","ar", "common", "convert-map", "cs",
	 "da", "de", "de-ch", "en-dv", "en-gb", "en-us", "es", "et", "fi", "fo",
	 "fr", "fr-be", "fr-ca", "fr-ch", "he", "hr", "hu", "is", "it", "ja", "ko",
	 "lt", "lv", "mk", "modifiers", "nl", "nl-be", "no", "pl", "pt", "pt-br",
	 "ru", "sl", "sv", "th", "tr"};*/
    const char *constStr;
    char *user = NULL, user_s[64] = "";
    char *password = NULL, password_s[64] = "";
    char *new_password = NULL;  // For password management
    char *domain = NULL, domain_s[64] = "";
    char *app = NULL, app_s[256] = "";
    char *sslgateway = NULL, sslgateway_s[256] = "";
    char *options_d = NULL, options[1024] = "";
    char *pitfilename_d = NULL, *pitfilename = NULL;
    char *tgtfilename = NULL;
    char ticket[8192];
    SimpleString ssCookie, ssRes, ssPitFile;
    int nat = 0, wait = -1, retry = -1;
    int list = 0, List = 0, lista = 0, Lista = 0, listd = 0, Listd = 0;
    int list_type = 0;
    int ret = 0;
    //unsigned char i;
    char *p, *q;
    char *tgt = 0;
    //int useTicket = 0;

    int argc;
    const char **argv;

    //int o_opt = 0
     int  res = 0;
	
	*response = '\0';

    poptContext optCon;   // context for parsing options
    struct poptOption optionsTable[] = {
        { NULL, 'u', POPT_ARG_STRING, &user, 0, "user name (If not specified, the user will be prompted for domain name)" },
        { NULL, 'p', POPT_ARG_STRING, &password, 0, "password (If not specified, the user will be prompted for domain name)" },
        { NULL, 'd', POPT_ARG_STRING, &domain, 0, "domain (If not specified, the user will be prompted for domain name)" },
        { NULL, 'l', POPT_ARG_NONE, &list, 0, "display cached list of published applications and desktops" },
        { NULL, 'L', POPT_ARG_NONE, &List, 0, "query broker for list of published applications and desktops" },
        { "la",  0,  POPT_ARG_NONE|POPT_ARGFLAG_ONEDASH, &lista, 0, "display cached list of published applications" },
        { "La",  0,  POPT_ARG_NONE|POPT_ARGFLAG_ONEDASH, &Lista, 0, "query broker for list of published applications" },
        { "ld",  0,  POPT_ARG_NONE|POPT_ARGFLAG_ONEDASH, &listd, 0, "display cached list of published desktops" },
        { "Ld",  0,  POPT_ARG_NONE|POPT_ARGFLAG_ONEDASH, &Listd, 0, "query broker for list of published desktops" },
        { NULL, 'a', POPT_ARG_STRING, &app, 0, "published application or desktop to launch" },
        { NULL, 'w', POPT_ARG_INT|POPT_ARGFLAG_SHOW_DEFAULT, &wait, 0, "retry interval" },
        { NULL, 'n', POPT_ARG_INT|POPT_ARGFLAG_SHOW_DEFAULT, &retry, 0, "number of retries" },
        { NULL, 't', POPT_ARG_NONE, &nat, 0, "enable NAT support for firewall traversal (for use with Terminal Services servers only)" },
        { NULL, 'o', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &options_d, 0, "qrdesktop options" },
        { NULL, 'f', POPT_ARG_STRING, &pitfilename_d, 0, "pit-file name" },
        { NULL, 'T', POPT_ARG_STRING, &tgtfilename, 0, "TGT-file name" },
        { NULL, 'e', POPT_ARG_STRING, &sslgateway, 0, "SSL gateway name" },
        { NULL, 'c', POPT_ARG_STRING, &new_password, 0, "change current password to this" },
        POPT_AUTOHELP
        POPT_TABLEEND
    };

    pnsession * current_session = (pnsession *) param;
    memset( ticket, 0x0, 8192 );

    {   // to avoid compiler error "crosses initialization of std::..."

    p = current_session->data() + 1;
    p[5] = ' ';

    // parse request
    poptParseArgvString (p, &argc, &argv);
    optCon = poptGetContext(NULL, argc, argv, optionsTable, 0);
    if ((res = poptGetNextOpt(optCon)) != -1)
    {
        sprintf (response, "Error options parsing");
        syslog(LOG_NOTICE, "%s", response);
        goto send_response;
    }
    if ((constStr = poptGetArg(optCon)) != NULL)
    {
        strcpy(broker, constStr);
    }
    if (pitfilename_d)
    {
        size_t len = strlen(pitfilename_d) + 1;
        pitfilename = (char *)malloc(len);
        strncpy(pitfilename, pitfilename_d, len);
    }
        
    free (argv);
    if (!user) user = user_s;
    if (!password) password = password_s;
    if (!domain) domain = domain_s;
    if (!app) app = app_s;
    if (!sslgateway) sslgateway = sslgateway_s;
    if (options_d != NULL) strcpy(options, options_d);

    if (list) list_type = 2;
    else if (List) list_type = 3;
    else if (lista) list_type = 4;
    else if (Lista) list_type = 5;
    else if (listd) list_type = 6;
    else if (Listd) list_type = 7;

    if (wait == -1)
    {
        wait = 5;
    }
    if (retry == -1)
    {
        retry = 20;
    }

	// preparing the pit-file data
	if (pitfilename != NULL)
	{
        syslog(LOG_NOTICE, "Reading pit file: %s", pitfilename);
		int pitfile = open (pitfilename, O_RDONLY);
        
		if (pitfile == -1)
		{
            sprintf (response, "Error while opening pit-file: %s", strerror (errno));
            syslog(LOG_NOTICE, "%s", response);
            free(pitfilename);
            goto send_response;
		}
    	int pitfilesize = lseek (pitfile, 0L, SEEK_END);
    	char *pitbuf = (char *) malloc (pitfilesize);
    	lseek (pitfile, 0L, SEEK_SET);
    	ret = read (pitfile, pitbuf, pitfilesize);
    	if (ret < 0)
    	{
            sprintf (response, "Error while reading pit-file: %s", strerror (errno));
            syslog(LOG_NOTICE, "%s", response);
            free(pitfilename);
            goto send_response;
    	}
    	else if (ret == 0)
    	{
    		ret = pitfilesize;
    	}
    	ssPitFile.LoadData (pitbuf, ret);
        
        bool del = false;
        char ye;
        
        if (ssPitFile.FindItem("deletefile:i:", &ye) && ye == '1')
            del = true;
        
    	close (pitfile);
        
        if (del)
            remove(pitfilename);
        
        free(pitfilename);
	}

    if ( ssPitFile.FindItem( "ticket:s:", ticket ) == 0 )
    {
        if (*broker == 0)
        {
            // search for a broker name in the pit-file
            char szTmp[10];
            if (ssPitFile.FindItem("llmprotocol:i:", szTmp) == 0 ||
                ssPitFile.FindItem("llmservers:s:", broker) == 0 ||
                ssPitFile.FindItem("llmtcpport:s:", port) == 0)
            {
                goto wrong_request;
            }
            if (strcmp(szTmp, "0") == 0)
            {
                strcpy(protocol, "https://");
            }
            else
            {
                strcpy(protocol, "http://");
            }
            p = broker;
        }
        else
            p = broker;
/*
	{
	    if (!strncmp(broker, "http://", 7))
	    {
		    strcpy(protocol, "http://");
	        p = broker + sizeof ("http://") - 1;
	    }
	    else
	    {
		    strcpy(protocol, "https://");
		    p = broker + sizeof ("https://") - 1;
	    }
	    if ((q = strchr(p, ':')) == NULL)
	    {
	        strcpy(port, "443");
	    }
	    else
	    {
		    strcpy(port, q + 1);
	        *q = 0;
	    }
	}
 */ 
    }
    //utils::wrap_brokers(p, protocol, port);  
    strncpy(broker_line, p, 1024);  //btw UNREAL exploit for reverser :)

    if ((tgtfilename == NULL) && pitfilename)
    {
        syslog( LOG_NOTICE, "Checking credentials" );
        // If a ticket was specified, then don't bother looking for other credentials
        if ( ssPitFile.FindItem( "ticket:s:", ticket ) == 0 )
        {
            if (*user == 0   && ssPitFile.FindItem("username:s:", user) == 0 ||
                *domain == 0 && ssPitFile.FindItem("domain:s:", domain) == 0)
            {
                syslog(LOG_ERR, "No credentials!");
                goto wrong_request;
            }
            if (*password == 0)
            {
                char szPassword[514];
                if (ssPitFile.FindItem("password:s:", szPassword) == 0)
                {
                    syslog(LOG_ERR, "No password found!");
                    goto wrong_request;
                }
                // Decrypt the password.
                unsigned char rgbBuffer[257];
                HexToBin(rgbBuffer, sizeof(rgbBuffer), szPassword, strlen(szPassword));
                Decrypt(rgbBuffer, sizeof (rgbBuffer));
                Decode256(rgbBuffer, password);
            }
        }
    }
        
    if (*app == 0 && list_type == 0 && pitfilename/*&& new_password && *new_password == 0*/)
    {
    	char szApp[2050];
    	if (ssPitFile.FindItem("alternate shell:s:", szApp) == 0)
    	{
    		goto wrong_request;
    	}
        unsigned char rgbBuffer1[1026];
        HexToBin(rgbBuffer1, sizeof(rgbBuffer1), szApp, strlen(szApp));
        Decrypt(rgbBuffer1, sizeof (rgbBuffer1));
        Decode1024(rgbBuffer1, app);
    }

//    syslog (LOG_NOTICE, "brokers:%s  user:%s  password:%s  domain:%s  app:%s", broker_line, user, password, domain, app);

	if (!nat)
	{
		char sznat[32];
		if (ssPitFile.FindItem("llmenablenat:i:", sznat))
		{
		    nat = atoi(sznat);
		}
	}

	if (list_type > 0) goto proceed;
        
	syslog (LOG_NOTICE, "qrdesktop options: %s", options);
	// try to find other unset options in the pit-file
	if (ssPitFile.Length() != 0)
	{
		if (*sslgateway == 0)
		{
			ssPitFile.FindItem("sslgateway:s:", sslgateway);
		}
        
		char option[32], option2[32];
        
        if (ssPitFile.FindItem("screen mode id:i:", option))
        {
            if (*option != '0')
            {
                switch (*option)
                {
                    case '1':
                        
                        if (utils::find_option(options, "-g", option) == 0)
                        {
                            if (ssPitFile.FindItem("desktopwidth:i:", option) && ssPitFile.FindItem("desktopheight:i:", option2))
                            {
                                if ((*option != '0') && (*option2 != '0'))
                                {
                                    strcat(options, " -g ");
                                    strcat(options, option);
                                    strcat(options, "x");
                                    strcat(options, option2);
                                }
                            }
                        }
                        break;
                        
                    default:
                        if (utils::find_option(options, "-f", option) == 0)
                        {
                            strcat(options, " -f ");
                        }
                }
            }
        }
		
		if (utils::find_option(options, "-a", option) == 0)
		{
			if (ssPitFile.FindItem("session bpp:i:", option) && *option != '0')
			{
                strcat(options, " -a ");
                strcat(options, option);
    		}
		}
        
        if (utils::find_option(options, "--gaenable", option) == 0)
		{
			if (ssPitFile.FindItem("bitmapacceleration:i:", option) && *option != '0')
			{
                strcat(options, " --gaenable ");
    		}
		}
        
        if (utils::find_option(options, "-f", option) && utils::find_option(options, "--span-monitors", option) == 0)
		{
			if (ssPitFile.FindItem("spanmonitors:i:", option) && *option != '0')
			{
                strcat(options, " --span-monitors ");
    		}
		}
        /*
		if (utils::find_option(options, "-n", option) == 0)
		{
			if (ssPitFile.FindItem("clientName:s:", option) && *option != 0)
			{
    			strcat(options, " -n ");
    			strcat(options, option);
    		}
		}
        
		if (utils::find_option(options, "-k", option) == 0)
		{
			if (ssPitFile.FindItem("keyboardLayout:i:", option) && *option != '0')
			{
			int layout = atoi(option);
			if (layout >= 0 && layout < MAX_LAYOUT_NUM)
			{
    			    strcat(options, " -k ");
    			    strcat(options, layouts[layout]);
    			}
    		}
		}
        */
        
        //enabledesktopcomposition
        //enablefontsmoothing
        //compression
        //redirecthandhelds
        //redirectmicrophone
        
        int hexcode = 0;
        
        if (ssPitFile.FindItem("disable wallpaper:i:", option) && *option == '1')
            hexcode |= 0x01;
        
        if (ssPitFile.FindItem("disable full window drag:i:", option) && *option == '1')            
            hexcode |= 0x02;
        
        if (ssPitFile.FindItem("disable menu anims:i:", option) && *option == '1')
            hexcode |= 0x04;
        
        if (ssPitFile.FindItem("disable themes:i:", option) && *option == '1')
            hexcode |= 0x08;

        snprintf(option, 32, "%#x", hexcode);
        strcat(options, " -x ");
        strcat(options, option);
        
		if (ssPitFile.FindItem("bitmapcachepersistenable:i:", option) && *option != '0')
		{
            strcat(options, " -P ");
		}
        
		if (ssPitFile.FindItem("audiomode:i:", option))
		{
		    switch (*option)
		    {
                case '1':
                    strcat(options, " -r sound:off");
                    break;
                case '2':
                    strcat(options, " -r sound:remote");
                    break;
                default :
                    strcat(options, " -r sound");
		    }
		}
        
        if (ssPitFile.FindItem("keyboardhook:i:", option))
		{
		    switch (*option)
		    {
                case '0':
                    strcat(options, " -K");
                    break;
                case '1':
                    break;
                default :
                    if (utils::find_option(options, "-f", option))
                        strcat(options, " -K");
		    }
		}
        
		if (ssPitFile.FindItem("redirectdrives:i:", option) && *option != '0')
		{
            FILE *res = popen("mount | grep /dev/disk | sed -e \'s/\\ (/:/g\' -e \'s/\\ on\\ /:/g\' | cut -d : -f 2", "r");
            char pbuffer[0x2000];
            char disks[0x2000] = "";
            char disk[0x100] = "";
            char mount[0x100] = "";
            int pos = 0;
            int resum = 0;
            bool fla = false;
            
            memset(pbuffer, 0, 0x2000);
            memset(disks, 0, 0x2000);
            memset(disk, 0, 0x100);
            memset(mount, 0, 0x100);
            
            fread(pbuffer, 1, 0x2000, res);
            
            for (;;)
            {
                resum = sscanf(pbuffer + pos, "%255[^\n]s", disk);
                
                if (resum <= 0)
                    break;
                
                pos += strlen(disk) + 1;
                
                if (fla)
                {
                    strcat(disks, ",");
                    fla = false;
                }
                
                if (strcmp(disk, "/"))
                {
                    std::string path = disk;
                    std::string::size_type lastSlashPos=path.find_last_of('/');
                    std::string filename=path.substr(lastSlashPos+1,path.size()-lastSlashPos-1);
                    
                    strncpy(mount, filename.c_str(), 255);
                }
                else
                    strcpy(mount, "Disk0");
                
                strcat(disks, "\"");
                strcat(disks, mount);
                strcat(disks, "\"=\"");
                strcat(disks, disk);                
                strcat(disks, "\"");

                fla = true;
            } 
            
            pclose(res);
            
            strcat(options, " -r disk:");
            strcat(options, disks);
		}
		
		if (ssPitFile.FindItem("redirectcomports:i:", option) && *option != '0')
		{
            //strcat(options, " -r comport:");
            //strcat(options, option);
        }
        
		if (ssPitFile.FindItem("redirectprinters:i:", option) && *option != '0' ||
            ssPitFile.FindItem("redirectuniversalprinters:i:", option) && *option != '0')
		{
            FILE *res = popen("lpstat -a | cut -d\" \" -f1", "r");
            char printer[0x100] = "";
            char printers[0x1000] = "";
            char pbuffer[0x1000];
            int pos = 0;
            int resum = 0;
            bool fla = false;
            
            memset(printer, 0, 0x100);
            memset(printers, 0, 0x1000);
            memset(pbuffer, 0, 0x1000);
            
            fread(pbuffer, 1, 0x1000, res);
            
            for (;;)
            {
                resum = sscanf(pbuffer + pos, "%s", printer);
                
                if (resum <= 0)
                    break;
                
                if (fla)
                {
                    strcat(printers, ",");
                    fla = false;
                }
                
                strcat(printers, printer);
                
                pos += strlen(printer) + 1;
                fla = true;
                
            } 
                    
            pclose(res);
            
            strcat(options, " -r printer:");
            strcat(options, printers);
		}
        
		if (ssPitFile.FindItem("redirectsmartcards:i:", option) && *option != '0')
		{
        //    strcat(options, " -r scard");
    	/*	if (ssPitFile.FindItem("smartcards:s:", option) && *option != 0)
	    	{
                strcat(options, ":");
                strcat(options, option);
    		}*/
		}

		if (ssPitFile.FindItem("redirectclipboard:i:", option) && *option != '0')
        {
            strcat(options, " -r clipboard:CLIPBOARD");
            
            changeX11ClipboardState(true);
        }
        else
            changeX11ClipboardState(false);
	}

    syslog (LOG_NOTICE, "SSL gateway: %s", sslgateway);
    syslog (LOG_NOTICE, "qrdesktop options: %s", options);
//	utils::put_options_in_quotes(options);

	goto proceed;


wrong_request:

    strcpy (response, "Wrong request");
    goto send_response;


proceed:

    if (tgtfilename)
    {
        syslog (LOG_NOTICE, "tgtfilename: %s", tgtfilename);
        
        tgt = ::create_tgt_response(tgtfilename);
        if (tgt == 0)
        {
            sprintf (response, "Error in reading TGT file %s: %s", tgtfilename, response);
            goto send_response;
        }
    }
		
		res = 0;

    if (list_type > 0)
    {
        int type = list_type / 2 - 1;
        char onebroker[256];
        while (utils::get_one_broker(broker_line, onebroker))
        {
            syslog(LOG_NOTICE, "Connect to broker: %s", onebroker);
            std::string broker_str (onebroker);
            if ((list_type & 1) || (current_session->get_list(broker_str, type) == ""))
            {
	            // getting a list of applications from given broker address
                if ((ret = Authenticate(onebroker, user, password, domain, tgt, nat, ssCookie)) != 0)
                {
                    sprintf (response, "%s", (char *)ssCookie);
                    syslog(LOG_NOTICE, "Authenticate error: %s", response);
                    continue;
                }
                if ((ret = GetApplicationList(onebroker, (char*)ssCookie, ssRes, type)) != 0)
                {
                    sprintf (response, "%s", (char *) ssRes);
                    syslog(LOG_NOTICE, "Get app list error: %s", response);
                    continue;
                }

                p = ssRes.GetData();
                int size = 0;

/* old code - without getting app icon and app folder name
                while (q = strchr(p, '\n'))
                {
                    if (*p == '[' && (*(q - 1) == ']'))
                    {
                        memcpy(applications + size, p + 1, q - p - 2);
                        size += q - p - 2;
                        applications[size++] = '\n';
                    }
                    p = q + 1;
                }
end of the old code */

		while (q = strchr(p, '\n'))
		{
			if (*p == '[' && (*(q - 1) == ']'))
                    	{
                        	memcpy(applications + size, p, q - p);
                        	size += q - p;
                        	applications[size++] = ',';
                    	}
		
			if (strncmp(p, "FolderName=", strlen("FolderName=")) == 0)
			{
				//p += strlen("FolderName=");
				memcpy(applications + size, p, q - p);
				size += q - p;
				applications[size++] = ',';
			}
			if (strncmp(p, "SmallIcon=", strlen("SmallIcon=")) == 0)
			{
				memcpy(applications + size, p, q - p);
				size += q - p;
				applications[size++] = '\n';
			}
			p = q + 1;
		}

                applications[size++] = 0;
                std::string s (applications);
                current_session->set_list(broker_str, s, type);

            }   // refreshing a list of application
			
			if (res != 0)
				goto send_response;

            strcpy (response, "List of applications:\n");
            strcat (response, (current_session->get_list(broker_str, type)).c_str());
            goto send_response;
        }

        goto send_response;
    }

    // Password management
    if (new_password != NULL)
    {
        char onebroker[256];
        while (utils::get_one_broker(broker_line, onebroker))
        {
            syslog(LOG_NOTICE, "Connect to broker: %s", onebroker);

            // Authentication request
            //if ((ret = Authenticate(onebroker, user, password, domain, tgt, nat, ssCookie)) != 0)
            //{
            //    sprintf (response, "%s", (char *) ssCookie);
            //    syslog(LOG_NOTICE, "Authenticate error: %s", response);
            //    continue;
            //}

            // Change password request
            if ((ret = ChangePassword(onebroker, user, domain, password, new_password,
                                      (char *) ssCookie, ssRes)) != 0)
            {
                sprintf(response, "%s", (char *) ssRes);
                syslog(LOG_NOTICE, "Change password error: %s", response);
                continue;
            }

            sprintf (response, "Password management: %s", (char *) ssRes);
            goto send_response;
        }
		goto send_response;
    }

    // -----------------------------------
    // starting application
    // -----------------------------------
        
        char serverip[256];
        char servername[256];
        char shell[2050] = "\"";
        char workdir[128];
        char ticket[1024];
        
        // If I have a ticket in the pitfile, then I can skip
        // this whole section about looking for a broker
if ( ssPitFile.FindItem( "ticket:s:", ticket ) == 0  )
{
    // search for working broker
    char onebroker[256];
    while (utils::get_one_broker(broker_line, onebroker))
    {
        syslog(LOG_NOTICE, "Connect to broker: %s", onebroker);
        int num_attempts = retry;
        ret = Authenticate(onebroker, user, password, domain, tgt, nat, ssCookie);
        while ((ret == PN_ERROR_VM_POWERING_UP ||
                ret == PN_ERROR_VM_OS_STARTING ||
                ret == PN_ERROR_BROKER_BUSY) &&
                --num_attempts > 0)
        {
            if (num_attempts == retry-1)
            {
                sprintf (response, "PENDING%s", (char *)ssCookie);
                syslog(LOG_NOTICE, "%s", response);
                current_session->send (response);
            }
            syslog(LOG_NOTICE, " sleeping..");
            sleep (wait);
            ret = Authenticate(onebroker, user, password, domain, tgt, nat, ssCookie);
        }
        if (ret != 0)
        {
            sprintf (response, "Broker authentication error: %s", (char *)ssCookie);
            syslog(LOG_NOTICE, "%s", (char *)response);
            continue;
        }
        break;
    }
    if (ret != 0)
    {
        strcpy (response, ssCookie);
        goto send_response;
    }

    syslog(LOG_NOTICE, "Authentication passed");
    // working with UI, the application name is always exactly the same from server
    strcpy(app_full, app);

    // here try to access to the broker
    ssRes = "";
    syslog(LOG_NOTICE, "Getting launch info, broker: %s", onebroker);
    int num_attempts = retry;
    ret = GetLaunchInfo(onebroker, ssCookie, app_full, ssRes);
    while ((ret == PN_ERROR_VM_POWERING_UP ||
            ret == PN_ERROR_VM_OS_STARTING ||
            ret == PN_ERROR_BROKER_BUSY) && --num_attempts > 0)
    {
        if (num_attempts == retry-1)
        {
            sprintf (response, "PENDING%s", (char *)ssRes);
            syslog (LOG_NOTICE, "%s", response);
            current_session->send (response);
        }
        syslog(LOG_NOTICE, " sleeping..");
        sleep (wait);
        ret = GetLaunchInfo(onebroker, ssCookie, app_full, ssRes);
    }
    if (ret != 0)
    {
        sprintf (response, "Unable to get information from broker: %s", (char *)ssRes);
        goto send_response;
    }
    // creating start application info string

    ssRes.FindItem("servername:s:", servername);
    ssRes.FindItem("serveraddr:s:", serverip);
    ssRes.FindItem("alternate shell:s:", shell + 1);
    ssRes.FindItem("shell working directory:s:", workdir);
    ssRes.FindItem("ticket:s:", ticket);
    }
    else
    {
    // creating start application info string
    ssPitFile.FindItem( "servername:s:", servername );
    ssPitFile.FindItem( "serveraddr:s:", serverip );
    ssPitFile.FindItem( "shell working directory:s:", workdir );
    ssPitFile.FindItem( "ticket:s:", ticket );

    // Already decrypted, see above
        
        int len = strlen(app);
        
        if (len > 2050)
            strncpy( shell + 1, app, 2050);
        else
            strncpy( shell + 1, app, len);
    }

    // no shell, it's a desktop, using explorer.exe as shell
    if (shell[1] == 0)
    {
        strcpy (shell + 1, "explorer.exe");
    }
    strcat(shell, "\"");

    syslog (LOG_NOTICE, "Launch info: %s %s %s %s", servername, serverip, shell, workdir);
	char *p1, *q1;
    p = servername;
    p1 = serverip;
    std::string session_name;
    pnsession * session = NULL;
    while (q = strchr(p, ','))
    {
	    // try to find existing session connected to given remote host
	    // if succeed share the session, otherwise create a new one
	    *q = '\0';
	    *(q1 = strchr(p1, ',')) = '\0';
	    session_name = p;
    	session = current_session->find_session (session_name);
		if (session != NULL) break;
        p = q + 1;
        p1 = q1 + 1;
    }
    if (session == NULL)
    {
        session_name = p;
   	    session = current_session->find_session (session_name);
    }

    if (session == NULL)
    {
        //unsigned int ip;

    	session_name = servername;
/*
        if ((i = utils::get_free_local_ip(&ip)) != 0)
        {
            sprintf (response, "Unable to launch qrdesktop: error getting local ip: %s (%d)",
                    strerror(i), i);
            syslog (LOG_NOTICE, "%s", response);
            goto send_response;
        }
*/
        char local_addr[64];
        sprintf (local_addr, "127.0.0.1"/*, ip*/);
        syslog (LOG_NOTICE, "Creating a new session: %s", session_name.c_str());

        if (tgt)    // if TGT file is present it's preferable to use TGT
        {
            sprintf (response, "STARTqrdesktop\n-G\n%s\n-s\n%s\n",
                     tgtfilename, shell);
        }
        else    
        {
            // if TGT file is absent then credentials are necessarily present
            // (it's been checked up before)
            //
            // DCT : If the ticket is in the launch file, then we can
            //       use it to login.  So we do not specify credentials.
            if ( strlen(ticket) == 0 )
            {
                sprintf (response, "STARTqrdesktop\n-u\n%s\n-p\n%s\n-d\n%s\n-s\n%s\n",
                    user, password, domain, shell);
            }
            else
            {
                sprintf (response, "STARTqrdesktop\n-s\n%s\n", shell );
            }
        }

        // add rdesktop options
        if (*options != 0)
        {
        	char *p = options;//, *p1, *q, *quot1, *quot2;
        	int num_quotes = 0;

            // first check if the number of double-quotes in the string is even
            while (p && *p)
            {
                if (*p++ == '"') ++num_quotes;
            }
            if (num_quotes & 1)
            {
                sprintf (response, "Wrong rdesktop parameters string");
                syslog (LOG_NOTICE, "%s", response);
                goto send_response;
            }
            // now parse the options string
            p = options;
        	while (p && isspace(*p)) ++p;
        	// till the end of the string:
        	while (p && *p)
        	{
        	    // go ahead while all whitespaces are between quotes
                q = p;
                
                bool qq = false;
                
                while (q && *q)
                {
                    if ('"' == *q)
                        qq = !qq;
                    
                    if (isspace(*q) && !qq)
                        break;
                    
                    ++q;
                }
            	//q = strchr(p, ' ');
            	/*quot1 = strchr(p, '"');
				if (quot1 == NULL) quot1=p+strlen(p);
            	quot2 = strchr(quot1 + 1, '"');
				if (quot2 == NULL) quot2=quot1+strlen(quot1);
            	while (q && q > quot1 && q < quot2)
            	{
            	    p1 = quot2+1;
                	q = strchr(p1, ' ');
					if (q == NULL) q=p1+strlen(p1);
                	quot1 = strchr(p1, '"');
					if (quot1 == NULL) quot1=p1+strlen(p1);
                	quot2 = strchr(quot1 + 1, '"');
					if (quot2 == NULL) quot2=quot1+strlen(quot1);
            	}*/
            	// add all the passed symbols as a separate parameter
   	        	strncat(response, p, q-p);
	        	strcat(response, "\n");
	        	p = q;
	        	while (p && isspace(*p)) ++p;
        	}
        }
        
        // add last four parameters
        char tail[256];
        snprintf (tail, 256, "%s\n%s\n%s\n", local_addr, serverip, session_name.c_str());
        if (*sslgateway) {
        	strcat(tail, sslgateway);
        	strcat(tail, "\n");
        }
        strcat (response, tail);
    }
    else
    {
        sprintf (response, "SHARED session: %s", session_name.c_str());
        syslog (LOG_NOTICE, "%s", response);
        char command[256];
        sprintf (command, "SA%s\n%s", shell, workdir);
        session->send (command);
    }

    }   // to avoid compiler error "crosses initialization of std::..."

send_response:

    if (tgt)
    {
        free(tgt);
    }
    poptFreeContext(optCon);
	if (*response == '\0')
		sprintf(response, "Operation failed: unknown error");
    syslog (LOG_NOTICE, "At send response: %s", response);
    current_session->send (response);
    
    return NULL;
}
