
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

//
//  AppPortal_AppDelegate.m
//  Provision AppPortal
//
//  Created by smalti on 4/1/09.
//  Copyright __MyCompanyName__ 2009 . All rights reserved.
//

#import "AppPortal_AppDelegate.h"
//#import <Carbon/Carbon.h>

extern pid_t childPID;

@implementation AppPortal_AppDelegate

#pragma mark -
#pragma mark Autogenerated code

/**
    Returns the support folder for the application, used to store the Core Data
    store file.  This code uses a folder named "AppPortal" for
    the content, either in the NSApplicationSupportDirectory location or (if the
    former cannot be found), the system's temporary directory.
 */

- (NSString *)applicationSupportFolder {

    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
    return [basePath stringByAppendingPathComponent:@"AppPortal"];
}


/**
    Creates, retains, and returns the managed object model for the application 
    by merging all of the models found in the application bundle.
 */
 
- (NSManagedObjectModel *)managedObjectModel {

    if (managedObjectModel != nil) {
        return managedObjectModel;
    }
		
    managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles:nil] retain];    
    return managedObjectModel;
}


/**
    Returns the managed object context for the application (which is already
    bound to the persistent store coordinator for the application.) 
 */
 
- (NSManagedObjectContext *) managedObjectContext {

    if (managedObjectContext != nil) {
        return managedObjectContext;
    }

    managedObjectContext = [[NSManagedObjectContext alloc] init];
    return managedObjectContext;
}


/**
    Returns the NSUndoManager for the application.  In this case, the manager
    returned is that of the managed object context for the application.
 */
 
- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window {
    return [[self managedObjectContext] undoManager];
}


/**
    Performs the save action for the application, which is to send the save:
    message to the application's managed object context.  Any encountered errors
    are presented to the user.
 */
 
- (IBAction) saveAction:(id)sender {

    NSError *error = nil;
    if (![[self managedObjectContext] save:&error]) {
        [[NSApplication sharedApplication] presentError:error];
    }
}


/**
    Implementation of the applicationShouldTerminate: method, used here to
    handle the saving of changes in the application managed object context
    before the application terminates.
 */
 
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {

    NSError *error;
    int reply = NSTerminateNow;
    
    if (managedObjectContext != nil) {
        if ([managedObjectContext commitEditing]) {
            if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
				
                // This error handling simply presents error information in a panel with an 
                // "Ok" button, which does not include any attempt at error recovery (meaning, 
                // attempting to fix the error.)  As a result, this implementation will 
                // present the information to the user and then follow up with a panel asking 
                // if the user wishes to "Quit Anyway", without saving the changes.

                // Typically, this process should be altered to include application-specific 
                // recovery steps.  

                BOOL errorResult = [[NSApplication sharedApplication] presentError:error];
				
                if (errorResult == YES) {
                    reply = NSTerminateCancel;
                } 

                else {
					
                    int alertReturn = NSRunAlertPanel(nil, @"Cannot save changes while quitting. Quit anyway?" , @"Quit anyway", @"Cancel", nil);
                    if (alertReturn == NSAlertAlternateReturn) {
                        reply = NSTerminateCancel;	
                    }
                }
            }
        } 
        
        else {
            reply = NSTerminateCancel;
        }
    }
    
    [self hideDockForX11Window: NO];
    
    return reply;
}


/**
    Implementation of dealloc, to release the retained variables.
 */

- (void) dealloc {
	
    [managedObjectContext release], managedObjectContext = nil;
    [managedObjectModel release], managedObjectModel = nil;
	[m_statusItem release];
    [farmsConnected release];
    [super dealloc];
}

#pragma mark -
#pragma mark Functional methods

// change password function
- (int)changePassword {
	// setting up the task arguments
	
	NSMutableArray *args = [NSMutableArray array];
    
	if (![manageConnectionWindow createChangePassRequest:args])
    {
        [errorMessage setStringValue: @"Password management disabled"];
            
        return -1;
    }
    
	[args addObject:@"-c"];
	[args addObject:[newPass stringValue]];
	
	// create and launch task
	NSMutableData *data = [NSMutableData dataWithLength:0];
	NSMutableData *errData = [NSMutableData dataWithLength:0];
	PntscTask *pntscT = [[PntscTask alloc] init];
	int changePassStatus = [pntscT launchPntscWithArgs:args outData:data errorData:errData];
	[pntscT release];
	if(changePassStatus != 0)
	{
		NSString *errMessage = [[NSString alloc] initWithData: errData encoding: NSASCIIStringEncoding];
		[errorMessage setStringValue:[errMessage autorelease]];
	}

	return changePassStatus;
}

- (int)doGetAppList {
	return [manageConnectionWindow getApplicationsList];
}

- (void)autolaunchConnections {
	NSArray *existingConnectionsArray = [NSArray arrayWithArray:[connectionArray arrangedObjects]];
	
	for(int i=0; i != [existingConnectionsArray count]; ++i)
	{
		[connectionArray setSelectionIndex:i];
		if([[existingConnectionsArray objectAtIndex:i] doAutoLaunch])
		{
			if (i==0)
			{
				[btnChangeLocation setEnabled:YES];
				[btnLogonAsDifUser setEnabled:YES];
				[btnRefresh setEnabled:YES];				
			}
			if (![[[connectionArray selection] valueForKey:@"credentials"] doSave])
				[NSApp runModalForWindow:logonAsDifUserPanel];
			if ([self doGetAppList] != 0)
				break;
			NSArray *curConnectionAppsArray = [NSArray arrayWithArray:[[existingConnectionsArray objectAtIndex:i] apps]];
			for(int j=0; j != [curConnectionAppsArray count]; ++j)
			{
				AppList *curApp = [[AppList alloc] initWithAppList:[curConnectionAppsArray objectAtIndex:j]];
				NSString *curAppName = [NSString stringWithString:[curApp appName]];
				[manageConnectionWindow launcAppWithName:curAppName];
				[curApp release];
			}
		}
	}	
}

- (void)changeLocationCrutch {
	// this is a crutch
	// it's been made on order to not to add dependencies between modules
	
	// reset default location
	NSArray *curLocationsArr = [NSArray arrayWithArray:[locationArray arrangedObjects]];
	int i=0;
	for(; i != [curLocationsArr count]; ++i)
	{
		if([[curLocationsArr objectAtIndex:i] doUseAsDefault])
		{
			[[curLocationsArr objectAtIndex:i] setDoUseAsDefault:NO];
			break;
		}
	}	
	
	[[[locationArray selectedObjects] objectAtIndex:0] setDoUseAsDefault:YES];
	
	[self doGetAppList];
	
	// undo all the changes
	[[[locationArray selectedObjects] objectAtIndex:0] setDoUseAsDefault:NO];
	if(i<[curLocationsArr count])
		[[curLocationsArr objectAtIndex:i] setDoUseAsDefault:YES];
	
	// TODO: return error sode of  pntsc task
}

#pragma mark -
#pragma mark Other useful methods

- (void) hideDockForX11Window: (BOOL)really {
	NSString *path = [NSString stringWithString:@"/Applications/Utilities/X11.app/Contents/Info.plist"];
	NSData *plistData = [NSData dataWithContentsOfFile:path];
	if (plistData == nil)
		return;
	NSString *error;
	NSPropertyListFormat format;
	NSMutableDictionary *plist;
	
	plist = (NSMutableDictionary *) [NSPropertyListSerialization propertyListFromData:plistData
											 mutabilityOption:NSPropertyListMutableContainersAndLeaves
													   format:&format
											 errorDescription:&error];
	if(!plist) {
		NSLog(error);
		[error release];
	}
	
    {
		[plist setValue:[NSNumber numberWithInt: really ? 1 : 0] forKey:@"LSUIPresentationMode"];
	
		plistData = [NSPropertyListSerialization dataFromPropertyList:plist
														 format:NSPropertyListXMLFormat_v1_0
											   errorDescription:&error];
		if(plistData) {
			[plistData writeToFile:path atomically:NO];
		}
		else {
			NSLog(error);
			[error release];
		}
	}
	
}

- (void) awakeFromNib {
	[self hideDockForX11Window: YES];
	manageConnectionWindow = [ManageConnectionAssist sharedInstance];
	[manageConnectionWindow initTabAssistsArray];
	[manageConnectionWindow initTabs];
	[farmList setDoubleAction:@selector (doConnectFarm:)];
	[appList setDoubleAction:@selector (doConnectApp:)];
    [appListDI setDoubleAction:@selector (doConnectApp:)];
    farmsConnected = [[NSMutableDictionary alloc] initWithCapacity: 10];
    openFileStart = NO; 
    autolaunchConnectionsStart = YES;
}

- (BOOL) application: (NSApplication*)theApp openFile: (NSString *)path
{   
#ifndef ENABLE_DESKTOP_INTEGRATION
    [NSApp hide: self];
#endif
    
    //NSFileManager *manager = [NSFileManager defaultManager];
    //NSString *new_path = [NSString stringWithFormat: @"/tmp/%@", [path lastPathComponent]];
    //NSError *err;
    
    //if ([manager fileExistsAtPath: new_path])
        //[manager removeItemAtPath: new_path error: &err];
    
    //if ([manager copyItemAtPath: path toPath: new_path error: &err])
    //{
        NSLog([NSString stringWithFormat: @"Launching: pntsc -f %@\n", path]);
        
        [NSTask launchedTaskWithLaunchPath: @"/usr/bin/pntsc" arguments: [NSArray arrayWithObjects: @"-f", path, nil]];
    //}
    
    openFileStart = YES;
    
    return YES;
}

- (void)applicationDidUnhide:(NSNotification *)aNotification
{
#ifdef ENABLE_DESKTOP_INTEGRATION
    [NSApp hide: self];
#else
    [window orderFront: self];
#endif
}

#pragma mark-
#pragma mark Desktop Integration

- (NSMenu *)createMenu
{
#ifdef ENABLE_DESKTOP_INTEGRATION
	NSZone *menuZone = [NSMenu menuZone];
	//NSMenu *
	menu = [[NSMenu allocWithZone:menuZone] init];
	NSMenuItem *menuItem;
	
	// Add to items
	menuItem = [menu addItemWithTitle:@"Connect/Disconnect" action:@selector (doShowDeskIntegrationWindow:) keyEquivalent:@""];
	[menuItem setTarget:self];
    
    // Add separator
	[menu addItem:[NSMenuItem separatorItem]];
    
    menuItem = [menu addItemWithTitle:@"Manage Connections..." action:@selector (manageConnection:) keyEquivalent:@""];
	[menuItem setTarget:self];
	menuItem = [menu addItemWithTitle:@"Change Password..." action:@selector (doChangePass:) keyEquivalent:@""];
	[menuItem setTarget:self];
   
    // Add separator
	[menu addItem:[NSMenuItem separatorItem]];
    
    menuItem = [menu addItemWithTitle:@"About AppPortal" action:@selector (orderFrontStandardAboutPanel:) keyEquivalent:@""];
	[menuItem setTarget: NSApp];
	
	// Add quit action
	menuItem = [menu addItemWithTitle:@"Quit AppPortal" action:@selector (closeApp:) keyEquivalent:@""];
	[menuItem setTarget:self];
	
	// Add separator and submenu
	//[menu addItem:[NSMenuItem separatorItem]];
    //	farmSubmenuArr = [[NSMutableArray alloc] init];
	
	return menu;
#else
    return nil;
#endif
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
#ifndef ENABLE_DESKTOP_INTEGRATION
    if (openFileStart)
        [window makeKeyWindow];
    else
        [window makeKeyAndOrderFront: self];
#else
	[self createMenu];
	
	m_statusItem = [[[NSStatusBar systemStatusBar] statusItemWithLength:NSSquareStatusItemLength] retain];
	[m_statusItem setMenu:menu];
	[m_statusItem setHighlightMode:YES];
	[m_statusItem setToolTip:@"vWorspace AppPortal"];
	[m_statusItem setImage:[NSImage imageNamed:@"welcome"]];
	
	[menu release];
#endif
    
    if (autolaunchConnectionsStart)
    {
        [self autolaunchConnections];
        autolaunchConnectionsStart = NO;
    }
}


#ifndef ENABLE_DESKTOP_INTEGRATION
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication {
	return YES;
}
#endif

- (void)closePanel:(NSWindow *) wndClose {
	[NSApp stopModal];
	[wndClose close];
}

- (BOOL)isValidTextField:(NSTextField *)textfield {
	NSString * typedText = [NSString stringWithString:[textfield stringValue]];
	return(![typedText isEqual:@""]);
}

- (void)addIAppsToTray {
    
    if (![[[connectionArray selection] valueForKey:@"launchedApps"] count])
        return;
	
	// remove farm submenu from array and menu if submenu with farm name already exists
	int j = [menu numberOfItems];
	for(int i = 0; i< j; ++i)
	{
		if([[[menu itemAtIndex:i] title] isEqualToString:[[connectionArray selection] valueForKey:@"farmName"]])
		{
			[menu removeItemAtIndex:i];
			break;
		}
	}
	
	// return if do not to create shortcuts
	if(![[[connectionArray selection] valueForKey:@"deskIntegration"] doCreateInApps])
		return;
	
    if (![farmsConnected count])
        [menu addItem:[NSMenuItem separatorItem]];
    
    if (![farmsConnected valueForKey: [[connectionArray selection] valueForKey:@"farmName"]])
        [farmsConnected setObject: [NSNull null] forKey: [[connectionArray selection] valueForKey:@"farmName"]];
    
	// add farm submenu to menu
	NSMenuItem *menuFarmItem = [menu addItemWithTitle:[[connectionArray selection] valueForKey:@"farmName"] action:nil keyEquivalent:@""];
	[menuFarmItem setTarget:self];
	[menuFarmItem setTag:[connectionArray selectionIndex]];
	NSMenu *currFarmSubMenu = [[NSMenu alloc] init];
	[menu setSubmenu:currFarmSubMenu forItem:menuFarmItem];
	[currFarmSubMenu setTitle:[[connectionArray selection] valueForKey:@"farmName"]];
		
	// add apps to submenu
	NSMenuItem *menuItem;
	NSArray *appsArrToAddToTray = [NSArray arrayWithArray:[[connectionArray selection] valueForKey:@"launchedApps"]];
	for(int i=0; i!= [appsArrToAddToTray count]; ++i)
	{
		// Add to items
		menuItem = [currFarmSubMenu addItemWithTitle:[[appsArrToAddToTray objectAtIndex:i] appName] action:@selector (doConnectApp:) keyEquivalent:@""];
		[menuItem setTarget:self];	
	}
	[currFarmSubMenu release];
}

#pragma mark -
#pragma mark Actions

- (IBAction)manageConnection:(id)sender
{
	//SetSystemUIMode(kUIModeAllHidden, kUIOptionAutoShowMenuBar);

	[manageConnectionWindow showModal:window sender:sender];

	//SetSystemUIMode(kUIModeNormal, 0);
}

- (IBAction)closeApp:(id)sender{
	[NSApp terminate:self];
}

- (IBAction)doConnectFarm:(id)sender {
	if (![[[connectionArray selection] valueForKey:@"credentials"] doSave])
	{
		[usernameField setStringValue:@""];
		[passwordField setStringValue:@""];
		[domainField setStringValue:@""];
		NSTextFieldCell *usernameCell = [usernameField cell];
		[usernameCell setPlaceholderString:@""];
		NSTextFieldCell *passwordCell = [passwordField cell];
		[passwordCell setPlaceholderString:@""];
		NSTextFieldCell *domainCell = [domainField cell];
		[domainCell setPlaceholderString:@""];
		
		if([NSApp runModalForWindow:logonAsDifUserPanel] == NSRunAbortedResponse)
		return;
	}
	[self doGetAppList];
#ifdef ENABLE_DESKTOP_INTEGRATION
	[self addIAppsToTray];
#else
	if([btnRefresh state] == 0)
	{
		[btnChangeLocation setEnabled:YES];
		[btnLogonAsDifUser setEnabled:YES];
		[btnRefresh setEnabled:YES];
	}
#endif
}

- (IBAction)doConnectApp:(id)sender {
#ifdef ENABLE_DESKTOP_INTEGRATION
    if ([[[connectionArray selection] valueForKey:@"deskIntegration"] doCreateInApps])
    {
        NSMenu *currFarmSubmenu = [sender menu];
        
        if (currFarmSubmenu)
        {
            NSMenu *mainMenu = [currFarmSubmenu supermenu];
            NSString *currFarmSubmenuTitle = [NSString stringWithString:[currFarmSubmenu title]];
            NSUInteger farmSubmenuIndex = [[mainMenu itemWithTitle:currFarmSubmenuTitle] tag];
    
            [connectionArray setSelectionIndex:farmSubmenuIndex];
            
            NSUInteger appIndex = [currFarmSubmenu indexOfItemWithTitle:[sender title]];
            
            [launchedAppArray setSelectionIndex:appIndex];
        }
    }
#endif
	[manageConnectionWindow launchApp];
}

- (IBAction)doChangePass:(id)sender {
	[usrName setStringValue:@""];
	[[usrName cell] setPlaceholderString:@""];
	[domain setStringValue:@""];
	[[domain cell] setPlaceholderString:@""];
	[oldPass setStringValue:@""];
	[[oldPass cell] setPlaceholderString:@""];
	[newPass setStringValue:@""];
	[[newPass cell] setPlaceholderString:@""];
	[confirmPass setStringValue:@""];
	[[confirmPass cell] setPlaceholderString:@""];

    [changePassPanel setWorksWhenModal:YES];
	[changePassPanel makeKeyAndOrderFront: sender];

	if([NSApp runModalForWindow:changePassPanel] == NSRunAbortedResponse)
		return;
	if([self changePassword] != 0)
	{
		// TODO: show error message with pntsc error data
		[self showErrorLaunchingPanel];
		return;
	}
	else
	{
        Credentials *cr = [[connectionArray selection] valueForKey:@"credentials"];
        [[[connectionArray selection] valueForKey:@"credentials"] setPassword: [newPass stringValue]];
		[self showSucceededPanel];
	}
}


- (IBAction)doOkChangePass:(id)sender {
	[usrName selectText:self];
	[domain selectText:self];
	[oldPass selectText:self];
	[newPass selectText:self];
	[confirmPass selectText:self];
	if(![[usrName stringValue] isEqualToString:[[[connectionArray selection] valueForKey:@"credentials"] username]]  ||
	   [[usrName stringValue] isEqualToString:@""])
	{
		[meaningMessage setStringValue:@"Please enter correct user name"];
		[NSApp runModalForWindow:enterCorrectvalueDialog];
		return;
	}
	if(![[domain stringValue] isEqualToString:[[[connectionArray selection] valueForKey:@"credentials"] domain]] ||
	   [[domain stringValue] isEqualToString:@""])
	{
		[meaningMessage setStringValue:@"Please enter correct domain"];
		[NSApp runModalForWindow:enterCorrectvalueDialog];
		return;
	}
	if(![[oldPass stringValue] isEqualToString:[[[connectionArray selection] valueForKey:@"credentials"] password]] ||
		[[oldPass stringValue] isEqualToString:@""])
	{
		[meaningMessage setStringValue:@"Please enter correct old password"];
		[NSApp runModalForWindow:enterCorrectvalueDialog];
		return;
	}
	if([[oldPass stringValue] isEqualToString:@""] || [[newPass stringValue] isEqualToString:@""])
	{
		[meaningMessage setStringValue:@"Please enter new password"];
		[NSApp runModalForWindow:enterCorrectvalueDialog];
		return;
	}
	if (![[newPass stringValue] isEqualToString:[confirmPass stringValue]])
	{
		[meaningMessage setStringValue:@"New passwords do not match"];
		[NSApp runModalForWindow:enterCorrectvalueDialog];
		return;
	}
	
	[self closePanel:changePassPanel];
}

- (IBAction)doCancelChangePass:(id)sender {
	[NSApp abortModal];
	[changePassPanel close];
}

- (IBAction)doOkConfirmCorrectDialog:(id)sender {
	[self closePanel:enterCorrectvalueDialog];
}

- (IBAction)doLogonAsDifUser:(id)sender {
	if([NSApp runModalForWindow:logonAsDifUserPanel] == NSRunAbortedResponse)
		return;
	if(![[[connectionArray selection] valueForKey:@"credentials"] doSave])
	{
		[usernameField setStringValue:@""];
		[passwordField setStringValue:@""];
		[domainField setStringValue:@""];
	}
	[NSApp runModalForWindow:logonAsDifUserPanel];
	[self doGetAppList];
}

- (IBAction)doRefresh:(id)sender {
	if (![[[connectionArray selection] valueForKey:@"credentials"] doSave])
		[NSApp runModalForWindow:logonAsDifUserPanel];
	[self doGetAppList];
}

- (IBAction)doChangeLocation:(id)sender {
	[NSApp runModalForWindow:changeLocationPanel];
}

- (IBAction)doConnectFromChangeLoc:(id)sender {
	[self closePanel:changeLocationPanel];
	[[[connectionArray selectedObjects] objectAtIndex:0] setCurrentLocationIndex:[locationArray selectionIndex]];
}

- (IBAction)doCancelChangeLocation:(id)sender {
	[NSApp abortModal];
	[changeLocationPanel close];
}

- (IBAction)doViewShortcuts:(id)sender {
    [NSApp beginSheet:viewShortcutsPanel modalForWindow:window modalDelegate:self didEndSelector:nil contextInfo:nil];
	//[NSApp runModalForWindow:viewShortcutsPanel];
}

- (IBAction)doCloseViewShortcuts:(id)sender {
    [NSApp endSheet:viewShortcutsPanel];
	[viewShortcutsPanel close];
    //[self closePanel: viewShortcutsPanel];
}

- (IBAction)doShowDeskIntegrationWindow:(id)sender {
	//[NSApp runModalForWindow:deskIntegrationWindow];
    [deskIntegrationWindow makeKeyAndOrderFront: sender];
}

- (IBAction)doCloseDeskIntegrationWindow:(id)sender {
	[self closePanel:deskIntegrationWindow];
}

- (void)showErrorLaunchingPanel {
	[NSApp beginSheet:errorLaunchingPanel modalForWindow:window modalDelegate:nil didEndSelector:nil contextInfo:nil];
}

- (void)showSucceededPanel {
	[NSApp beginSheet:succeededPanel modalForWindow:window modalDelegate:nil didEndSelector:nil contextInfo:nil];
}

- (IBAction)doCloseErrMessagePanel:(id)sender {
	[NSApp endSheet:errorLaunchingPanel];
	[errorLaunchingPanel close];
}

- (IBAction)doCloseSucceededMessagePanel:(id)sender {
	[NSApp endSheet:succeededPanel];
	[succeededPanel close];
}

- (IBAction)doDisconnectFarm:(id)sender {
	[[connectionArray selection] setValue:@"Disconnected" forKey:@"farmStatus"];
    [launchedAppArray removeObjectsAtArrangedObjectIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(0, [[launchedAppArray arrangedObjects] count])]];
    
	int j = [menu numberOfItems];
    
	for(int i = 7; i< j; ++i) //7 = current submenu items number
	{
		if([[[menu itemAtIndex:i] title] isEqualToString:[[connectionArray selection] valueForKey:@"farmName"]])
		{
			[menu removeItemAtIndex:i];
            [farmsConnected removeObjectForKey: [[connectionArray selection] valueForKey:@"farmName"]];
		}
	}	
 
    j = [menu numberOfItems];
    
    if (![farmsConnected count] && [[menu itemAtIndex: j - 1] isSeparatorItem])
        [menu removeItemAtIndex: j - 1];
}

@end
