
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "ConfigSrcTabAssist.h"

@implementation ConfigSrcTabAssist

#pragma mark -
#pragma mark Initialization

static id sharedInstance = nil;

+ (id)sharedInstance {
	if (sharedInstance == nil)
		sharedInstance = [[self alloc] init];
	return sharedInstance;
}

-(id)init {
	if(sharedInstance) {
		[NSException raise:NSInternalInconsistencyException
					format:@"[%@] cannot be called; use +[%@ %@] instead",
		 NSStringFromClass([self class]), 
		 NSStringFromClass([self class]),
		 NSStringFromSelector(@selector(sharedInstance))];							 
	}
	
	else
		sharedInstance = [super init];
	return sharedInstance;
}

- (void)dealloc {
	[super dealloc];
}

#pragma mark -
#pragma mark Useful methods

- (BOOL)needToDownloadFarm {
	return ([behaviourMatrix selectedRow] == 0);
}

- (void)closeWnd:(NSWindow *) wndClose {
	[NSApp endSheet:wndClose];
	[wndClose close];
}

- (void)doDownloadFarm {
	//TODO hang this method to someone if needToDownload returns true
	[NSApp beginSheet:panelNewConfig modalForWindow:manageWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
}

- (BOOL)isValidTextField:(NSTextField *)textfield {
	NSString * typedText = [NSString stringWithString:[textfield stringValue]];
	return(![typedText isEqual:@""]);
}

- (BOOL)isDialogDataCorrect {
	if([self isValidTextField:url])
	{
		[url selectText:self];
		if([self isValidTextField:file])
		{
			[file selectText:self];
			if([self isValidTextField:proxy])
			{
				[proxy selectText:self];
				return YES;
			}
		}
	}
	return NO;
}

#pragma mark -
#pragma mark Actions

- (IBAction)okBtnNewConfiguration:(id)sender {
	//TODO download config file
	if([self isDialogDataCorrect])
		[self closeWnd:panelNewConfig];
	else
		[NSApp beginSheet:confirmDialog modalForWindow:panelNewConfig modalDelegate:self didEndSelector:nil contextInfo:nil];
}

- (IBAction)doCancelNewConfiguration:(id)sender {
	//TODO do not save fields
	[self closeWnd:panelNewConfig];
}

- (IBAction)doOk:(id)sender {
	[self closeWnd:confirmDialog];
}

#pragma mark -
#pragma mark Overloaded methods

- (void)setDefaultTabState {
}

- (BOOL)isDataCorrect {
	return YES;
}

- (void)rDParam:(NSMutableArray *)input {
}

@end
