
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

//
//  Connection.m
//  Provision AppPortal
//
//  Created by smalti on 4/10/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "Connection.h"

@implementation Connection

#pragma mark -
#pragma mark Initialization

- (id)init {
	if (self = [super init])
	{
		farmName = [[NSString alloc] initWithString:@"New Farm"];
		// flags
		doAutoLaunch = NO;
		//doPromptLocation = NO;
		doReconnect = NO;
		doUsePasswordManagement = NO;
		isIntegratedMode = NO;
		
		locations = [[NSMutableArray alloc] init];
		currentLocationIndex = -1;
		
		credentials  = [[Credentials alloc] init];
		displaySettings = [[Display alloc] init];
		resources = [[Resources alloc] init];
		performance = [[ConnectionPerformance alloc] init];
		passManager = [[PasswordManagementServer alloc] init];
		deskIntegration = [[DesktopIntegration alloc] init];
		apps = [[NSMutableArray alloc] init];
		
		launchedApps = [[NSMutableArray alloc] init];
		
		farmStatus = [[NSString alloc] initWithString:@"Disconnected"];
	}
	return self;
}

- (id)initWithConnection:(Connection *)input {
	if(self = [super init])
	{
		[self setFarmName:[input farmName]];
		
		// flags
		[self setDoAutoLaunch:[input doAutoLaunch]];
		//[self setDoPromptLocation:[input doPromptLocation]];
		[self setDoReconnect:[input doReconnect]];
		[self setDoUsePasswordManagement:[input doUsePasswordManagement]];
		[self setIsIntegratedMode:[input isIntegratedMode]];
        [self setSpanMonitors: [input spanMonitors]];
		
		[self setLocations:[input locations]];
		[self setCurrentLocationIndex:[input currentLocationIndex]];
		[self setCredentials:[input credentials]];
		[self setDisplaySettings:[input displaySettings]];
		[self setResources:[input resources]];
		[self setPerformance:[input performance]];
		[self setPassManager:[input passManager]];
		[self setDeskIntegration:[input deskIntegration]];
		[self setApps:[input apps]];
		[self setLaunchedApps:[input launchedApps]];
		
	}
	return self;
}

- (void)dealloc {
	[farmName release];
	[locations release];
	
	[credentials release];
	[displaySettings release];
	[resources release];
	[performance release];
	[passManager release];
	[deskIntegration release];
	[apps release];
	
	[launchedApps release];
	
	[farmStatus release];

	[super dealloc];
}

#pragma mark -
#pragma mark Setters

- (void)setFarmName:(NSString *)input {
	[farmName autorelease];
	if(input == nil)
	{
		farmName = [[NSString alloc] initWithString:@"New Farm"];
	}
	else
	{
		farmName = [input retain];
	}
}

// flags
- (void)setDoAutoLaunch:(BOOL)input {
	doAutoLaunch = input;
}

- (void)setDoReconnect:(BOOL)input {
	doReconnect = input;
}

- (void)setDoUsePasswordManagement:(BOOL)input {
	doUsePasswordManagement = input;
}

- (void)setIsIntegratedMode:(BOOL)input {
	isIntegratedMode = input;
}

- (void)setSpanMonitors:(BOOL)input {
    spanMonitors = input;
}

// end of flags

- (void)setLocations:(NSArray *)input {
	[locations autorelease];
	if(input == nil)
	{
		locations = [[NSMutableArray alloc] init];
	}
	else
	{
		locations = [[NSMutableArray alloc] initWithArray:input];
	}
}

- (void)setCurrentLocationIndex:(int)input {
	currentLocationIndex = input;
}

- (void)setCredentials:(Credentials *)input {
	[credentials autorelease];
	if(input == nil)
	{
		credentials  = [[Credentials alloc] init];
	}
	else// ftopku if(credentials != input)
	{
		credentials = [[Credentials alloc] initWithCredentials:input];
	}
}

- (void)setDisplaySettings:(Display *)input {
	[displaySettings autorelease];
	if(input == nil)
		displaySettings = [[Display alloc] init];
	else
		displaySettings = [[Display alloc] initWithDisplay:input];
}

- (void)setResources:(Resources *)input {
	[resources autorelease];
	if(input == nil)
		resources = [[Resources alloc] init];
	else
		resources = [[Resources alloc] initWithResource:input];
}

- (void)setPerformance:(ConnectionPerformance *)input {
	[performance autorelease];
	if(input == nil)
		performance = [[ConnectionPerformance alloc] init];
	else
		performance = [[ConnectionPerformance alloc] initWithPerformance:input];
}

- (void)setPassManager:(PasswordManagementServer *)input {
	[passManager autorelease];
	if(input == nil)
		passManager = [[PasswordManagementServer alloc] init];
	else
		passManager = [[PasswordManagementServer alloc] initWithPasswordManagementServer:input];
}

- (void)setDeskIntegration:(DesktopIntegration *)input {
	[deskIntegration autorelease];
	if(input == nil)
		deskIntegration = [[DesktopIntegration alloc] init];
	else
		deskIntegration = [[DesktopIntegration alloc] initWithDesktopIntegration:input];
}

- (void)setApps:(NSArray *)input {
	[apps autorelease];
	if(input == nil)
		apps = [[NSMutableArray alloc] init];
	else
		apps = [[NSMutableArray alloc] initWithArray:input];
}

- (void)setLaunchedApps:(NSArray *)input {
	[launchedApps autorelease];
	if(input == nil)
		launchedApps = [[NSMutableArray alloc] init];
	else
		launchedApps = [[NSMutableArray alloc] initWithArray:input];
}

- (void)setFarmStatus:(NSString *)input {
	[farmStatus autorelease];
	if(input == nil)
	{
		farmStatus = [[NSString alloc] initWithString:@"Disconnected"];
	}
	else
	{
		farmStatus = [input retain];
	}
}


#pragma mark -
#pragma mark Getters

- (NSString *)farmName {
	return farmName;
}

// flags
- (BOOL)doAutoLaunch {
	return doAutoLaunch;
}

- (BOOL) doReconnect {
	return doReconnect;
}

- (BOOL) doUsePasswordManagement {
	return doUsePasswordManagement;
}

- (BOOL) isIntegratedMode {
	return isIntegratedMode;
}

- (BOOL) spanMonitors {
    return spanMonitors;
}

// end of flags

- (NSMutableArray *)locations {
	return locations;
}

- (int)currentLocationIndex {
	return currentLocationIndex;
}

- (Credentials *)credentials {
	return credentials;
}

- (Display *)displaySettings {
	return displaySettings;
}

- (Resources *)resources {
	return resources;
}

- (ConnectionPerformance *)performance {
	return performance;
}

- (PasswordManagementServer *)passManager {
	return passManager;
}

- (DesktopIntegration *)deskIntegration {
	return deskIntegration;
}

- (NSMutableArray *)apps {
	return apps;
}

- (NSMutableArray *)launchedApps {
	return launchedApps;
}

- (NSString *)farmStatus {
	return farmStatus;
}

#pragma mark -
#pragma mark Encode-Decode

- (void)encodeWithCoder:(NSCoder *)coder {
	[coder encodeObject:farmName forKey:@"farmName"];
	// booleans
	[coder encodeBool:doAutoLaunch forKey:@"doAutoLaunch"];
	//[coder encodeBool:doPromptLocation forKey:@"doPromptLocation"];
	[coder encodeBool:doReconnect forKey:@"doReconnect"];
	[coder encodeBool:doUsePasswordManagement forKey:@"doUsePasswordManagement"];
	[coder encodeBool:isIntegratedMode forKey:@"isIntegratedMode"];
    [coder encodeBool:spanMonitors forKey:@"spanMonitors"];
	
	[coder encodeObject:locations forKey:@"locations"];
	for(int i=0; i!= [locations count]; ++i)
	{
		if([[locations objectAtIndex:i] doUseAsDefault])
		{
			[self setCurrentLocationIndex:i];
//			[coder encodeInt:currentLocationIndex forKey:@"currentLocationIndex"];
			break;
		}
	}
	
	[coder encodeObject:credentials forKey:@"credentials"];
	[coder encodeObject:displaySettings forKey:@"displaySettings"];
	[coder encodeObject:resources forKey:@"resources"];
	[coder encodeObject:performance forKey:@"performance"];
	[coder encodeObject:passManager forKey:@"passManager"];
	[coder encodeObject:deskIntegration forKey:@"deskIntegration"];
	[coder encodeObject:apps forKey:@"apps"];
	
//#ifdef ENABLE_DESKTOP_INTEGRATION
//	[self setFarmStatus:@"Disconnected"];
//	[coder encodeObject:farmStatus forKey:@"farmStatus"];
//#endif
}

- (id)initWithCoder:(NSCoder *)coder {
	if(self = [super init])
	{
		[self setFarmName:[coder decodeObjectForKey:@"farmName"]];
		// booleans
		[self setDoAutoLaunch:[coder decodeBoolForKey:@"doAutoLaunch"]];
		//[self setDoPromptLocation:[coder decodeBoolForKey:@"doPromptLocation"]];
		[self setDoReconnect:[coder decodeBoolForKey:@"doReconnect"]];
		[self setDoUsePasswordManagement:[coder decodeBoolForKey:@"doUsePasswordManagement"]];
		[self setIsIntegratedMode:[coder decodeBoolForKey:@"isIntegratedMode"]];
        [self setSpanMonitors:[coder decodeBoolForKey:@"spanMonitors"]];
		
		[self setLocations:[coder decodeObjectForKey:@"locations"]];
		currentLocationIndex = -1;
		for(int i=0; i!= [locations count]; ++i)
		{
			if([[locations objectAtIndex:i] doUseAsDefault])
			{
				[self setCurrentLocationIndex:i]; //[coder decodeIntForKey:@"currentLocationIndex"]];
				break;
			}
		}
		
		[self setCredentials:[coder decodeObjectForKey:@"credentials"]];
		[self setDisplaySettings:[coder decodeObjectForKey:@"displaySettings"]];
		[self setResources:[coder decodeObjectForKey:@"resources"]];
		[self setPerformance:[coder decodeObjectForKey:@"performance"]];
		[self setPassManager:[coder decodeObjectForKey:@"passManager"]];
		[self setDeskIntegration:[coder decodeObjectForKey:@"deskIntegration"]];
		[self setApps:[coder decodeObjectForKey:@"apps"]];
		[self setLaunchedApps:nil];
	#ifdef ENABLE_DESKTOP_INTEGRATION
		[self setFarmStatus:[coder decodeObjectForKey:@"farmStatus"]];
	#endif
	}
	return self;
}

@end
