
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

//
//  ConnectionPerformance.m
//  Provision AppPortal
//
//  Created by smalti on 4/10/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "ConnectionPerformance.h"


@implementation ConnectionPerformance

#pragma mark -
#pragma mark Initialization

- (id)init {
	if(self = [super init])
	{
		connectionSpeed = [[NSString alloc] initWithString:@"Modem (28.8 Kbps)"];
		doUseBitmapCaching = YES;
		doShowDesktopBackground = NO;
		doUseAnimation = NO;
		doShowWindowContent = NO;
		doUseThemes = NO;
		doUseGA = NO;
        doFlashMMR = NO;
	}
	return self;
}

- (id)initWithPerformance:(const ConnectionPerformance *)input {
	if(self = [super init])
	{
		[self setConnectionSpeed:[input connectionSpeed]];
		[self setDoUseBitmapCaching:[input doUseBitmapCaching]];
		[self setDoShowDesktopBackground:[input doShowDesktopBackground]];
		[self setDoUseAnimation:[input doUseAnimation]];
		[self setDoShowWindowContent:[input doShowWindowContent]];
		[self setDoUseThemes:[input doUseThemes]];
		[self setDoUseGA:[input doUseGA]];
        [self setDoFlashMMR:[input doFlashMMR]];
	}
	return self;
}

- (void)dealloc {
	[connectionSpeed release];
	[super dealloc];
}

#pragma mark -
#pragma mark Setters

- (void)setConnectionSpeed:(NSString *)input {
	[connectionSpeed autorelease];
	connectionSpeed = [input retain];
}

- (void)setDoUseBitmapCaching:(BOOL)input {
	doUseBitmapCaching = input;
}

- (void)setDoShowDesktopBackground:(BOOL)input {
	doShowDesktopBackground = input;
}

- (void)setDoUseAnimation:(BOOL)input {
	doUseAnimation = input;
}

- (void)setDoShowWindowContent:(BOOL)input {
	doShowWindowContent = input;
}

- (void)setDoUseThemes:(BOOL)input {
	doUseThemes = input;
}

- (void)setDoUseGA:(BOOL)input {
	doUseGA = input;
}

- (void)setDoFlashMMR:(BOOL)input {
	doFlashMMR = input;
}

#pragma mark -
#pragma mark Getters

- (NSString *)connectionSpeed {
	return connectionSpeed;
}

- (BOOL)doUseBitmapCaching {
	return doUseBitmapCaching;
}

- (BOOL)doShowDesktopBackground {
	return doShowDesktopBackground;
}

- (BOOL)doUseAnimation {
	return doUseAnimation;
}

- (BOOL)doShowWindowContent {
	return doShowWindowContent;
}

- (BOOL)doUseThemes {
	return doUseThemes;
}

- (BOOL)doUseGA {
	return doUseGA;
}

- (BOOL)doFlashMMR {
	return doFlashMMR;
}

#pragma mark -
#pragma mark Encode-Decode

- (void)encodeWithCoder:(NSCoder *)coder {
	[coder encodeObject:connectionSpeed forKey:@"connectionSpeed"];
	// booleans
	[coder encodeBool:doUseBitmapCaching forKey:@"doUseBitmapCaching"];
	[coder encodeBool:doShowDesktopBackground forKey:@"doShowDesktopBackground"];
	[coder encodeBool:doUseAnimation forKey:@"doUseAnimation"];
	[coder encodeBool:doShowWindowContent forKey:@"doShowWindowContent"];
	[coder encodeBool:doUseThemes forKey:@"doUseThemes"];
	[coder encodeBool:doUseGA forKey:@"doUseGA"];
    [coder encodeBool:doFlashMMR forKey:@"doFlashMMR"];
}

- (id)initWithCoder:(NSCoder *)coder {
	if(self = [super init])
	{
		[self setConnectionSpeed:[coder decodeObjectForKey:@"connectionSpeed"]];
		
		// booleans
		[self setDoUseBitmapCaching:[coder decodeBoolForKey:@"doUseBitmapCaching"]];
		[self setDoShowDesktopBackground:[coder decodeBoolForKey:@"setDoShowDesktopBackground"]];
		[self setDoUseAnimation:[coder decodeBoolForKey:@"doUseAnimation"]];
		[self setDoShowWindowContent:[coder decodeBoolForKey:@"setDoShowWindowContent"]];
		[self setDoUseThemes:[coder decodeBoolForKey:@"doUseThemes"]];
		[self setDoUseGA:[coder decodeBoolForKey:@"doUseGA"]];
        [self setDoFlashMMR:[coder decodeBoolForKey:@"doFlashMMR"]];
	}
	return self;
}

@end
