
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "ConnectivityTabAssist.h"

@implementation ConnectivityTabAssist

#pragma mark -
#pragma mark Initialization

static id sharedInstance = nil;

+ (id)sharedInstance {
	if (sharedInstance == nil)
		sharedInstance = [[self alloc] init];
	return sharedInstance;
}

-(id)init {
	if(sharedInstance) {
		[NSException raise:NSInternalInconsistencyException
					format:@"[%@] cannot be called; use +[%@ %@] instead",
		 NSStringFromClass([self class]), 
		 NSStringFromClass([self class]),
		 NSStringFromSelector(@selector(sharedInstance))];							 
	}
	else
		sharedInstance = [super init];

	return sharedInstance;
}

- (void)initGUI {
	NSMutableCharacterSet *characterSet = [[NSMutableCharacterSet alloc] init];
	[characterSet addCharactersInString:@"0123456789"];
	[port setFormatter:[[[PartialFormatter alloc] init] autorelease]];
	[[port formatter] setMaximumValue:65535];
	[[port formatter] setAcceptableCharacterSet:characterSet];
	[characterSet release];
	
	NSSecureTextFieldCell *cell = [password cell];
//	[cell setEchosBullets:NO];
	[password setNeedsDisplay:YES];
}

- (void)dealloc {
	[super dealloc];
}

#pragma mark-
#pragma mark Useful methods

- (void)closeWnd:(NSWindow *) wndClose {
	[NSApp endSheet:wndClose];
	[wndClose close];
}

- (BOOL)isValidTextField:(NSTextField *)textfield {
	NSString * typedText = [NSString stringWithString:[textfield stringValue]];
	return(![typedText isEqual:@""]);
}

- (int)testConnection{
	// setting up the task arguments
	NSMutableArray *args = [NSMutableArray array];
	
	// this is a crutch, needed because there may be no default location yet at this point
	//TODO: add check for default location in locations array
	
	// setting index of the current location
	[[[connectionArray selectedObjects] objectAtIndex:0] setCurrentLocationIndex:[locationArray selectionIndex]];
	
	[args addObject:[self composeBrokersString]];
	// reset index of the current location
	[[[connectionArray selectedObjects] objectAtIndex:0] setCurrentLocationIndex:-1];

	m_assistObserver = [CredentialsTabAssist sharedInstance];
	[m_assistObserver rDParam:args];
	[args addObject:@"-L"];
	
	NSMutableData *data = [NSMutableData dataWithLength:0];
	NSMutableData *errData = [NSMutableData dataWithLength:0];

	// create and launching pntsc task
	PntscTask *pntscT = [[PntscTask alloc] init];
	int testConnectStatus = [pntscT launchPntscWithArgs:args outData:data errorData:errData];
	[pntscT release];
	
	return testConnectStatus;
}

- (void)showTestConnectResult {
	if([self testConnection] == 0)
	{
		[testConnectionStatus setStringValue:@"Connection succeeded."];
		[NSApp beginSheet:testConnectionStatusPanel modalForWindow:manageWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
	}
	else
	{
		[testConnectionStatus setStringValue:@"Connection failed."];
		[NSApp beginSheet:testConnectionStatusPanel modalForWindow:manageWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
	}	
}

- (NSString *)composeBrokersString {
	int locationIndex = [[[connectionArray selectedObjects] objectAtIndex:0] currentLocationIndex];
	Location *curLocation = [[Location alloc] initWithLocation:
							 [[[connectionArray selection] valueForKey:@"locations"] objectAtIndex:locationIndex]
							 ];	
	NSString *proxyStr = [NSString stringWithString:[curLocation proxyServer]];
	// compose exclusive brokers array
	NSArray *excBrokersArr = [NSArray arrayWithArray:[[curLocation exclusiveServers] componentsSeparatedByString:@";"]];
	
	NSString *brokersList = [NSString string];
	BOOL needToAddProxy = YES;
	int numBrokers = [[curLocation brokers] count];
	if(numBrokers == 0)
		return brokersList;
	
	for (int i = 0; i < numBrokers; ++i)
	{
		if(i>0)
			brokersList = [brokersList stringByAppendingString:@","];
		brokersList = [[[[[brokersList stringByAppendingString:[[curLocation protocol] lowercaseString]]
						 stringByAppendingString:@"://"]
						stringByAppendingString:[[[curLocation brokers] objectAtIndex:i] address]]
					   stringByAppendingString:@":"]
					   stringByAppendingString:[curLocation port]];
		if(![curLocation doUseProxy])
			continue;
		for(int j=0; j < [excBrokersArr count]; ++j)
		{
			if([[[[curLocation brokers] objectAtIndex:i] address] isEqualToString:[excBrokersArr objectAtIndex:j]])
				needToAddProxy = NO;
		}
		if(needToAddProxy)
			brokersList = [[brokersList stringByAppendingString:@"PROXY"] stringByAppendingString:proxyStr];
		needToAddProxy = YES;
	}
	[curLocation release];
	return brokersList;
}

#pragma mark -
#pragma mark Actions

- (IBAction)setLocationAsDefault:(id)sender {
	NSArray *curLocationsArr = [NSArray arrayWithArray:[locationArray arrangedObjects]];
	
	BOOL defaultStatus = [[[locationArray selectedObjects] objectAtIndex:0] doUseAsDefault];
	int i=0;
	for(; i != [curLocationsArr count]; ++i)
	{
		if([[curLocationsArr objectAtIndex:i] doUseAsDefault])
		{
			[[curLocationsArr objectAtIndex:i] setDoUseAsDefault:NO];
		}
	}
	[[[locationArray selectedObjects] objectAtIndex:0] setDoUseAsDefault:defaultStatus];
}

- (IBAction)doAddBroker:(id)sender {
	[brokerAddressToAdd setStringValue:@""];
	[NSApp beginSheet:panelAddBroker modalForWindow:manageWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
}

- (IBAction)doCancelAddBroker:(id)sender {
	[self closeWnd:panelAddBroker];
}

- (IBAction)okAddBroker:(id)sender {
	if([self isValidTextField:brokerAddressToAdd])
	{
		[brokerAddressToAdd selectText:self];
		Broker *brokerItem = [[Broker alloc] init];
		[brokerItem setValue:[brokerAddressToAdd stringValue] forKey:@"address"];
		[brokerArray addObject:brokerItem];
		[brokerItem release];
		[self closeWnd:panelAddBroker];
	}
	else
		[NSApp beginSheet:confirmDialog modalForWindow:panelAddBroker modalDelegate:self didEndSelector:nil contextInfo:nil];
}

- (IBAction)doCancelRenameLocation:(id)sender {
	[self closeWnd:panelRenameLocation];
}

- (IBAction)okRenameLocation:(id)sender {
	if([self isValidTextField:locationName])
	{
		[locationName selectText:self];
		[[locationArray selection] setValue:[locationName stringValue] forKey:@"name"];
		[self closeWnd:panelRenameLocation];
	}
	else
		[NSApp beginSheet:confirmDialog modalForWindow:panelRenameLocation modalDelegate:self didEndSelector:nil contextInfo:nil];
}

- (IBAction)doCancelTestConnect:(id)sender {
	[NSApp abortModal];
	[panelTestConnection close];
}

- (IBAction)okTestConnect:(id)sender {
	if ([self isValidTextField:username] && [self isValidTextField:password] && [self isValidTextField:domain])
	{
		[username selectText:self];
		[password selectText:self];
		[domain selectText:self];
		[NSApp stopModal];
		[panelTestConnection close];
	}
	else
		[NSApp beginSheet:confirmDialog modalForWindow:panelTestConnection modalDelegate:self didEndSelector:nil contextInfo:nil];
}

- (IBAction)okConfirmDialog:(id)sender {
	[self closeWnd:confirmDialog];
}

- (IBAction)doMoveBrokerEntryDown:(id)sender {
	int selIndex = [brokerArray selectionIndex];
	if(selIndex < 0 || selIndex >= [[brokerArray arrangedObjects] count] - 1)
		return;
	Broker *brokerToMove = [[Broker alloc] initWithBroker:[[brokerArray selectedObjects] objectAtIndex:0]];
	int indexToInsert = selIndex + 1;
	[brokerArray setSelectionIndex:indexToInsert];
	Broker *brokerWhereMove = [[Broker alloc] initWithBroker:[[brokerArray selectedObjects] objectAtIndex:0]];
	[brokerArray removeObjectAtArrangedObjectIndex:indexToInsert];
	[brokerArray removeObjectAtArrangedObjectIndex:selIndex];
	[brokerArray insertObject:brokerWhereMove atArrangedObjectIndex:selIndex];
    [brokerArray insertObject:brokerToMove atArrangedObjectIndex:indexToInsert];
	[brokerArray setSelectionIndex:indexToInsert];
	[brokerToMove release];
	[brokerWhereMove release];
	[brokersTable reloadData];
}

- (IBAction)doMoveBrokerEntryUp:(id)sender {
	int selIndex = [brokerArray selectionIndex];
	if(selIndex <= 0)
		return;
	Broker *brokerToMove = [[Broker alloc] initWithBroker:[[brokerArray selectedObjects] objectAtIndex:0]];
	int indexToInsert = selIndex - 1;
	[brokerArray setSelectionIndex:indexToInsert];
	Broker *brokerWhereMove = [[Broker alloc] initWithBroker:[[brokerArray selectedObjects] objectAtIndex:0]];
	[brokerArray removeObjectAtArrangedObjectIndex:selIndex];
	[brokerArray removeObjectAtArrangedObjectIndex:indexToInsert];
    [brokerArray insertObject:brokerToMove atArrangedObjectIndex:indexToInsert];
	[brokerArray insertObject:brokerWhereMove atArrangedObjectIndex:selIndex];
	[brokerArray setSelectionIndex:indexToInsert];
	[brokerToMove release];
	[brokerWhereMove release];
	[brokersTable reloadData];
}

- (IBAction)doRenamelocation:(id)sender {
	[locationName setStringValue:@""];
	[NSApp beginSheet:panelRenameLocation modalForWindow:manageWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
}

- (IBAction)doTestConnect:(id)sender {
	[port selectText:self];
	if ([[port stringValue] isEqualToString:@""])
	{
		[testConnectionStatus setStringValue:@"Please enter port value"];
		[NSApp beginSheet:testConnectionStatusPanel modalForWindow:manageWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
		return;
	}
	if (![[[connectionArray selection] valueForKey:@"credentials"] doSave])
	{
		[testConCredFarmName setStringValue:[[connectionArray selection] valueForKey:@"farmName"]];
		[username setStringValue:@""];
		[domain setStringValue:@""];
		[password setStringValue:@""];
		if([NSApp runModalForWindow:panelTestConnection] == NSRunAbortedResponse)
			return;
	}
	else
	{
		m_assistObserver = [CredentialsTabAssist sharedInstance];
		if(![m_assistObserver isDataCorrect])
		{
			[testConnectionStatus setStringValue:@"Please enter correct credentials"];
			[NSApp beginSheet:testConnectionStatusPanel modalForWindow:manageWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
			return;
		}
	}
	[self showTestConnectResult];
}

- (IBAction)okTestConnectStatusPanel:(id)sender {
	[self closeWnd:testConnectionStatusPanel];
}

#pragma mark -
#pragma mark Overloaded methods

- (void)setDefaultTabState {
}

- (BOOL)isDataCorrect {
	if([self isValidTextField:port])
	{
		[port selectText:self];
		if([brokerArray canRemove])
		{			
			return YES;
		}
	}
	return NO;
}

- (void)rDParam:(NSMutableArray *)input {
	int locationIndex = [[[connectionArray selectedObjects] objectAtIndex:0] currentLocationIndex];
	if(locationIndex == -1)
		if ([NSApp runModalForWindow:promptLocationPanel] == NSRunAbortedResponse)
			return;
	
	locationIndex = [[[connectionArray selectedObjects] objectAtIndex:0] currentLocationIndex];
//	if(locationIndex == -1)
//		return;
	Location *curLocation = [[Location alloc] initWithLocation:
							 [[[connectionArray selection] valueForKey:@"locations"] objectAtIndex:locationIndex]
							 ];
	
	// Here we form a list of broker addresses that has the following format:
	// <protocol>://<broker1:portPROXYaddress:port>[,<broker2>,...,<brokerN>]
	// // old format <protocol>://<broker1>[,<broker2>,...,<brokerN>][:port]
	// where protocol is either "http" or "https" 
	
	int numBrokers = [[curLocation brokers] count];
	if(numBrokers == 0)
	{
		[input addObject:@"Empty"];
		[curLocation release];
		return;
	}
	
	[input addObject:[self composeBrokersString]];
	[curLocation release];
}

@end
