
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "CredentialsTabAssist.h"

@implementation CredentialsTabAssist

#pragma mark -
#pragma mark Initialization

static id sharedInstance = nil;

+ (id)sharedInstance {
	if (sharedInstance == nil)
		sharedInstance = [[self alloc] init];
	return sharedInstance;
}

-(id)init {
	if(sharedInstance) {
		[NSException raise:NSInternalInconsistencyException
					format:@"[%@] cannot be called; use +[%@ %@] instead",
		NSStringFromClass([self class]), 
		NSStringFromClass([self class]),
		NSStringFromSelector(@selector(sharedInstance))];							 
	}
	else
	{
		sharedInstance = [super init];
	}
	return sharedInstance;
}

- (void)dealloc {
	[super dealloc];
}

- (void)initGUI {
	NSSecureTextFieldCell *cell = [password cell];
//	[cell setEchosBullets:NO];
	[password setNeedsDisplay:YES];
}

#pragma mark -
#pragma mark Useful methods

- (BOOL)isValidTextField:(NSTextField *)textfield {
	NSString * typedText = [NSString stringWithString:[textfield stringValue]];
	return(![typedText isEqual:@""]);
}

#pragma mark -
#pragma mark Actions

- (IBAction)doUseCachedCredentials:(id)sender {
    // may be wiil be implemented if there is any specification describes detailed how should this work
}

#pragma mark -
#pragma mark Overloaded methods

- (void)setDefaultTabState {
	id currentConnection = [connectionArray selection];
	if (currentConnection == NSNoSelectionMarker || 
		currentConnection == NSMultipleValuesMarker ||
		currentConnection == NSNotApplicableMarker)
		return;
	@try {
		if(![[currentConnection valueForKey:@"credentials"] doSave])
		{
			[username setStringValue:@""];
			[password setStringValue:@""];
			[domain setStringValue:@""];
		}
	}
	@catch (NSException * e) {
		// do nothing - it just means there're no connections in the array controller
	}
}


- (BOOL)isDataCorrect {
	if([self isValidTextField:username])
	{
		[username selectText:self];
		if([self isValidTextField:password])
		{
			[password selectText:self];
			if([self isValidTextField:domain])
			{
				[domain selectText:self];
				return YES;
			}
		}
	}
	return NO;
}

- (void)rDParam:(NSMutableArray *)input {
	NSString *thirdArg = [NSString stringWithString:
						  [[[connectionArray selection] valueForKey:@"credentials"] username]
						  ];
	NSString *fithArg = [NSString stringWithString:
						 [[[connectionArray selection] valueForKey:@"credentials"] password]
						 ];
	NSString *seventhArg = [NSString stringWithString:
							[[[connectionArray selection] valueForKey:@"credentials"] domain]
							];
	[input addObject:@"-u"];
	[input addObject:thirdArg];
	[input addObject:@"-p"];
	[input addObject:fithArg];
	[input addObject:@"-d"];
	[input addObject:seventhArg];
}

@end
