
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "DisplayTabAssist.h"
#import <Carbon/Carbon.h>

@implementation DisplayTabAssist

#pragma mark -
#pragma mark Initialization

static id sharedInstance = nil;

+ (id)sharedInstance {
	if (sharedInstance == nil)
		sharedInstance = [[self alloc] init];
	return sharedInstance;
}

-(id)init {
	if(sharedInstance) {
		[NSException raise:NSInternalInconsistencyException
					format:@"[%@] cannot be called; use +[%@ %@] instead",
		 NSStringFromClass([self class]), 
		 NSStringFromClass([self class]),
		 NSStringFromSelector(@selector(sharedInstance))];							 
	}	
	else
	{
		sharedInstance = [super init];
		
		{
			NSArray *keys = [NSArray arrayWithObjects: @"Full Screen", @"1024 by 768 pixels", @"800 by 600 pixels", @"640 by 480 pixels", @"Custom", nil];
			NSArray *values = [NSArray arrayWithObjects: @"-f", @"1024x768", @"800x600", @"640x480", @"", nil];
			resolutionsDict = [[NSMutableDictionary alloc] initWithObjects:values forKeys:keys];
		}
		{
			NSArray *keys = [NSArray arrayWithObjects: @"True Color (24 bit)", @"High Color (16 bit)", @"High Color (15 bit)", @"256 Colors", nil];
			NSArray *values = [NSArray arrayWithObjects: @"24", @"16", @"15", @"8", nil];
			colorsDict = [[NSMutableDictionary alloc] initWithObjects:values forKeys:keys];
		}		
	}
	return sharedInstance;
}

- (void)dealloc {
	[resolutionsDict release];
	[colorsDict release];
	[super dealloc];
}

- (void)initGUI {	
	NSMutableCharacterSet *characterSet = [[NSMutableCharacterSet alloc] init];
	[characterSet addCharactersInString:@"0123456789"];
	[desktopWidth setFormatter:[[[PartialFormatter alloc] init] autorelease]];
	[[desktopWidth formatter] setMaximumValue:65535];
	//[[desktopWidth formatter] setMinimumValue:1];
	[[desktopWidth formatter] setAcceptableCharacterSet:characterSet];
	[desktopHeight setFormatter:[[[PartialFormatter alloc] init] autorelease]];
	[[desktopHeight formatter] setMaximumValue:65535];
	//[[desktopHeight formatter] setMinimumValue:1];
	[[desktopHeight formatter] setAcceptableCharacterSet:characterSet];
	[characterSet release];
}

#pragma mark -
#pragma mark Useful methods

- (BOOL)isValidTextField:(NSTextField *)textfield {
	NSString * typedText = [NSString stringWithString:[textfield stringValue]];
	return(![typedText isEqual:@""]);
}

#pragma mark -
#pragma mark Actions

- (IBAction)doSelectCustom:(id)sender {
	Display *tmp = [[connectionArray selection] valueForKey:@"displaySettings"];
	if([[tmp resolution] isEqualToString:@"Custom"])
	{
		[desktopHeight setEnabled:YES];
		[desktopWidth setEnabled:YES];
	}
	else
	{
		[desktopHeight setEnabled:NO];
		[desktopWidth setEnabled:NO];
		[tmp setHeight:@""];
		[tmp setWidth:@""];
	}
}

#pragma mark -
#pragma mark Overloaded methods

- (void)setDefaultTabState {
}


- (BOOL)isDataCorrect {
	BOOL result = YES;
	Display *tmp = [[Display alloc] initWithDisplay:[[connectionArray selection] valueForKey:@"displaySettings"]];
	if([[tmp resolution] isEqualToString:@"Custom"])
	{
		if([self isValidTextField:desktopWidth] && [self isValidTextField:desktopHeight])
		{
			[desktopWidth selectText:self];
			[desktopHeight selectText:self];
		}
		else
			result = NO;
	}
	[tmp release];
	return result;
}

- (void)rDParam:(NSMutableArray *)input {
	
	[input addObject:@"-o"];
	NSString *rArg = [NSString stringWithString:@" "];
	NSString *curResolution = [NSString stringWithString:[resolutionsDict valueForKey:[[[connectionArray selection] valueForKey:@"displaySettings"] resolution]]];
	if([curResolution isEqualToString:@"-f"])
	{
		rArg = [rArg stringByAppendingString:curResolution];
	}
	else 
	{
		if ([curResolution isEqualToString:@""])
		{
			Display *tmp = [[Display alloc] initWithDisplay:[[connectionArray selection] valueForKey:@"displaySettings"]];
			curResolution = [NSString stringWithFormat:@"%@x%@", [tmp width], [tmp height]];
			[tmp release];
		}
		rArg = [[rArg stringByAppendingString:@"-g "] stringByAppendingString:curResolution];	
	}
	Display *disp = [[connectionArray selection] valueForKey:@"displaySettings"];
	NSString *curColor = [colorsDict valueForKey:[disp color]];

	rArg = [[[rArg stringByAppendingString:@" -a "] stringByAppendingString:curColor] stringByAppendingString:@" "];
    
    if (NSOnState == [spanMonitors state])
        rArg = [rArg stringByAppendingString: @"--span-monitors "];
		
	[input addObject:rArg];
}

@end
