
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "FirewallProxyTabAssist.h"

@implementation FirewallProxyTabAssist

#pragma mark -
#pragma mark Initialization

static id sharedInstance = nil;

+ (id)sharedInstance {
	if (sharedInstance == nil)
		sharedInstance = [[self alloc] init];
	return sharedInstance;
}

-(id)init {
	if(sharedInstance) {
		[NSException raise:NSInternalInconsistencyException
					format:@"[%@] cannot be called; use +[%@ %@] instead",
		 NSStringFromClass([self class]), 
		 NSStringFromClass([self class]),
		 NSStringFromSelector(@selector(sharedInstance))];							 
	}
	
	else
		sharedInstance = [super init];
	return sharedInstance;
}

- (void)dealloc {
	[super dealloc];
}

#pragma mark -
#pragma mark Useful methods

- (BOOL)isValidTextField:(NSTextField *)textfield {
	NSString * typedText = [NSString stringWithString:[textfield stringValue]];
	return(![typedText isEqual:@""]);
}

#pragma mark -
#pragma mark Actions

- (IBAction)doEnableSsl:(id)sender {
	if([chkBtnSsl state] == 1)
	{
		[sslGateway setEnabled:YES];
	}
	else
	{
		[sslGateway setEnabled:NO];
		[sslGateway setStringValue:@""];
		[[sslGateway cell] setPlaceholderString:@""];
	}
}

- (IBAction)doUseProxy:(id)sender {
	if([behaviourMatrix selectedRow] == 1)
	{
		[proxyServer setEnabled:YES];
		[exclusiveServers setEnabled:YES];
	}
	else
	{
		[proxyServer setEnabled:NO];
		[exclusiveServers setEnabled:NO];
		[proxyServer setStringValue:@""];
		[[proxyServer cell] setPlaceholderString:@""];
		[exclusiveServers setStringValue:@""];
		[[exclusiveServers cell] setPlaceholderString:@""];
	}
}

#pragma mark -
#pragma mark Overloaded methods

- (void)setDefaultTabState {
	[exclusiveServers setStringValue:@""];
	[proxyServer setStringValue:@""];
	[sslGateway setStringValue:@""];
}

- (BOOL)isDataCorrect {
	if([behaviourMatrix selectedRow] == 0)
		return YES;
	else
		if([self isValidTextField:proxyServer])
		{
			[proxyServer selectText:self];
			return YES;
		}
	return NO;
}

- (void)rDParam:(NSMutableArray *)input {
	BOOL doUseNat = [[[locationArray selectedObjects] objectAtIndex:0] doUseNatSupport];
	if(doUseNat)
	{
		NSString *nat = [NSString stringWithString:@"-t"];
		[input addObject:nat];
	}
	BOOL doUseSsl = [[[locationArray selectedObjects] objectAtIndex:0] doUseSsl];
	if(doUseSsl)
	{
		NSString *sslArg = [NSString stringWithString:@"-e"];
		NSString *ssl = [NSString stringWithString:[[locationArray selection] valueForKey:@"sslGateway"]];
		[input addObject:sslArg];
		[input addObject:ssl];
	}
}

@end
