
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "LocalResourcesTabAssist.h"

@implementation LocalResourcesTabAssist

#pragma mark -
#pragma mark Initialization

static id sharedInstance = nil;

+ (id)sharedInstance {
	if (sharedInstance == nil)
		sharedInstance = [[self alloc] init];
	return sharedInstance;
}

-(id)init {
	if(sharedInstance) {
		[NSException raise:NSInternalInconsistencyException
					format:@"[%@] cannot be called; use +[%@ %@] instead",
		 NSStringFromClass([self class]), 
		 NSStringFromClass([self class]),
		 NSStringFromSelector(@selector(sharedInstance))];							 
	}	
	else
	{
		sharedInstance = [super init];
		{
			NSArray *keys = [NSArray arrayWithObjects:@"On the local computer", @"On the remote computer", @"In full screen mode only", nil];
			NSArray *values = [NSArray arrayWithObjects:@"-K", @"", @"-K", nil];
			keyboardsDict = [[NSMutableDictionary alloc] initWithObjects:values forKeys:keys];
		}				
		{
			NSArray *keys = [NSArray arrayWithObjects:@"Bring to Local Computer", @"Leave at Remote Computer", @"Don't Play", nil];
			NSArray *values = [NSArray arrayWithObjects:@"local", @"remote", @"off", nil];
			soundsDict = [[NSMutableDictionary alloc] initWithObjects:values forKeys:keys];
		}
	}
	return sharedInstance;
}

- (void)dealloc {
	[keyboardsDict release];
	[soundsDict release];
	[super dealloc];
}

#pragma mark -
#pragma mark Get devices methods

- (NSArray *)getDiskDrives {
	NSMutableArray *diskDrivesArr = [NSMutableArray array];
	
	// init process
	NSTask *mountTask = [[NSTask alloc] init];
	[mountTask setLaunchPath:@"/sbin/mount"];
		
	// setting up the environment of the task
    NSPipe *stdOutPipe = [NSPipe pipe];
    NSFileHandle *stdOutReadHandle = [stdOutPipe fileHandleForReading];
	
	[mountTask setStandardOutput:stdOutPipe];
	
	// getting disk drives list
	[mountTask launch];
	[mountTask release];
	
	NSData *data = [NSData dataWithData:[stdOutReadHandle availableData]];

	NSString *string = [[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding];
	NSArray *stringsArr = [string componentsSeparatedByString:@"\n"];

	[string release];
	
	int diskIndex = 0;
	for(int i = 0; i != [stringsArr count]; ++i)
	{
		if([[stringsArr objectAtIndex:i] hasPrefix:@"/"])
		{
			NSArray *arrContainingPath = [[stringsArr objectAtIndex:i] componentsSeparatedByString:@" "];

			int drivePathIndex = 2;
			NSString *drivePath = [arrContainingPath objectAtIndex:drivePathIndex];
			NSString *drivePathWithSpaces = nil;
			++drivePathIndex;
			if(![[arrContainingPath objectAtIndex:drivePathIndex] hasPrefix:@"("])
			{
				// this device has a name with spaces - this is unsupported for now.
//				continue;
				
				drivePathWithSpaces = [[[NSString stringWithString:drivePath]
									    stringByAppendingString:@" "]
										stringByAppendingString:[arrContainingPath objectAtIndex:drivePathIndex]];
				++drivePathIndex;
				while(![[arrContainingPath objectAtIndex:drivePathIndex] hasPrefix:@"("])
				{
					drivePathWithSpaces = [[drivePathWithSpaces stringByAppendingString:@" "]
										   stringByAppendingString:[arrContainingPath objectAtIndex:drivePathIndex]];
					++drivePathIndex;
				}
			}
			NSArray *arrContainingName = [drivePath componentsSeparatedByString:@"/"];
			if([arrContainingName objectAtIndex:1])
			{
				NSString *driveName = nil;
				
				if([[arrContainingName objectAtIndex:1] isEqualToString:@""])
				{
					driveName = [[NSString stringWithString:@"Disk"] 
										   stringByAppendingString:[[NSNumber numberWithInt:diskIndex] stringValue]];
					++diskIndex;
				}
				else
				{
					if(drivePathWithSpaces == nil)
						driveName = [NSString stringWithString:drivePath];
					else
						driveName = [[NSString stringWithString:drivePathWithSpaces] 
									 stringByReplacingOccurrencesOfString:@" " withString:@"_"];
					NSCharacterSet *setChToRemove = [NSCharacterSet characterSetWithCharactersInString:@"/Volumes/"];
					driveName = [driveName stringByTrimmingCharactersInSet:setChToRemove];
					if ([driveName hasPrefix:@"/"])
						driveName = [driveName substringFromIndex:1];
				}
				NSString *fullDriveEntry = nil;
				if(drivePathWithSpaces == nil)
				{
					fullDriveEntry = [[driveName stringByAppendingString:@"="] stringByAppendingString:drivePath];
				}
				else
				{
					/// embrace the strings with backslashed double-quotes
					// then backslash all spaces
					drivePathWithSpaces = [[NSString stringWithFormat:@"\\\"%@\\\"", drivePathWithSpaces] 
										   stringByReplacingOccurrencesOfString:@" " withString:@"\\ "];
					fullDriveEntry = [[driveName stringByAppendingString:@"="] stringByAppendingString:drivePathWithSpaces];
				}
				[diskDrivesArr addObject:fullDriveEntry];
				
			}
		}
		else
			continue;
	}
	
	return diskDrivesArr;
}

- (NSArray *)getSerialPorts {
	NSMutableArray *serialPortsArr = [NSMutableArray array];
	
	// init process
	NSTask *lsTask = [[NSTask alloc] init];
	[lsTask setLaunchPath:@"/bin/ls"];
	NSMutableArray *args = [NSArray arrayWithObjects:@"/dev", /*@"|", @"grep", @"ttys00", */nil];
	[lsTask setArguments:args];
	
	// setting up the environment of the task
    NSPipe *stdOutPipe = [NSPipe pipe];
    NSFileHandle *stdOutReadHandle = [stdOutPipe fileHandleForReading];
	
	[lsTask setStandardOutput:stdOutPipe];
	
	// getting disk drives list
	[lsTask launch];
	[lsTask release];
	
	NSData *data = [NSData dataWithData:[stdOutReadHandle availableData]];
	
	NSString *string = [[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding];
	NSArray *stringsArr = [string componentsSeparatedByString:@"\n"];
	
	[string release];
	
	for(int i = 0; i != [stringsArr count]; ++i)
	{
		if([[stringsArr objectAtIndex:i] isEqualToString:@""])
			continue;
		else
		{
			if(![[stringsArr objectAtIndex:i] hasPrefix:@"ttyS"])
				continue;
			NSString *portName = [[[NSString stringWithString:@"COM"] stringByAppendingString:
								  [[NSNumber numberWithInt:[serialPortsArr count] + 1] stringValue]] stringByAppendingString:
								  @"=/dev/"];
			[serialPortsArr addObject:[portName stringByAppendingString:[stringsArr objectAtIndex:i]]];
		}			
	}
	
	return serialPortsArr;
}

- (NSArray *)getParallelPorts {
	NSMutableArray *parallelPortsArr = [NSMutableArray array];
	// init process
	NSTask *lsTask = [[NSTask alloc] init];
	[lsTask setLaunchPath:@"/bin/ls"];
	NSMutableArray *args = [NSArray arrayWithObjects:@"/dev", /*@"|", @"grep", @"lp",*/ nil];
	[lsTask setArguments:args];
	
	// setting up the environment of the task
    NSPipe *stdOutPipe = [NSPipe pipe];
    NSFileHandle *stdOutReadHandle = [stdOutPipe fileHandleForReading];
	
	[lsTask setStandardOutput:stdOutPipe];
	
	// getting disk drives list
	[lsTask launch];
	[lsTask release];
	
	NSData *data = [NSData dataWithData:[stdOutReadHandle availableData]];
	
	NSString *string = [[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding];
	NSArray *stringsArr = [string componentsSeparatedByString:@"\n"];
	
	[string release];
	
	for(int i = 0; i != [stringsArr count]; ++i)
	{
		if([[stringsArr objectAtIndex:i] isEqualToString:@""])
			continue;
		else
		{
			if(![[stringsArr objectAtIndex:i] hasPrefix:@"lp"])
				continue;
			NSString *portName = [[[NSString stringWithString:@"LPT"] stringByAppendingString:
								   [[NSNumber numberWithInt:[parallelPortsArr count] + 1] stringValue]] stringByAppendingString:
								  @"=/dev/"];
			[parallelPortsArr addObject:[portName stringByAppendingString:[stringsArr objectAtIndex:i]]];
		}			
	}
	
	
	return parallelPortsArr;
}

- (NSArray *)getPrinters
{
    NSMutableArray *ret = nil;
    FILE *res = popen("lpstat -a | cut -d\" \" -f1", "r");
    
    if (res)
    {
        NSFileHandle *f = [[NSFileHandle alloc] initWithFileDescriptor: fileno(res)];
        NSString *sr = [[NSString alloc] initWithData: [f readDataToEndOfFile] encoding: NSUTF8StringEncoding];
        
        ret = [NSMutableArray arrayWithArray: [sr componentsSeparatedByString: @"\n"]];
        
        if ([ret count])
            [ret removeLastObject];
        
        [f release];
        [sr release];
        
        pclose(res);
    }
    
    return ret;
}

#pragma mark -
#pragma mark Overloaded methods

- (void)setDefaultTabState {
}

- (BOOL)isDataCorrect {
	return YES;
}

void launchDefaultsForX11Clipboard(NSArray *args)
{
    NSTask *task = [[NSTask alloc] init];
    
    [task setLaunchPath: @"/usr/bin/defaults"];
    [task setArguments: args];
    [task launch];
    [task waitUntilExit];
    [task release];
}

void setX11Clipboard(BOOL state)
{
    NSMutableArray *args = [[NSMutableArray alloc] initWithObjects: @"write", @"org.x.X11", @"", @"-boolean", state ? @"true" : @"false", nil];
    
    [args replaceObjectAtIndex: 2 withObject: @"sync_clipboard_to_pasteboard"];
    launchDefaultsForX11Clipboard(args);
    [args replaceObjectAtIndex: 2 withObject: @"sync_pasteboard"];
    launchDefaultsForX11Clipboard(args);  
    [args replaceObjectAtIndex: 2 withObject: @"sync_pasteboard_to_clipboard"];
    launchDefaultsForX11Clipboard(args);   
    [args replaceObjectAtIndex: 2 withObject: @"sync_pasteboard_to_primary"];
    launchDefaultsForX11Clipboard(args);   
    [args replaceObjectAtIndex: 2 withObject: @"sync_primary_on_select"];
    launchDefaultsForX11Clipboard(args);
    
    [args release];
}

- (void)rDParam:(NSMutableArray *)input {
	NSString *rArg = [NSString stringWithString:@"-r"];
	Resources *tmpResourses = [[Resources alloc] initWithResource:[[connectionArray selection] valueForKey:@"resources"]];
	NSString *curSound = [NSString stringWithString: [soundsDict valueForKey:[tmpResourses sound]]];
	rArg = [[rArg stringByAppendingString:@" sound:"] stringByAppendingString:curSound];
	
	NSString *curKeyboard = [NSString stringWithString:[keyboardsDict valueForKey:[tmpResourses keyboard]]];
	NSString *curScreenResol = [NSString stringWithString:[[[connectionArray selection] valueForKey:@"displaySettings"] resolution]];
    
	if ([[tmpResourses keyboard] isEqualToString: @"On the local computer"] || ([[tmpResourses keyboard] isEqualToString:  @"In full screen mode only"] && [curScreenResol isEqualToString:@"Full Screen"]))
        rArg = [[rArg stringByAppendingString:@" "] stringByAppendingString:curKeyboard];
	
	// local devices
	
	NSString *diskDrivesString = nil;
	NSString *serialPortsString = nil;
	NSString *parallelPortsString = nil;
	NSString *clipboardString = nil;
	NSString *printersString = nil;
	NSString *smartcardsString = nil;
	
	// disk drives
	if([tmpResourses doRedirectDiskDrives])
	{
		NSArray *diskDrivesStringsArr = [self getDiskDrives]; // returns an object in autoreleased pool
		if ([diskDrivesStringsArr count] != 0)
			diskDrivesString = [NSString stringWithString:@" -r disk:"];
		for(int i = 0; i != [diskDrivesStringsArr count]; ++i)
		{
			if(i>0)
				diskDrivesString = [diskDrivesString stringByAppendingString:@","];
			diskDrivesString = [diskDrivesString stringByAppendingString:[diskDrivesStringsArr objectAtIndex:i]];
		}
	}
	
	// serial ports
	if([tmpResourses doRedirectSerialPorts])
	{
		NSArray *serialPortsStringsArr = [self getSerialPorts];
		serialPortsString = [NSString stringWithString:@" -r comport:"];
		for(int i = 0; i != [serialPortsStringsArr count]; ++i)
		{
			if(i>0)
				serialPortsString = [serialPortsString stringByAppendingString:@","];
			serialPortsString = [serialPortsString stringByAppendingString:[serialPortsStringsArr objectAtIndex:i]];
		}
	}
	
	// parallel ports
	if([tmpResourses doRedirectParallelPorts])
	{
		NSArray *parallelPortsStringsArr = [self getParallelPorts];
		parallelPortsString = [NSString stringWithString:@" -r lptport:"];
		for(int i = 0; i != [parallelPortsStringsArr count]; ++i)
		{
			if(i>0)
				parallelPortsString = [parallelPortsString stringByAppendingString:@","];
			parallelPortsString = [parallelPortsString stringByAppendingString:[parallelPortsStringsArr objectAtIndex:i]];
		}
	}
	
	// clipboard
	if([tmpResourses doRedirectClipboard])
	{
		clipboardString = [NSString stringWithString:@" -r clipboard:CLIPBOARD"];
        setX11Clipboard(YES);
	}
    else
        setX11Clipboard(NO);
	
	// printers
	if([tmpResourses doRedirectPrinters])
	{
        NSArray *printersStringsArr = [NSMutableArray arrayWithArray:[self getPrinters]];
		printersString = [NSString stringWithString:@" -r printer:"];
		for(int i = 0; i != [printersStringsArr count]; ++i)
		{
			if(i>0)
				printersString = [printersString stringByAppendingString:@","];
			printersString = [printersString stringByAppendingString:[printersStringsArr objectAtIndex:i]];
		}
	}
	
	// smart cards
	if([tmpResourses doRedirectSmartcards])
	{
		smartcardsString = [NSString stringWithString:@" -r scard"];
	}
	
	if(diskDrivesString !=nil)
		rArg = [rArg stringByAppendingString:diskDrivesString];
	if(serialPortsString !=nil)
		rArg = [rArg stringByAppendingString:serialPortsString];
	if(parallelPortsString !=nil)
		rArg = [rArg stringByAppendingString:parallelPortsString];
	if(clipboardString !=nil)
		rArg = [rArg stringByAppendingString:clipboardString];
	if(printersString != nil)
		rArg = [rArg stringByAppendingString:printersString];
	if(smartcardsString != nil)
		rArg = [rArg stringByAppendingString:smartcardsString];
	
	rArg = [rArg stringByAppendingString:@" "];
	NSString *tmp = nil;
	if (tmp = [input lastObject])
	{
		[input removeLastObject];
		tmp = [tmp stringByAppendingString:rArg];
	}
	[input addObject:tmp];
	
	[tmpResourses release];
}

@end
