
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

//
//  Location.m
//  Provision AppPortal
//
//  Created by smalti on 4/10/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "Location.h"


@implementation Location

#pragma mark -
#pragma mark Initialization

- (id)init {
	if (self = [super init])
	{
		name = [[NSString alloc] initWithString:@"Location"];
		protocol = [[NSString alloc] initWithString:@"HTTP"];
		port = [[NSString alloc] initWithString:@""];
		proxyServer = [[NSString alloc] initWithString:@""];;
		exclusiveServers = [[NSString alloc] initWithString:@""];
		sslGateway = [[NSString alloc] initWithString:@""];
		doUseAsDefault = NO;
	    doUseNatSupport = NO;
		doUseSsl = NO;
		doUseProxy = NO;
		
		brokers = [[NSMutableArray alloc] init];
	}
	return self;
}

- (id)initWithLocation:(Location *)input {
	if(self = [super init])
	{
		[self setName:[input name]];
		[self setProtocol:[input protocol]];
		[self setPort:[input port]];
		[self setProxyServer:[input proxyServer]];
		[self setExclusiveServers:[input exclusiveServers]];
		[self setSslGateWay:[input sslGateway]];
		[self setDoUseAsDefault:[input doUseAsDefault]];
		[self setDoUseNatSupport:[input doUseNatSupport]];
		[self setDoUseSsl:[input doUseSsl]];
		[self setDoUseProxy:[input doUseProxy]];
		[self setBrokers:[input brokers]];
	}
	return self;
}
- (void)dealloc {
	[name release];
	[protocol release];
	[port release];
	[proxyServer release];
	[exclusiveServers release];
	[sslGateway release];
	[brokers release];
	[super dealloc];
}

#pragma mark -
#pragma mark Setters

- (void)setName:(NSString *)input {
	[name autorelease];
	if(input == nil)
		name = [[NSString alloc] initWithString:@"Location"];
	else
	    name = [input retain];
}

- (void)setProtocol:(NSString *)input {
	[protocol autorelease];
	if(input == nil)
		protocol = [[NSString alloc] initWithString:@""];
	else
		protocol = [input retain];
}

- (void)setPort:(NSString *)input {
	[port autorelease];
	if(input == nil)
		port = [[NSString alloc] initWithString:@""];
	else
		port = [input retain];
}

- (void)setProxyServer:(NSString *)input {
	[proxyServer autorelease];
	if(input == nil)
		proxyServer = [[NSString alloc] initWithString:@""];
	else
		proxyServer = [input retain];
}

- (void)setExclusiveServers:(NSString *)input {
	[exclusiveServers autorelease];
	if(input == nil)
		exclusiveServers = [[NSString alloc] initWithString:@""];
	else
		exclusiveServers = [input retain];
}

- (void)setSslGateWay:(NSString *)input {
	[sslGateway autorelease];
	if(input == nil)
		sslGateway = [[NSString alloc] initWithString:@""];
	else
		sslGateway = [input retain];
}

- (void)setDoUseAsDefault:(BOOL)input {
	doUseAsDefault = input;
}

- (void)setDoUseNatSupport:(BOOL)input {
	doUseNatSupport = input;
}

- (void)setDoUseSsl:(BOOL)input {
	doUseSsl = input;
}

- (void)setDoUseProxy:(BOOL)input {
	doUseProxy = input;
}

- (void)setBrokers:(NSMutableArray *)input{
	[brokers autorelease];
	if(input == nil)
		brokers = [[NSMutableArray alloc] init];
	else
		brokers = [[NSMutableArray alloc] initWithArray:input];
}

#pragma mark -
#pragma mark Getters

- (NSString *)name {
	return name;
}

- (NSString *)protocol {
	return protocol;
}

- (NSString *)port {
	return port;
}

- (NSString *)proxyServer {
	return proxyServer;
}

- (NSString *)exclusiveServers {
	return exclusiveServers;
}

- (NSString *)sslGateway {
	return sslGateway;
}

- (BOOL)doUseAsDefault {
	return doUseAsDefault;
}

- (BOOL)doUseNatSupport {
	return doUseNatSupport;
}

- (BOOL)doUseSsl {
	return doUseSsl;
}

- (BOOL)doUseProxy {
	return doUseProxy;
}

- (NSMutableArray *)brokers {
	return brokers;
}

#pragma mark -
#pragma mark Encode-Decode

- (void)encodeWithCoder:(NSCoder *)coder {
	[coder encodeObject:name forKey:@"name"];
	[coder encodeObject:protocol forKey:@"protocol"];
	[coder encodeObject:port forKey:@"port"];
	if(doUseProxy)
	{
		[coder encodeObject:proxyServer forKey:@"proxyServer"];
		[coder encodeObject:exclusiveServers forKey:@"exclusiveServers"];
	}
	if(doUseSsl)
	{
		[coder encodeObject:sslGateway forKey:@"sslGateway"];
	}
	[coder encodeBool:doUseAsDefault forKey:@"doUseAsDefault"];
	[coder encodeBool:doUseNatSupport forKey:@"doUseNatSupport"];
	[coder encodeBool:doUseSsl forKey:@"doUseSsl"];
	[coder encodeBool:doUseProxy forKey:@"doUseProxy"];
	[coder encodeObject:brokers forKey:@"brokers"];
	
}

- (id)initWithCoder:(NSCoder *)coder {
	if (self = [super init])
	{
		[self setName:[coder decodeObjectForKey:@"name"]];
		[self setProtocol:[coder decodeObjectForKey:@"protocol"]];
		[self setPort:[coder decodeObjectForKey:@"port"]];
		[self setProxyServer:[coder decodeObjectForKey:@"proxyServer"]];
		[self setExclusiveServers:[coder decodeObjectForKey:@"exclusiveServers"]];
		[self setSslGateWay:[coder decodeObjectForKey:@"sslGateway"]];
		[self setDoUseAsDefault:[coder decodeBoolForKey:@"doUseAsDefault"]];
		[self setDoUseNatSupport:[coder decodeBoolForKey:@"doUseNatSupport"]];
		[self setDoUseSsl:[coder decodeBoolForKey:@"doUseSsl"]];
		[self setDoUseProxy:[coder decodeBoolForKey:@"doUseProxy"]];
		[self setBrokers:[coder decodeObjectForKey:@"brokers"]];
	}
	return self;
}

@end
