
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "ManageConnectionAssist.h"
#import "SelFarmTabAssist.h"
#import "ConfigSrcTabAssist.h"
#import "ConnectivityTabAssist.h"
#import "FirewallProxyTabAssist.h"
#import "CredentialsTabAssist.h"
#import "DisplayTabAssist.h"
#import "LocalResourcesTabAssist.h"
#import "UserExpTabAssist.h"
#import "PassManageTabAssist.h"
#import "DesktopIntegrationTabAssist.h"
#import "AutoLaunchTabAssist.h"

@implementation ManageConnectionAssist

#define numberOfLocations 3
#define defaultLocationName @"Location"
#define configFileName @"settings.plist"
#define configPath  @"~/Library/Application Support/Quest Software/vWorkspace AppPortal/"

static id sharedInstance = nil;


#pragma mark -
#pragma mark Initialization

+ (id)sharedInstance
{
	if (sharedInstance == nil) 
	{ 
		sharedInstance = [[self alloc] init];
	} 
	return sharedInstance; 
}

- (void)initTabAssistsArray {
	
	m_assistsArray  = [[NSMutableArray alloc] initWithCapacity:[tabView numberOfTabViewItems]];
	[m_assistsArray addObject:[SelFarmTabAssist sharedInstance]];
	[m_assistsArray addObject:[ConfigSrcTabAssist sharedInstance]];
	[m_assistsArray addObject:[ConnectivityTabAssist sharedInstance]];
	[m_assistsArray addObject:[FirewallProxyTabAssist sharedInstance]];
	[m_assistsArray addObject:[CredentialsTabAssist sharedInstance]];
	[m_assistsArray addObject:[DisplayTabAssist sharedInstance]];
	[m_assistsArray addObject:[LocalResourcesTabAssist sharedInstance]];
	[m_assistsArray addObject:[UserExpTabAssist sharedInstance]];
	[m_assistsArray addObject:[PassManageTabAssist sharedInstance]];
	[m_assistsArray addObject:[DesktopIntegrationTabAssist sharedInstance]];
	[m_assistsArray addObject:[AutoLaunchTabAssist sharedInstance]];
}

- (id)init {
    if (sharedInstance) {
		[NSException raise:NSInternalInconsistencyException
					format:@"[%@] cannot be called; use +[%@ %@] instead",
		 NSStringFromClass([self class]), 
		 NSStringFromClass([self class]),
		 NSStringFromSelector(@selector(sharedInstance))];							 
	}	
	else 
	{
		if (sharedInstance = [super init])
		{	
			m_connections = [[NSMutableArray alloc] init];
			[self loadDataFromDisk];
		}
    }
    return sharedInstance;
}

- (void)dealloc {
	[m_connections release];
	[m_assistsArray release];
	[super dealloc];
}


#pragma mark -
#pragma mark Getters/Setters

- (NSMutableArray *)connections {
	return m_connections;
}

- (void)setConnections:(NSArray *)newConnections {
	[m_connections autorelease];
	if(newConnections == nil)
		m_connections = [[NSMutableArray alloc] init];
	else
		m_connections = [[NSMutableArray alloc] initWithArray:newConnections];
}

#pragma mark -
#pragma mark IBActions
	 
- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	[sheet close];
		if(m_doCancel)
		{
			[self closeWnd:window];
			if(m_currentConnectionItem)
			{
				[m_currentConnectionItem release];
				m_currentConnectionItem = nil;
			}
		}		
}

- (IBAction)doCancel:(id)sender {
	[NSApp beginSheet:confirmCancelPanel modalForWindow:window modalDelegate:self 
	   didEndSelector:@selector(sheetDidEnd: returnCode: contextInfo:) contextInfo:nil];
}
	 
- (IBAction)doConfirmCancel:(id)sender {
	m_doCancel = YES;
	[NSApp endSheet:confirmCancelPanel];
	
	// restore saved state
	[self loadDataFromDisk];
	[connectionArray setContent:m_connections];
}

- (IBAction)doDeclineCancel:(id)sender {
	m_doCancel = NO;
	[NSApp endSheet:confirmCancelPanel];
}

- (IBAction)doFinish:(id)sender {
    [self closeWnd:window];
	[self saveDataToDisk];
	[m_currentConnectionItem release];
	m_currentConnectionItem = nil;
}

- (IBAction)doOk:(id)sender {
	if(![connectionArray canRemove])
	{
		[self closeWnd:window];
		[self saveDataToDisk];
		if(m_currentConnectionItem)
		{
			[m_currentConnectionItem release];
			m_currentConnectionItem = nil;
		}
		return;
	}
	for(int i=1; i != [tabView numberOfTabViewItems]; ++i)
	{
		m_assistsObserver = [m_assistsArray objectAtIndex:i];
		if(![m_assistsObserver isDataCorrect])
		{
			[NSApp beginSheet:fillFieldMesPanel modalForWindow:window modalDelegate:self didEndSelector:nil contextInfo:nil];
			return;
		}
	}
	
	[self closeWnd:window];
	[self saveDataToDisk];
	if(m_currentConnectionItem)
	{
		[m_currentConnectionItem release];
		m_currentConnectionItem = nil;
	}
}

- (IBAction)goNext:(id)sender {
	// get the current tab assist - it will do all the work
	m_assistsObserver = [m_assistsArray objectAtIndex:m_currentTabIndex];
	m_tabAssist = [m_assistsArray objectAtIndex:m_currentTabIndex];

	// check the corectness of the data in input fileds of the current tab  
	if(![m_assistsObserver isDataCorrect])
	{   
		[NSApp beginSheet:fillFieldMesPanel modalForWindow:window modalDelegate:self didEndSelector:nil contextInfo:nil];
		return;
	}
	
	if(m_currentTabIndex == 0)
	{
		[self createNewConnection];
	}
	else if(m_currentTabIndex == 1 && [m_tabAssist needToDownloadFarm])
	{
		[m_tabAssist doDownloadFarm];
		return;
	}

	++m_currentTabIndex;
	[tabView selectNextTabViewItem:sender];
	
	// the following actions regard only to buttons after all other actions are done
	if(m_currentTabIndex == 1)
		[btnPrev setEnabled:YES];
	if(m_currentTabIndex == [tabView numberOfTabViewItems]-1) {
		[btnNext setEnabled:NO];
		[btnNext setTransparent:YES];
		[btnFinish setEnabled:YES];
		[btnFinish setTransparent:NO];
    }
}

- (IBAction)goPrev:(id)sender {
    if(m_currentTabIndex == [tabView numberOfTabViewItems]-1)
	{
		[btnFinish setEnabled:NO];
		[btnFinish setTransparent:YES];
		[btnNext setEnabled:YES];
		[btnNext setTransparent:NO];
	}
	if (m_currentTabIndex == 1)
	{
		if (m_currentConnectionItem)
		{
			[m_currentConnectionItem release];
			m_currentConnectionItem = nil;
		}
		[self setCurrentMode];
	}

	--m_currentTabIndex;
	[tabView selectPreviousTabViewItem:sender];
}

- (IBAction)showModal:(id)ownerWnd sender:(id)sender {
	[self setInitialManageFarmState];
	[self setCurrentMode];
	[NSApp runModalForWindow:window];
}

- (IBAction)setModifyMode:(id)sender {
	// crutch =(((
	
	
	[btnOk setEnabled:YES];
	[btnOk setTransparent:NO];
	[btnNext setEnabled:NO];
	[btnNext setTransparent:YES];
	[btnPrev setEnabled:NO];
	[btnPrev setTransparent:YES];
	[btnFinish setEnabled:NO];
	[btnFinish setTransparent:YES];
}
	
- (IBAction)setCreateMode:(id)sender {
	[btnOk setEnabled:NO];
	[btnOk setTransparent:YES];
	[btnNext setEnabled:YES];
	[btnNext setTransparent:NO];
	[btnPrev setEnabled:NO];
	[btnPrev setTransparent:NO];
	[btnFinish setEnabled:NO];
	[btnFinish setTransparent:YES];
	[self clearTabs];
}

- (IBAction)doGetAppList:(id)sender {
	if (![[[connectionArray selection] valueForKey:@"credentials"] doSave])
	{		
		[usernameField setStringValue:@""];
		[passwordField setStringValue:@""];
		[domainField setStringValue:@""];
		NSTextFieldCell *usernameCell = [usernameField cell];
		[usernameCell setPlaceholderString:@""];
		NSTextFieldCell *passwordCell = [passwordField cell];
		[passwordCell setPlaceholderString:@""];
		NSTextFieldCell *domainCell = [domainField cell];
		[domainCell setPlaceholderString:@""];
		
		[NSApp beginSheet:credPanel modalForWindow:window modalDelegate:self didEndSelector:nil contextInfo:nil];
	}
	[self getApplicationsList];
}

- (IBAction)doOkFillFieldMessage:(id)sender {
	[NSApp endSheet:fillFieldMesPanel];
	[fillFieldMesPanel close];
}

#pragma mark -
#pragma mark Icons decoding

- (NSString *)binaryStringfromInt:(const int)mask {
	NSMutableString * tmpStr = [NSMutableString string];
	int res = mask;
	while(res)
	{
		tmpStr = [NSMutableString stringWithFormat:@"%d%@", res%2, tmpStr];
		res >>= 1;
	}
	return tmpStr;
}

// In the following functions (and everywhere else :) rangeOfCharFromSet return a range beginning from 0, not 1
- (int)maskRightShift:(const int) mask {
	// Translating number to bin string
	NSMutableString *binMaskStr = [NSMutableString stringWithString:[self binaryStringfromInt:mask]];
	if([binMaskStr isEqualToString:@""])
		return 0;
	int binMaskSize = [binMaskStr length];
	
	// Getting right numbers
	NSCharacterSet *chSet = [NSCharacterSet characterSetWithCharactersInString:@"1"];
	NSRange  chRange = [binMaskStr rangeOfCharacterFromSet:chSet options:NSBackwardsSearch];
	int lastOneIndex = chRange.location;
	if (lastOneIndex < 0)
		return binMaskSize;
	
	return binMaskSize - lastOneIndex - 1;
}

- (int)maskLeftShift:(int) mask {
	// Translating number to bin string
	NSMutableString *binMaskStr = [NSMutableString stringWithString:[self binaryStringfromInt:mask]];
	if([binMaskStr isEqualToString:@""])
		return 0;
	
	// Getting one string
	NSCharacterSet *chSet = [NSCharacterSet characterSetWithCharactersInString:@"1"];
	NSRange  chRange = [binMaskStr rangeOfCharacterFromSet:chSet];
	int firstOneIndex = chRange.location;
	chRange = [binMaskStr rangeOfCharacterFromSet:chSet options:NSBackwardsSearch];
	int lastOneIndex = chRange.location;
	
	if (firstOneIndex < 0 || lastOneIndex < 0)
		return 0;
	
	return 8 - (lastOneIndex - firstOneIndex + 1);
}

- (BOOL)getInt:(uint *)inputInt fromData:(NSData *)inputData {
	// pure "c" code converting raw buffer to int %)
	int datalen = [inputData length];
	char *tmp = malloc(datalen + 1);
	if (tmp == NULL) 
		return FALSE;
	memcpy(tmp, [inputData bytes], datalen);
	tmp[datalen] = '\0';
	errno = 0;
	*inputInt = strtol(tmp, NULL, 16);
	free (tmp);
	if (errno != 0)
		return FALSE;
	return TRUE;
}

- (BOOL)getInt:(uint *)inputInt fromData:(NSData *)inputData withRange:(NSRange)range {
	NSMutableData *tmpMutableData = [NSMutableData dataWithLength:range.length];
	[inputData getBytes:[tmpMutableData mutableBytes] range:range];
	NSData *tmpData = [NSData dataWithData:tmpMutableData];
	return [self getInt:inputInt fromData:tmpData];
}

- (NSImage *)createAppIconRepWithFile:(NSString *)fileName {
	const int SYMBOLS_ON_BYTE = 2;
	const int SYMBOLS_ON_INT = 8;
	const int HEAD_OFFSET = 17;
	const int PICSIZE_BYTE = 5;
	//const int AMASK_BYTE = 9;
	const int RMASK_BYTE = 10;
	const int GMASK_BYTE = 11;
	const int BMASK_BYTE = 12;
	const int WIDTH_BYTE = 6;
	const int HEIGHT_BYTE = 7;
	
	NSFileHandle *iconFile = [NSFileHandle fileHandleForReadingAtPath:fileName];
	if(iconFile == nil)
		return nil;
	NSData *fileData = [NSData dataWithData:[iconFile readDataToEndOfFile]];
	[iconFile closeFile];
	if(fileData == nil || [fileData length] == 0)
		return nil;

	// getting image dimentions in pixels
	uint imageHeight = 0, imageWidth = 0;
    if(![self getInt:&imageHeight fromData:fileData withRange:NSMakeRange(HEIGHT_BYTE*SYMBOLS_ON_INT, SYMBOLS_ON_INT)] ||
       ![self getInt:&imageWidth fromData:fileData withRange:NSMakeRange(WIDTH_BYTE*SYMBOLS_ON_INT, SYMBOLS_ON_INT)])
		return nil;
	
	NSRange range = NSMakeRange(0, SYMBOLS_ON_INT);

	// Getting color masks
	uint blueMask = 0, greenMask = 0, redMask = 0;
	range.location = (HEAD_OFFSET + imageWidth + BMASK_BYTE) * SYMBOLS_ON_INT;
    if(![self getInt:&blueMask fromData:fileData withRange:range])
		return nil;
	range.location = (HEAD_OFFSET + imageWidth + GMASK_BYTE) * SYMBOLS_ON_INT;
    if(![self getInt:&greenMask fromData:fileData withRange:range])
		return nil;
	range.location = (HEAD_OFFSET + imageWidth + RMASK_BYTE) * SYMBOLS_ON_INT;
    if(![self getInt:&redMask fromData:fileData withRange:range])
		return nil;
	
	// color shifts
	int brShift = [self maskRightShift:blueMask],
		blShift = [self maskLeftShift:blueMask],
		grShift = [self maskRightShift:greenMask],
		glShift = [self maskLeftShift:greenMask],
		rrShift = [self maskRightShift:redMask],
		rlShift = [self maskLeftShift:redMask];
	
	// Getting image size in bytes
	uint imageSize = 0;
	range.location = (HEAD_OFFSET + imageWidth + PICSIZE_BYTE) * SYMBOLS_ON_INT;
    if(![self getInt:&imageSize fromData:fileData withRange:range])
		return nil;
	
	// bpp
	int bytesPerPixel = imageSize / (imageHeight * imageWidth);
	
	// Getting image data
	range.length = imageSize * SYMBOLS_ON_BYTE;
	range.location = [fileData length] - range.length;
//	NSMutableData *imageData = [NSMutableData dataWithData:[[fileData subdataWithRange:range] mutableCopy]];
	
	// 1) NEVER pass to [fromData getBytes:toData] toData argument of NSData type - only [toData bytes], i.e. (void *)
	// 2) 1 means - ALWAYS check for the correctness of the types of the variables used in expressions
	// 3) do not use getBytes at all - use the expression in the line several lines above (subdataWithRange)
	// 4) use the 1, 2 and 3 - below (in the for() loop), and everywhere.
	// 5) don't worry, be happy)
	
	// Create out picture
	NSBitmapImageRep *imageRep = [[NSBitmapImageRep alloc] 
								  initWithBitmapDataPlanes:NULL
								  pixelsWide:imageWidth
								  pixelsHigh:imageHeight
								  bitsPerSample:bytesPerPixel*8/4 //(8 bits per byte, four samples - Alpha,R,G,B)
								  samplesPerPixel:4 //(four samples - Alpha,R,G,B)
								  hasAlpha:YES
								  isPlanar:NO
								  colorSpaceName:NSCalibratedRGBColorSpace
								  bitmapFormat:NSAlphaFirstBitmapFormat
								  bytesPerRow:0
								  bitsPerPixel:bytesPerPixel*8];
	
	// Fill pixels in this picture
	for (int i = 0; i < imageHeight; i++)
	{
		for (int j = 0; j < imageWidth; j++)
		{
			// the starting offset for current pixel bytes
			int offset = (j * imageWidth + i) * bytesPerPixel * SYMBOLS_ON_BYTE;
			// Getting cross over bytes of our color
			int size = bytesPerPixel*SYMBOLS_ON_BYTE;
			NSMutableData *colorArray = [[NSMutableData alloc] init];
			[colorArray setLength:size];
			range.length = SYMBOLS_ON_BYTE;
			for (int counter = 0; counter < bytesPerPixel; counter++)
			{
				range.location = offset + (bytesPerPixel - counter - 1) * SYMBOLS_ON_BYTE;
				NSMutableData *curSampleData = [NSMutableData dataWithLength:range.length];
				[colorArray appendBytes:curSampleData length:range.length];
			}
			
			// Color value for this pixel
			uint color = 0;
			if(![self getInt:&color fromData:colorArray])
				return nil;
			
			int r = ((color & redMask) >> rrShift) << rlShift;
			int g = ((color & greenMask) >> grShift) << glShift;
			int b = ((color & blueMask) >> brShift) << blShift;
			
			NSUInteger colors[4];
			colors[0] = 255; 
			colors[1] = r; 
			colors[2] = g; 
			colors[3] = b;
			[imageRep setPixel:colors atX:i y:imageWidth - j - 1];
		}
	}

	NSImage *image = [[NSImage alloc] initWithSize:[imageRep size]];
	[image addRepresentation:imageRep];
	return [image autorelease];
	
//	*pixmap = QPixmap::fromImage(image);
//	pixmap->setMask(pixmap->createMaskFromColor(Qt::black));
	
//	return pixmap;

}

#pragma mark -
#pragma mark PNTSC Launching

- (int)getAppListWithArgs:(NSArray *)input {
	
	NSMutableData *data = [[NSMutableData alloc] init];
	NSMutableData *errData = [[NSMutableData alloc] init];
	
	// create and launching pntsc task
	PntscTask *pntscT = [[PntscTask alloc] init];
	int getAppStatus = [pntscT launchPntscWithArgs:input outData:data errorData:errData];
	[pntscT release];

	if (getAppStatus != 0)
	{
		NSString *errMessage = [[NSString alloc] initWithData: errData encoding: NSASCIIStringEncoding];
		[errorMessage setStringValue:[errMessage autorelease]];
		return getAppStatus;
	}
	
	if (data == nil || [data length] == 0)
	{
		[errorMessage setStringValue:@"No data returned from broker"];
		return -1;
	}
	NSString *string = [[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding];
	NSArray *stringsArr = [string componentsSeparatedByString:@"\n"];
	[string release];
	BOOL hasList = NO;
	for(int i = 0; i != [stringsArr count]; ++i)
	{
		NSString *curString = [NSString stringWithString:[stringsArr objectAtIndex:i]];
		if([curString hasPrefix:@"-"] || [curString hasPrefix:@" -"])
		{
			if([curString hasPrefix:@"- List"] || [curString hasPrefix:@" - List"])
				hasList = YES;
			continue;
		}
		if([curString isEqualToString:@""])
			continue;
		
		// pars object at index i
		NSArray *effectiveStringsArr = [curString componentsSeparatedByString:@","];
		
		AppList *currentAppItem = [[AppList alloc] init];;
		for(int j=0; j != [effectiveStringsArr count]; ++j)
		{
			if([[effectiveStringsArr objectAtIndex:j] hasPrefix:@"["])
			{
				NSCharacterSet *setChToRemove = [NSCharacterSet characterSetWithCharactersInString:@"[]"];
				NSString *curAppName = [NSString stringWithString:
										[[effectiveStringsArr objectAtIndex:j] stringByTrimmingCharactersInSet:setChToRemove]];
				[appListCmbBox addItemWithObjectValue:curAppName];
				[currentAppItem setValue:curAppName forKey:@"appName"];
			}
			else if([[effectiveStringsArr objectAtIndex:j] hasPrefix:@"SmallIcon"])
			{
				NSCharacterSet *setChToRemove = [NSCharacterSet characterSetWithCharactersInString:@"SmallIcon="];
				NSString *curIconPath = [NSString stringWithString:
										[[effectiveStringsArr objectAtIndex:j] stringByTrimmingCharactersInSet:setChToRemove]];
				
				NSImage *curIcon = [self createAppIconRepWithFile:curIconPath];
				
				[currentAppItem setValue:curIcon forKey:@"appIcon"];
					//[image drawInRect:NSMakeRect(0, 0, imageWidth, imageHeight) fromRect:NSZeroRect operation:NSCompositeCopy fraction:1];
			}
            else if (1 == [effectiveStringsArr count])
            {
                [currentAppItem release];
                currentAppItem = nil;
            }
		}
        
		if (currentAppItem)
            [launchedAppArray addObject:currentAppItem];
		
        [currentAppItem release];
	}
	[errData release];
	if(!hasList)
	{
		NSString *errMessage = [[NSString alloc] initWithData: [data autorelease] encoding: NSASCIIStringEncoding];
		[errorMessage setStringValue:[errMessage autorelease]];
		return -1;
	}
	[data release];
	return getAppStatus;
}

- (int)getApplicationsList {
//	[self initX11Wnd];

	[appListCmbBox removeAllItems];
	while([launchedAppArray canRemove])
	{
		[launchedAppArray removeObjects:[launchedAppArray selectedObjects]];
	}
	
	// setting up the task arguments
	
	// set proper arguments: <liza> [pntscTask setArguments:(NSArray*)arguments];
	// pntsc <protocol>://<broker>:port -u <username> -p <password> -d <domain> -L
	// every part is one argument; so there must be 8 arguments.p
	
	NSMutableArray *args = [NSMutableArray array];
	
	BOOL isDefaultLocation = ([[[connectionArray selectedObjects] objectAtIndex:0] currentLocationIndex] != -1);
	m_assistsObserver = [m_assistsArray objectAtIndex:ConnectivityTabAssistIndex];
	[m_assistsObserver rDParam:args];
	
	// executes when location is invalid: broker list is empty
	if([[args objectAtIndex:0] isEqualToString:@"Empty"])
	{
		[errorMessage setStringValue:@"Invalid location"];
		[NSApp beginSheet:errorLaunchingPanel modalForWindow:mainWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
		if(!isDefaultLocation)
			[[[connectionArray selectedObjects] objectAtIndex:0] setCurrentLocationIndex: -1];
		return -1;
	}
	if([args count] == 0)
		return 0;
	
	m_assistsObserver = [m_assistsArray objectAtIndex:CredentialsTabAssistIndex];
	[m_assistsObserver rDParam:args];
	[args addObject:@"-L"];
	
	if ([self getAppListWithArgs:args] != 0)
	{
#ifndef ENABLE_DESKTOP_INTEGRATION
		[NSApp beginSheet:errorLaunchingPanel modalForWindow:mainWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
#else
		[NSApp beginSheet:errorLaunchingPanel modalForWindow:deskIntegrationPanel modalDelegate:self didEndSelector:nil contextInfo:nil];
#endif
		if(!isDefaultLocation)
			[[[connectionArray selectedObjects] objectAtIndex:0] setCurrentLocationIndex: -1];
		return -1;
	}
#ifdef ENABLE_DESKTOP_INTEGRATION
	[[connectionArray selection] setValue:@"Connected" forKey:@"farmStatus"];
#endif
	return 0;
}

- (void)launchApp {
	NSMutableArray *args = [NSMutableArray array];
    //int j = [tabView numberOfTabViewItems];
	
	for(int i=0; i != [tabView numberOfTabViewItems]; ++i)
	{
		m_assistsObserver = [m_assistsArray objectAtIndex:i];
		[m_assistsObserver rDParam:args];
	}
	// TODO: hotfix, repair later
	NSString * curAppName = [NSString stringWithString:[[launchedAppArray selection] valueForKey:@"appName"]];
	NSString *ninethArg = [NSString stringWithString:curAppName];
//	NSRange range = [curAppName rangeOfString:@" "];
//	if(range.location == NSNotFound)
//		ninethArg = [[[NSString stringWithString:@"\""] stringByAppendingString:curAppName] stringByAppendingString:@"\""];
//	else
		
	[args addObject:@"-a"];
	[args addObject:ninethArg];
	
	NSMutableData *data = [[NSMutableData alloc] init];
	NSMutableData *errData = [[NSMutableData alloc] init];
	
	// create and launching pntsc task
	PntscTask *pntscT = [[PntscTask alloc] init];
	int launchStatus  = [pntscT launchPntscWithArgs:args outData:data errorData:errData];
	
	[pntscT release];
	if(launchStatus != 0)
	{
		NSString *errMessage = [[NSString alloc] initWithData: errData encoding: NSASCIIStringEncoding];
		[errorMessage setStringValue:[errMessage autorelease]];
#ifndef ENABLE_DESKTOP_INTEGRATION
		[NSApp beginSheet:errorLaunchingPanel modalForWindow:mainWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
#else
		[NSApp beginSheet:errorLaunchingPanel modalForWindow:deskIntegrationPanel modalDelegate:self didEndSelector:nil contextInfo:nil];
#endif
	}
	[data release];
	[errData release];
}

- (int)launcAppWithName:(NSString *)input {
	NSMutableArray *args = [NSMutableArray array];
	
	for(int i=0; i != [tabView numberOfTabViewItems]; ++i)
	{
		m_assistsObserver = [m_assistsArray objectAtIndex:i];
		[m_assistsObserver rDParam:args];
	}
	[args addObject:@"-a"];
	[args addObject:input];
	
	NSMutableData *data = [[NSMutableData alloc] init];
	NSMutableData *errData = [[NSMutableData alloc] init];
	
	// create and launching pntsc task
	PntscTask *pntscT = [[PntscTask alloc] init];
	int launchStatus = [pntscT launchPntscWithArgs:args outData:data errorData:errData];
	[pntscT release];

	if(launchStatus != 0)
	{
		NSString *errMessage = [[NSString alloc] initWithData: errData encoding: NSASCIIStringEncoding];
		[errorMessage setStringValue:[errMessage autorelease]];
#ifndef ENABLE_DESKTOP_INTEGRATION
		[NSApp beginSheet:errorLaunchingPanel modalForWindow:mainWnd modalDelegate:self didEndSelector:nil contextInfo:nil];
#else
		[NSApp beginSheet:errorLaunchingPanel modalForWindow:deskIntegrationPanel modalDelegate:self didEndSelector:nil contextInfo:nil];
#endif
	}
	[data release];
	[errData release];
	return launchStatus;
}

- (BOOL)createChangePassRequest:(NSMutableArray *)input {
    NSString *sr = [[m_assistsArray objectAtIndex:PassManageTabAssistIndex] passManagementRequest];
	NSString *passManageServer = sr ? [NSString stringWithString: sr] : nil;
    
	if (passManageServer == nil)
		return NO;
    
	[input addObject:passManageServer];
	m_assistsObserver = [m_assistsArray objectAtIndex:CredentialsTabAssistIndex];
	[m_assistsObserver rDParam:input];
    
    return YES;
}

- (void)createConnectArgsRequest:(NSMutableArray *)input {
	m_assistsObserver = [m_assistsArray objectAtIndex:ConnectivityTabAssistIndex];
	[m_assistsObserver rDParam:input];
}

#pragma mark -
#pragma mark Sly 'Look-Like-A-List' Buttons Actions

// TODO: liza: kill this horror, in future, light and beautiful future
- (IBAction)goToTab0:(id)sender {
	[tabView selectTabViewItemAtIndex:0];
}

- (IBAction)goToTab1:(id)sender {
	[tabView selectTabViewItemAtIndex:1];
}

- (IBAction)goToTab2:(id)sender {
	[tabView selectTabViewItemAtIndex:2];
}

- (IBAction)goToTab3:(id)sender {
	[tabView selectTabViewItemAtIndex:3];
}

- (IBAction)goToTab4:(id)sender {
	[tabView selectTabViewItemAtIndex:4];
}

- (IBAction)goToTab5:(id)sender {
	[tabView selectTabViewItemAtIndex:5];
}

- (IBAction)goToTab6:(id)sender {
	[tabView selectTabViewItemAtIndex:6];
}

- (IBAction)goToTab7:(id)sender {
	[tabView selectTabViewItemAtIndex:7];
}

- (IBAction)goToTab8:(id)sender {
	[tabView selectTabViewItemAtIndex:8];
}

- (IBAction)goToTab9:(id)sender {
	[tabView selectTabViewItemAtIndex:9];
}

- (IBAction)goToTab10:(id)sender {
	[tabView selectTabViewItemAtIndex:10];
}

#pragma mark -
#pragma mark Other Actions

- (void)setCurrentMode {
	if(![connectionArray canRemove])
	{
		[stateMatrix selectCellAtRow:0 column:0];
		[self setCreateMode:self];
	}
	else
	{
		[stateMatrix selectCellAtRow:1 column:0];
		[self setModifyMode:self];
	}
}

- (void)setInitialManageFarmState {
	//[self initTabAssistsArray]; // here memory is allocated, that is deallocated in class dealloc method
	
	m_currentTabIndex = 0;
	[tabView selectFirstTabViewItem:self];
	
	// TODO: the following is the fix of a bug with not cleared farm name. It needs to be refactored in future set default tabs state - clearTabs fnc(clear all excess)
	m_assistsObserver = [m_assistsArray objectAtIndex:m_currentTabIndex];
	[m_assistsObserver setDefaultTabState];
	[[m_assistsArray objectAtIndex:CredentialsTabAssistIndex] setDefaultTabState];
	//[self clearTabs];
}

- (void)initTabs {
	[[m_assistsArray objectAtIndex:ConnectivityTabAssistIndex] initGUI];
	[[m_assistsArray objectAtIndex:CredentialsTabAssistIndex] initGUI];
	[[m_assistsArray objectAtIndex:DisplayTabAssistIndex] initGUI];
}

- (void)clearTabs {
	for(int i=0; i != [tabView numberOfTabViewItems]; ++i)
	{
		m_assistsObserver = [m_assistsArray objectAtIndex:i];
		[m_assistsObserver setDefaultTabState];
	}
}

- (void)createNewConnection {

	
	// create a connection
	// do NOT release m_currentConnectionItem - we'll do it on Finish (or Cancel) 
	m_currentConnectionItem = [[Connection alloc] init];
	[m_currentConnectionItem setValue:[[m_assistsArray objectAtIndex:0] getFarmName] forKey:@"farmName"];
	[connectionArray addObject:m_currentConnectionItem];
	
	// create default locations for this connection
	for (int i = 0; i < numberOfLocations; ++i)
	{
		Location *currentLocation = [[Location alloc] init];
		[currentLocation setValue:[defaultLocationName stringByAppendingString:[[NSNumber numberWithInt:i + 1] stringValue]] forKey:@"name"];
		[locationArray addObject:currentLocation];
		[currentLocation release];
	}
	[locationArray setSelectionIndex:0];
}

- (void)closeWnd:(NSWindow *) wndClose {
	[NSApp stopModal];
	[wndClose close];
}

- (void)closeSheet:(NSWindow *) wndClose {
	[NSApp endSheet:wndClose];
}

#pragma mark -
#pragma mark Saving and Loading data

- (NSString *)pathForDataFile {
	NSFileManager *fileManager = [NSFileManager defaultManager];
	NSString *folder = configPath;
    
	folder = [folder stringByExpandingTildeInPath];
    
	if ([fileManager fileExistsAtPath: folder] == NO)
		[fileManager createDirectoryAtPath: folder withIntermediateDirectories: YES attributes: nil error: NULL];
    
	NSString *fileName = configFileName;
    
	return [folder stringByAppendingPathComponent: fileName];
}

- (void)saveCredentials
{
    Connection *conn;
    
    for (conn in m_connections)
    {
        NSString *username = [[conn credentials] username];
        NSString *password = [[conn credentials] password];
        char *passwordData = nil;
        UInt32 passwordLength;
        SecKeychainItemRef itemRef = nil;
        OSStatus status;
        
        if (!username)
            continue;
        
        status = SecKeychainFindGenericPassword(NULL, 9, "AppPortal", [username length], [username UTF8String], &passwordLength, (void **)&passwordData, &itemRef);
        
        if (noErr == status)
            status = SecKeychainItemModifyAttributesAndData(itemRef, NULL, [password length], [password UTF8String]);
        else
            status = SecKeychainAddGenericPassword(NULL, 9, "AppPortal", [username length], [username UTF8String], [password length], [password UTF8String], NULL);
        
        [[conn credentials] setPassword: @""];
        
        SecKeychainItemFreeContent(NULL, passwordData);
        
        if (itemRef)
            CFRelease(itemRef);
    }
}

- (void)loadCredentials
{
    Connection *conn;
    
    for (conn in m_connections)
    {
        NSString *username = [[conn credentials] username];
        char *passwordData = nil;
        UInt32 passwordLength;
        SecKeychainItemRef itemRef = nil;
        OSStatus status;
        
        if (nil == username)
            continue;
        
        status = SecKeychainFindGenericPassword(NULL, 9, "AppPortal", [username length], [username UTF8String], &passwordLength, (void **)&passwordData, &itemRef);
        
        if (noErr == status )
            passwordData[passwordLength] = 0;
        else
            passwordData = NULL;
        
        [[conn credentials] setPassword: passwordData ? [NSString stringWithUTF8String: passwordData] : @""];
        
        SecKeychainItemFreeContent(NULL, passwordData);
        
        if (itemRef)
            CFRelease(itemRef);
    }
}

- (void)saveDataToDisk {
	NSString * path = [self pathForDataFile];
	NSMutableDictionary * rootObject;
    
	rootObject = [NSMutableDictionary dictionary];
	[rootObject setValue: [self connections] forKey:@"m_connections"];
    
    [self saveCredentials];
    
	[NSKeyedArchiver archiveRootObject: rootObject toFile: path]; 
    
    [self loadCredentials];
}

- (void)loadDataFromDisk {
	NSString * path = [NSString stringWithString:[self pathForDataFile]];
	NSDictionary * rootObject;
    
	rootObject = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
    
	[self setConnections: [rootObject valueForKey:@"m_connections"]];
    
    [self loadCredentials];
}


#pragma mark -
#pragma mark X11

- (IBAction)createXWnd:(id)sender {
	//[self initX11Wnd];
}

@end
