
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

//
//  PartialFormatter.m
//  FilteringFieldTest
//
//  Created by Scott Gruby on 3/30/08.
//  Copyright 2008 GGT Enterprises, LLC. All rights reserved.
//

#import "PartialFormatter.h"


@implementation PartialFormatter
- (void) dealloc
{
	[filterCharacterSet release];
	[super dealloc];
}

- (BOOL)isPartialStringValid:(NSString *)partialString newEditingString:(NSString **)newString errorDescription:(NSString **)error {
	if (filterCharacterSet != nil)
	{
		int i = 0;
		int len = [partialString length];
		for (i = 0; i < len; i++)
		{
			if (![filterCharacterSet characterIsMember:[partialString characterAtIndex:i]])
			{
				return NO;
			}
		}
	}

	if (maxLength)
	{
		if ([partialString length] > maxLength)
		{
			return NO;
		}
	}

	if (maxValue && [partialString intValue] > maxValue)
	{
		return NO;
	}
	
	if (minLength)
	{
		if ([partialString length] < minLength)
		{
			return NO;
		}
	}
	
	if (minValue && [partialString intValue] < minValue)
	{
		return NO;
	}
	
	if([partialString intValue] == 0 && ![partialString isEqualToString:@""])
		return NO;
	
	return YES;
}

- (void) setAcceptableCharacterSet:(NSCharacterSet *) inCharacterSet
{
	[inCharacterSet retain];
	[filterCharacterSet release];
	filterCharacterSet = inCharacterSet;
}

- (void) setMaximumLength:(int) inLength
{
	maxLength = inLength;
}

- (void) setMaximumValue:(int) inValue
{
	maxValue = inValue;
}

- (void) setMinimumLength:(int) inLength
{
	minLength = inLength;
}

- (void) setMinimumValue:(int) inValue
{
	minValue = inValue;
}

- (NSString *)stringForObjectValue:(id)anObject
{
	if ([anObject isKindOfClass:[NSString class]])
	{
		return anObject;
	}
	
	return nil;
}

- (BOOL)getObjectValue:(id *)anObject forString:(NSString *)string errorDescription:(NSString **)error
{
	if (anObject && string)
	{
		*anObject = [NSString stringWithString:string];
	}
	
	return YES;
}
@end
