
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

//
//  PntscTask.m
//  Provision AppPortal
//
//  Created by lizard on 5/14/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "PntscTask.h"


@implementation PntscTask

#pragma mark -
#pragma mark Initialization

-(id)init {
	if(self = [super init])
	{
		pntscTask = [[NSTask alloc] init];
		launchPath = [[NSString alloc] init];
		
		stdErrPipe = [NSPipe pipe],
		stdOutPipe = [NSPipe pipe];
		stdErrReadHandle = [stdErrPipe fileHandleForReading],
		stdOutReadHandle = [stdOutPipe fileHandleForReading];
		
	}
	return self;
}

- (void)dealloc {
	[pntscTask release];
	[launchPath release];
	[super dealloc];
}

#pragma mark -
#pragma mark Pseudo getters

- (NSTask *)pntscTask {
//	[self setEnvironment];
	return pntscTask;
}

- (NSData *)outData {
	return [stdOutReadHandle availableData];
}

- (NSData *)errorData {
	return [stdErrReadHandle availableData];
}

#pragma mark -
#pragma mark Pseudo setters

- (void)setEnvironment {
	[pntscTask setLaunchPath:[self launchPath]];
	[pntscTask setStandardError:stdErrPipe];
	[pntscTask setStandardOutput:stdOutPipe];
    
    NSMutableDictionary *environment = [NSMutableDictionary dictionaryWithCapacity: 10];
    
    [environment addEntriesFromDictionary: [[NSProcessInfo processInfo] environment]];
    [environment setObject: [[NSLocale currentLocale] localeIdentifier] forKey: @"LANG"];
    
    [pntscTask setEnvironment: environment];
}

- (NSString *)launchPath {
	[launchPath autorelease];
	
	NSString *allUsersPath = [NSString stringWithString:@"/usr/bin/pntsc"];
	NSString *userHome = [self userHomeDir];
	userHome = [userHome stringByAppendingString:@"/pntsc"];
	
	NSFileManager *pntscPath = [NSFileManager defaultManager];
	if([pntscPath fileExistsAtPath:userHome])
		launchPath = [userHome retain];
	else
		launchPath = [allUsersPath retain];
	return launchPath;
}

- (NSString *)userHomeDir {
	
	// getting current user name
	NSTask *whoamiTask = [[NSTask alloc] init];
	[whoamiTask setLaunchPath:@"/usr/bin/whoami"];
	
	NSPipe *whoamiStdOutPipe = [NSPipe pipe];
    NSFileHandle *whoamiStdOutReadHandle = [whoamiStdOutPipe fileHandleForReading];

	[whoamiTask setStandardOutput:whoamiStdOutPipe];
	[whoamiTask launch];
	
	NSData *whoamiData = [whoamiStdOutReadHandle availableData];
	NSString *string = [[NSString alloc] initWithData: whoamiData encoding: NSASCIIStringEncoding];
	NSArray *stringsArr = [string componentsSeparatedByString:@"\n"];

	[string release];
	[whoamiTask release];
	
	for(int i = 0; i != [stringsArr count]; ++i)
	{
		if([[stringsArr objectAtIndex:i] isEqualToString:@""])
			continue;
		else
			return [NSString stringWithFormat:@"/Users/%@", [stringsArr objectAtIndex:i]];
	}
	return [NSString string];
}

#pragma mark -
#pragma mark Launching Applications

void composeProxyRequestFromUrl(char* url)
{
	char *p, *q, *r;
	int port=80;
//	int brokerPort = 80;
	char *targ = 0, host[BUFSIZ];
	bool isProxy = false;
	
    p = url + 7;
	if((q=strstr(p, "PROXY")))
	{
		isProxy = true;
		r = q;
		*r = '\0';
		p = q + 5;
	}
	if (!(q=strchr(p,'/')))
    {
		return;
    }
	
    memcpy(host,p,q-p);
    host[q-p]='\0';
    if ((q=strchr(host,':')))
    {
        port=atoi(q+1);
        *q='\0';
    }
	
	if(!isProxy)
		targ=q;
	else
	{
		targ = url + 7;

/*		if ((r=strchr(targ,':')))
		{
			brokerPort=atoi(r+1);
			*r='\0';
		}
 */
	}
}

- (int)launchPntscWithArgs:(NSArray *)input outData:(NSMutableData *)data errorData:(NSMutableData *)errData
{
	[pntscTask setArguments:input];
	[self setEnvironment];
	
	// test proxy parsing
/*	char proxyReq[BUFSIZ];
	NSString *broker = [NSString stringWithString:[input objectAtIndex:0]];
	strcpy (proxyReq, [broker UTF8String]);
	strcat(proxyReq, "/");

	composeProxyRequestFromUrl(proxyReq);
*/
	[pntscTask launch];
	
	[pntscTask waitUntilExit];
	int status = [pntscTask terminationStatus];
	
	if (status == 0)
	{
		NSData *curData;
		do
		{
			curData = [NSData dataWithData:[stdOutReadHandle availableData]];
			[data appendData:curData];
		}
		while(curData != nil && [curData length] != 0);
	}
//	else
	{
		NSData *curData;
		do
		{
			curData = [NSData dataWithData:[stdErrReadHandle availableData]];
			[errData appendData:curData];
		}
		while(curData != nil && [curData length] != 0);
	}

	//NSString *string = [[NSString alloc] initWithData:data encoding: NSASCIIStringEncoding];
	

	return status;
	
}
@end
