
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

//
//  Resources.m
//  Provision AppPortal
//
//  Created by smalti on 4/10/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "Resources.h"

@implementation Resources

#pragma mark -
#pragma mark Initialization

- (id)init {
	if(self = [super init])
	{
		sound = [[NSString alloc] initWithString:@"Bring to Local Computer"];
		keyboard = [[NSString alloc] initWithString:@"On the local computer"];
		
		// booleans
		doRedirectDiskDrives = NO;
		doRedirectSerialPorts = NO;
		doRedirectParallelPorts = NO;
		doRedirectClipboard = NO;
		doRedirectPrinters = NO;
		doRedirectSmartcards = NO;
	}
	return self;
}

- (id)initWithResource:(const Resources *)input {
	if(self = [super init])
	{
		[self setSound:[input sound]];
		[self setKeyboard:[input keyboard]];
		// booleans
		[self setDoRedirectDiskDrives:[input doRedirectDiskDrives]];
		[self setDoRedirectSerialPorts:[input doRedirectSerialPorts]];
		[self setDoRedirectParallelPorts:[input doRedirectParallelPorts]];
		[self setDoRedirectClipboard:[input doRedirectClipboard]];
		[self setDoRedirectPrinters:[input doRedirectPrinters]];
		[self setDoRedirectSmartcards:[input doRedirectSmartcards]];
	}
	return self;
}

- (void)dealloc {
	[sound release];
	[keyboard release];
	[super dealloc];
}

#pragma mark -
#pragma mark Setters

- (void)setSound:(NSString *)input {
	[sound autorelease];
	sound = [input retain];
}

- (void)setKeyboard:(NSString *)input {
	[keyboard autorelease];
	keyboard = [input retain];
}

- (void)setDoRedirectDiskDrives:(BOOL)input {
	doRedirectDiskDrives = input;
}

- (void)setDoRedirectSerialPorts:(BOOL)input {
	doRedirectSerialPorts = input;
}

- (void)setDoRedirectParallelPorts:(BOOL)input {
	doRedirectParallelPorts = input;
}

- (void)setDoRedirectClipboard:(BOOL)input {
	doRedirectClipboard = input;
}

- (void)setDoRedirectPrinters:(BOOL)input {
	doRedirectPrinters = input;
}

- (void)setDoRedirectSmartcards:(BOOL)input {
	doRedirectSmartcards = input;
}

#pragma mark -
#pragma mark Getters

- (NSString *)sound {
	return sound;
}

- (NSString *)keyboard {
	return keyboard;
}

- (BOOL)doRedirectDiskDrives {
	return doRedirectDiskDrives;
}

- (BOOL)doRedirectSerialPorts {
	return doRedirectSerialPorts;
}

- (BOOL)doRedirectParallelPorts {
	return doRedirectParallelPorts;
}

- (BOOL)doRedirectClipboard {
	return doRedirectClipboard;
}

- (BOOL)doRedirectPrinters {
	return doRedirectPrinters;
}

- (BOOL)doRedirectSmartcards {
	return doRedirectSmartcards;
}

#pragma mark -
#pragma mark Encode-Decode

- (void)encodeWithCoder:(NSCoder *)coder {
	[coder encodeObject:sound forKey:@"sound"];
	[coder encodeObject:keyboard forKey:@"keyboard"];
	// booleans
	[coder encodeBool:doRedirectDiskDrives forKey:@"doRedirectDiskDrives"];
	[coder encodeBool:doRedirectSerialPorts forKey:@"doRedirectSerialPorts"];
	[coder encodeBool:doRedirectParallelPorts forKey:@"doRedirectParallelPorts"];
	[coder encodeBool:doRedirectClipboard forKey:@"doRedirectClipboard"];
	[coder encodeBool:doRedirectPrinters forKey:@"doRedirectPrinters"];
	[coder encodeBool:doRedirectSmartcards forKey:@"doRedirectSmartcards"];
}

- (id)initWithCoder:(NSCoder *)coder {
	if(self = [super init])
	{
		[self setSound:[coder decodeObjectForKey:@"sound"]];
		[self setKeyboard:[coder decodeObjectForKey:@"keyboard"]];
		// booleans
		[self setDoRedirectDiskDrives:[coder decodeBoolForKey:@"doRedirectDiskDrives"]];
		[self setDoRedirectSerialPorts:[coder decodeBoolForKey:@"doRedirectSerialPorts"]];
		[self setDoRedirectParallelPorts:[coder decodeBoolForKey:@"doRedirectParallelPorts"]];
		[self setDoRedirectClipboard:[coder decodeBoolForKey:@"doRedirectClipboard"]];
		[self setDoRedirectPrinters:[coder decodeBoolForKey:@"doRedirectPrinters"]];
		[self setDoRedirectSmartcards:[coder decodeBoolForKey:@"doRedirectSmartcards"]];
	}
	return self;
}

@end
