
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#import "UserExpTabAssist.h"

@implementation UserExpTabAssist

#pragma mark -
#pragma mark Initialization

static id sharedInstance = nil;

+ (id)sharedInstance {
	if (sharedInstance == nil)
		sharedInstance = [[self alloc] init];
	return sharedInstance;
}

-(id)init {
	if(sharedInstance) {
		[NSException raise:NSInternalInconsistencyException
					format:@"[%@] cannot be called; use +[%@ %@] instead",
		 NSStringFromClass([self class]), 
		 NSStringFromClass([self class]),
		 NSStringFromSelector(@selector(sharedInstance))];							 
	}	
	else
	{
		sharedInstance = [super init];
		{
			NSArray *keys = [NSArray arrayWithObjects:@"Modem (28.8 Kbps)", @"Modem (56 Kbps)", @"Modem (128 Kbps - 1.5 Mbps)",
														@"LAN (10 Mbps or higher)", @"Custom", nil];
			
			NSArray *values = [NSArray arrayWithObjects:@"-P", @"0x07", @"0x03", @"0x00", @"", nil];
			modemsDict = [[NSMutableDictionary alloc] initWithObjects:values forKeys:keys];
		}
	}
	return sharedInstance;
}

- (void)dealloc {
	[modemsDict release];
	[super dealloc];
}

#pragma mark -
#pragma mark Actions

- (IBAction)doSelectCustomSpeed:(id)sender {
	NSString *curConSpeed = [NSString stringWithString:[[[connectionArray selection] valueForKey:@"performance"] connectionSpeed]];
	if ([curConSpeed isEqualToString:@"Modem (28.8 Kbps)"])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseBitmapCaching:YES];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowDesktopBackground:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseAnimation:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowWindowContent:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseThemes:NO];		
	}
	if([curConSpeed isEqualToString:@"Modem (56 Kbps)"])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseBitmapCaching:YES];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowDesktopBackground:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseAnimation:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowWindowContent:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseThemes:YES];
	}
	if([curConSpeed isEqualToString:@"Modem (128 Kbps - 1.5 Mbps)"])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseBitmapCaching:YES];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowDesktopBackground:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseAnimation:YES];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowWindowContent:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseThemes:YES];
	}
	if([curConSpeed isEqualToString:@"LAN (10 Mbps or higher)"])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseBitmapCaching:YES];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowDesktopBackground:YES];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseAnimation:YES];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowWindowContent:YES];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseThemes:YES];
	}
	if ([curConSpeed isEqualToString:@"Custom"])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseBitmapCaching:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowDesktopBackground:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseAnimation:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoShowWindowContent:NO];
		[[[connectionArray selection] valueForKey:@"performance"] setDoUseThemes:NO];		
	}
}

- (IBAction)doCheckUserExpOption:(id)sender {
	if([[[connectionArray selection] valueForKey:@"performance"] doUseBitmapCaching] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doShowDesktopBackground] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doUseAnimation] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doShowWindowContent] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doUseThemes])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setConnectionSpeed:@"Modem (28.8 Kbps)"];
		return;
	}
	if([[[connectionArray selection] valueForKey:@"performance"] doUseBitmapCaching] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doShowDesktopBackground] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doUseAnimation] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doShowWindowContent] && 
	   [[[connectionArray selection] valueForKey:@"performance"] doUseThemes])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setConnectionSpeed:@"Modem (56 Kbps)"];
		return;
	}
	if([[[connectionArray selection] valueForKey:@"performance"] doUseBitmapCaching] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doShowDesktopBackground] && 
	   [[[connectionArray selection] valueForKey:@"performance"] doUseAnimation] && 
	   ![[[connectionArray selection] valueForKey:@"performance"] doShowWindowContent] && 
	   [[[connectionArray selection] valueForKey:@"performance"] doUseThemes])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setConnectionSpeed:@"Modem (128 Kbps - 1.5 Mbps)"];
		return;
	}
	if([[[connectionArray selection] valueForKey:@"performance"] doUseBitmapCaching] && 
	   [[[connectionArray selection] valueForKey:@"performance"] doShowDesktopBackground] && 
	   [[[connectionArray selection] valueForKey:@"performance"] doUseAnimation] && 
	   [[[connectionArray selection] valueForKey:@"performance"] doShowWindowContent] && 
	   [[[connectionArray selection] valueForKey:@"performance"] doUseThemes])
	{
		[[[connectionArray selection] valueForKey:@"performance"] setConnectionSpeed:@"LAN (10 Mbps or higher)"];
		return;
	}
	[[[connectionArray selection] valueForKey:@"performance"] setConnectionSpeed:@"Custom"];
}

#pragma mark -
#pragma mark Overloaded methods

- (void)setDefaultTabState {
}

- (BOOL)isDataCorrect {
	return YES;
}

- (void)rDParam:(NSMutableArray *)input {
	NSString *mArg = nil;
	NSString *tmp = nil;
	NSString *curSpeed = [NSString stringWithString:[modemsDict valueForKey:[[[connectionArray selection] valueForKey:@"performance"] connectionSpeed]]];
    
	if([curSpeed isEqualToString:[modemsDict valueForKey:@"Custom"]])
	{
		BOOL doUseBitmap = [[[connectionArray selection] valueForKey:@"performance"] doUseBitmapCaching];
		BOOL doShowDesktop = [[[connectionArray selection] valueForKey:@"performance"] doShowDesktopBackground];
		BOOL doUseAnime = [[[connectionArray selection] valueForKey:@"performance"] doUseAnimation];
		BOOL doShowWindow = [[[connectionArray selection] valueForKey:@"performance"] doShowWindowContent];
		BOOL doUseTheme = [[[connectionArray selection] valueForKey:@"performance"] doUseThemes];
		long flagsVal = 0;
		if(doUseBitmap)
		{
			curSpeed = [modemsDict valueForKey:@"Modem (28.8 Kbps)"];
			mArg = [NSString stringWithString:curSpeed];
		}
		if(!doShowDesktop)
			flagsVal |= PERF_DISABLE_WALLPAPER;
		if(!doUseAnime)
			flagsVal |= PERF_DISABLE_MENUANIMATIONS;
		if(!doShowWindow)
			flagsVal |= PERF_DISABLE_FULLWINDOWDRAG;
		if(!doUseTheme)
			flagsVal |= PERF_DISABLE_THEMING;
		
        if(!mArg)
            mArg = [[NSString stringWithString:@"-x "] stringByAppendingString:[NSString stringWithFormat:@"0x%x", flagsVal]];
        else
            mArg = [[mArg stringByAppendingString:@" -x "] stringByAppendingString:[NSString stringWithFormat:@"0x%x", flagsVal]];
		
		if(mArg && ![mArg isEqualToString:@""])
		{
			if (tmp = [NSString stringWithString:[input lastObject]])
			{
				[input removeLastObject];
				tmp = [[tmp stringByAppendingString:mArg] stringByAppendingString:@" "];
			}
		}
	}
	else 
	{
		if ([curSpeed isEqualToString:[modemsDict valueForKey:@"Modem (28.8 Kbps)"]])
			mArg = [NSString stringWithString:curSpeed];
		else
			mArg = [[[NSString stringWithString:[modemsDict valueForKey:@"Modem (28.8 Kbps)"]]
					 stringByAppendingString:@" -x "] stringByAppendingString:curSpeed];
		
		if (tmp = [NSString stringWithString:[input lastObject]])
		{
			[input removeLastObject];
			tmp = [[tmp stringByAppendingString:mArg] stringByAppendingString:@" "];
		}
	}
    
	if([[[connectionArray selection] valueForKey:@"performance"] doUseGA])
	{
		tmp = [[tmp stringByAppendingString:@"--gaenable"] stringByAppendingString:@" "];
	}
    
    if([[[connectionArray selection] valueForKey:@"performance"] doFlashMMR])
	{
		tmp = [[tmp stringByAppendingString:@"-r addin:PNFlashMMR:/usr/bin/fmmr"] stringByAppendingString:@" "];
	}
    
    if (tmp)
        [input addObject:tmp];
}

@end
