
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#ifndef PNRDP_H
#define PNRDP_H

#include "rdesktop.h"
#include <assert.h>


// PNRDP protocol declarations
#define PNRDP_NEG_MESSAGE_LEN   16
#define PNRDP_NEG_REQ_STRING    "CHECK"
#define PNRDP_NEG_RSP_STRING    "READY"
#define PNRDP_NEG_CNF_STRING    "OK"
#define PNRDP_EMFP_COMMENT_STRING   "PNRDP"
#define PNRDP_EMFP_COMMENT_TYPE     0x4003

enum RDP_ALTSEC_DRAWING_ORDER_TYPE
{
	TS_ALTSEC_SWITCH_SURFACE = 0x00,
	TS_ALTSEC_CREATE_OFFSCR_BITMAP = 0x01,
	TS_ALTSEC_STREAM_BITMAP_FIRST = 0x02,
	TS_ALTSEC_STREAM_BITMAP_NEXT = 0x03,
	TS_ALTSEC_CREATE_NINEGRID_BITMAP = 0x04,
	TS_ALTSEC_GDIP_FIRST = 0x05,
	TS_ALTSEC_GDIP_NEXT = 0x06,
	TS_ALTSEC_GDIP_END = 0x07,
	TS_ALTSEC_GDIP_CACHE_FIRST = 0x08,
	TS_ALTSEC_GDIP_CACHE_NEXT = 0x09,
	TS_ALTSEC_GDIP_CACHE_END = 0x0a,
	TS_ALTSEC_WINDOW = 0x0b,
	TS_ALTSEC_COMPDESK_FIRST = 0x0c
};


// EMF+ protocol declarations
enum
{
    ImageDataTypeUnknown = 0x00000000L,
    ImageDataTypeBitmap = 0x00000001L,
    ImageDataTypeMetafile = 0x00000002L
};

enum
{
    RegionNodeDataTypeAnd = 0x00000000L,
    RegionNodeDataTypeOr = 0x00000001L,
    RegionNodeDataTypeXor = 0x00000002L,
    RegionNodeDataTypeExclude = 0x00000003L,
    RegionNodeDataTypeComplement = 0x00000004L,
    RegionNodeDataTypeRect = 0x10000000L,
    RegionNodeDataTypePath = 0x10000001L,
    RegionNodeDataTypeEmpty = 0x10000002L,
    RegionNodeDataTypeInfinite = 0x10000003L
};

enum
{
    CombineModeReplace = 0x00000000L,
    CombineModeIntersect = 0x00000001L,
    CombineModeUnion = 0x00000002L,
    CombineModeXor = 0x00000003L,
    CombineModeExclude = 0x00000004L,
    CombineModeComplement = 0x00000005L
};

enum
{
    PathPointTypeDashMode = 0x01,
    PathPointTypePathMarker = 0x02,
    PathPointTypeCloseSubpath = 0x08
};

enum
{
    PathPointTypeStart = 0x00,
    PathPointTypeLine = 0x01,
    PathPointTypeBezier = 0x03
};

enum
{
    BitmapDataTypePixel = 0x00000000L,
    BitmapDataTypeCompressed = 0x00000001L
};

enum
{
    ObjectTypeInvalid = 0x00000000,
    ObjectTypeBrush = 0x00000001,
    ObjectTypePen = 0x00000002,
    ObjectTypePath = 0x00000003,
    ObjectTypeRegion = 0x00000004,
    ObjectTypeImage = 0x00000005,
    ObjectTypeFont = 0x00000006,
    ObjectTypeStringFormat = 0x00000007,
    ObjectTypeImageAttributes = 0x00000008,
    ObjectTypeCustomLineCap = 0x00000009
};

typedef enum {
    EmfPlusHeader = 0x4001,
    EmfPlusEndOfFile = 0x4002,
    EmfPlusComment = 0x4003,
    EmfPlusGetDC = 0x4004,
    EmfPlusMultiFormatStart = 0x4005,
    EmfPlusMultiFormatSection = 0x4006,
    EmfPlusMultiFormatEnd = 0x4007,
    EmfPlusObject = 0x4008,
    EmfPlusClear = 0x4009,
    EmfPlusFillRects = 0x400A,
    EmfPlusDrawRects = 0x400B,
    EmfPlusFillPolygon = 0x400C,
    EmfPlusDrawLines = 0x400D,
    EmfPlusFillEllipse = 0x400E,
    EmfPlusDrawEllipse = 0x400F,
    EmfPlusFillPie = 0x4010,
    EmfPlusDrawPie = 0x4011,
    EmfPlusDrawArc = 0x4012,
    EmfPlusFillRegion = 0x4013,
    EmfPlusFillPath = 0x4014,
    EmfPlusDrawPath = 0x4015,
    EmfPlusFillClosedCurve = 0x4016,
    EmfPlusDrawClosedCurve = 0x4017,
    EmfPlusDrawCurve = 0x4018,
    EmfPlusDrawBeziers = 0x4019,
    EmfPlusDrawImage = 0x401A,
    EmfPlusDrawImagePoints = 0x401B,
    EmfPlusDrawstring = 0x401C,
    EmfPlusSetRenderingOrigin = 0x401D,
    EmfPlusSetAntiAliasMode = 0x401E,
    EmfPlusSetTextRenderingHint = 0x401F,
    EmfPlusSetTextContrast = 0x4020,
    EmfPlusSetInterpolationMode = 0x4021,
    EmfPlusSetPixelOffsetMode = 0x4022,
    EmfPlusSetCompositingMode = 0x4023,
    EmfPlusSetCompositingQuality = 0x4024,
    EmfPlusSave = 0x4025,
    EmfPlusRestore = 0x4026,
    EmfPlusBeginContainer = 0x4027,
    EmfPlusBeginContainerNoParams = 0x4028,
    EmfPlusEndContainer = 0x4029,
    EmfPlusSetWorldTransform = 0x402A,
    EmfPlusResetWorldTransform = 0x402B,
    EmfPlusMultiplyWorldTransform = 0x402C,
    EmfPlusTranslateWorldTransform = 0x402D,
    EmfPlusScaleWorldTransform = 0x402E,
    EmfPlusRotateWorldTransform = 0x402F,
    EmfPlusSetPageTransform = 0x4030,
    EmfPlusResetClip = 0x4031,
    EmfPlusSetClipRect = 0x4032,
    EmfPlusSetClipPath = 0x4033,
    EmfPlusSetClipRegion = 0x4034,
    EmfPlusOffsetClip = 0x4035,
    EmfPlusDrawDriverstring = 0x4036,
    EmfPlusStrokeFillPath = 0x4037,
    EmfPlusSerializableObject = 0x4038,
    EmfPlusSetTSGraphics = 0x4039,
    EmfPlusSetTSClip = 0x403A
} RecordType;

#define EMFP_OBJECT_TABLE_SIZE  64

//----- PNRDP EMF+ Comment record ------------

typedef struct _EMFP_PNRDP_COMMENT
{
    uint16 type;
    uint16 flags;
    uint32 size;
    unsigned char *data;
} EMFP_PNRDP_COMMENT;


//----- EMF+ Region Object ------------

typedef struct _EMFP_RECT
{
    uint16 x;
    uint16 y;
    uint16 width;
    uint16 height;
} EMFP_RECT;

typedef struct _EMFP_RECTF
{
    float x;
    float y;
    float width;
    float height;
} EMFP_RECTF;

typedef struct _EMFP_POINT
{
    uint16 x;
    uint16 y;
} EMFP_POINT;

typedef struct _EMFP_POINTF
{
    float x;
    float y;
} EMFP_POINTF;

typedef struct _EMFP_POINTR
{
    union {
        int8_t int7_val;
        int16_t int15_val;
    } x;
    union {
        int8_t int7_val;
        int16_t int15_val;
    } y;
} EMFP_POINTR;

typedef union
{
    EMFP_POINT point;
    EMFP_POINTF pointf;
    EMFP_POINTR pointr;
} EMFP_PATH_POINT;

typedef struct __attribute__((__packed__))
{
    uint8 type : 4;
    uint8 flags : 4;
} EMFP_PATH_POINT_TYPE_NORMAL;

typedef struct __attribute__((__packed__))
{
    uint8 b : 1;
    uint8   : 1;
    uint8 run_count : 6;
    uint8 type;
} EMFP_PATH_POINT_TYPE_RLE;

typedef union __attribute__((__packed__))
{
    EMFP_PATH_POINT_TYPE_NORMAL normal;
    EMFP_PATH_POINT_TYPE_RLE rle;
} EMFP_PATH_POINT_TYPE;

typedef struct __attribute__((__packed__)) _EMFP_PATH
{
    uint32 version;
    uint32 pcount;
    union {
        uint16 flags;
        struct __attribute__((__packed__)) {
            uint16 : 11;
            uint16 pbit : 1;
            uint16 rbit : 1;
            uint16 : 1;
            uint16 cbit : 1;
            uint16 : 1;
        };
    };
    uint16 reserved;
    EMFP_PATH_POINT *points;
    EMFP_PATH_POINT_TYPE *types;
} EMFP_PATH;

typedef struct __attribute__((__packed__)) _EMFP_REGION_NODE
{
    uint32 type;
    uint32 _unknown;
    union __attribute__((__packed__)) {
        EMFP_PATH path;
        EMFP_RECTF rectf;
        struct __attribute__((__packed__)) {
            struct _EMFP_REGION_NODE *left;
            struct _EMFP_REGION_NODE *right;
        } childs;
    };
} EMFP_REGION_NODE;

typedef struct _EMFP_REGION_OBJECT
{
    uint32 version;
    uint32 ncount;
    EMFP_REGION_NODE *nodes;
} EMFP_REGION_OBJECT;


//----- EMF+ Image Object ------------

// this is for compressed images ONLY!
typedef struct _EMFP_COMPRESSED_IMAGE
{
    uint32 width;   // MUST be ignored
    uint32 height;   // MUST be ignored
    uint32 stride;   // MUST be ignored
    uint32 pixel_format;   // MUST be ignored
    uint32 type; // MUST be BitmapDataTypeCompressed
    unsigned char *cimage;
} EMFP_COMPRESSED_IMAGE;

typedef struct _EMFP_IMAGE_OBJECT
{
    uint32 version;
    uint32 type;
    unsigned char *data;
} EMFP_IMAGE_OBJECT;


// ----- EMF+ Object record ------------------

typedef struct __attribute__((__packed__)) _EMFP_OBJECT
{
    union {
        uint16 flags;
        struct __attribute__((__packed__)) {
            uint8 id;
            uint8 type : 7;
            uint8 c : 1;
        };
    };
    uint32 size;
//    uint32 datasize;
    unsigned char * data;
} EMFP_OBJECT;

// only for EmfPlusObject (NOT EmfPlusSerializableObject) records
typedef struct __attribute__((__packed__)) _EMFP_OBJECT_RECORD
{
    uint16 type;    // MUST be EmfPlusObject
    EMFP_OBJECT object;
} EMFP_OBJECT_RECORD;


// ----- EMF+ SetClipRegion and DrawImage records

typedef struct __attribute__((__packed__)) _EMFP_SET_CLIP_REGION_RECORD
{
    uint16 type;    // MUST be EmfPlusSetClipRegion
    union {
        uint16 flags;
        struct __attribute__((__packed__)) {
            uint8 object_id;
            uint8 combine_mode : 4;
            uint8 : 4;
        };
    };
    uint32 size;        // MUST be 0x0000000C, but PN sets it to 0x08
//    uint32 datasize;    // MUST be 0, but it's totally absent in PN's records
} EMFP_SET_CLIP_REGION_RECORD;

typedef struct __attribute__((__packed__)) _EMFP_DRAW_IMAGE_RECORD
{
    uint16 type;
    union {
        uint16 flags;
        struct __attribute__((__packed__)) {
            uint8 object_id;
            uint8 : 6;
            uint8 c : 1;
            uint8 : 1;
        };
    };
    uint32 size;
//    uint32 datasize;  // is absent in PN's records
    uint32 image_attr_id;   // not used
    int32_t src_unit;
    EMFP_RECTF src_rect;
    union{
        EMFP_RECT dst_rect;
        EMFP_RECTF dst_rectf;
    };
} EMFP_DRAW_IMAGE_RECORD;

#endif
