
/******************************************************************************
 *
 * � Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/

#ifndef PNSSLCLI_H
#define PNSSLCLI_H

#include <boost/aligned_storage.hpp>
#include <boost/array.hpp>
#include <boost/enable_shared_from_this.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/asio.hpp>
#include <boost/asio/ssl.hpp>

#include "handler_alloc.h"

using boost::asio::ip::tcp;

typedef boost::asio::ssl::stream < boost::asio::ip::tcp::socket > ssl_socket;

// ------------------------------------------------------------------------------
// session: representes user session
// ------------------------------------------------------------------------------
class session: public boost::enable_shared_from_this < session >
{
public:

    explicit session (boost::asio::io_service& io_service, boost::asio::ssl::context& context);

    // gets private member socket_
    tcp::socket& socket ();

    // gets private member ssl_socket_
    ssl_socket& sslsocket ();

    // starts proxy cycle
    void start (const char *rdphost, const char *username);

private:    
    // timer handler
    void on_timer (const boost::system::error_code& e);

    // asyncronous operations handlers
    void handle_read (const boost::system::error_code & error, size_t bytes_transferred);

    void handle_server_read (const boost::system::error_code & error, size_t bytes_transferred);

    void handle_write (const boost::system::error_code & error);

    void handle_server_write (const boost::system::error_code & error);
    
    // The socket used to communicate with the client.
    tcp::socket socket_;

    // The socket used to communicate with the remote server.
    ssl_socket  ssl_socket_;

    // Buffer used to store client data.
    boost::array < char, DATA_BUFFER_SIZE > data_in_;

    // Buffer used to store remote server data.
    boost::array < char, DATA_BUFFER_SIZE > data_out_;

    // The allocators to use for handler-based custom memory allocation.
    handler_allocator allocator_in_;
    handler_allocator allocator_out_;

    // The timer used to prevent of hanging server write operation 
    boost::asio::deadline_timer timer_;
};

#endif
